@@ -115,8 +115,7 @@ class DockerContainer(resource.Resource):
         ),
         MEMORY: properties.Schema(
             properties.Schema.INTEGER,
-            _('Memory limit (Bytes).'),
-            default=0
+            _('Memory limit (Bytes).')
         ),
         PORT_SPECS: properties.Schema(
             properties.Schema.LIST,
@@ -555,7 +554,8 @@ def available_resource_mapping():
     if DOCKER_INSTALLED:
         return resource_mapping()
     else:
-        LOG.warn(_LW("Docker plug-in loaded, but docker lib not installed."))
+        LOG.warning(_LW("Docker plug-in loaded, but docker lib "
+                        "not installed."))
         return {}
 
 
