@@ -44,10 +44,10 @@ class CloudServer(server.Server):
         status=support.UNSUPPORTED,
         message=_('This resource is not supported, use at your own risk.'))
 
-    # Managed Cloud automation statuses
-    MC_STATUS_IN_PROGRESS = 'In Progress'
-    MC_STATUS_COMPLETE = 'Complete'
-    MC_STATUS_BUILD_ERROR = 'Build Error'
+    # Rackspace Cloud automation statuses
+    SM_STATUS_IN_PROGRESS = 'In Progress'
+    SM_STATUS_COMPLETE = 'Complete'
+    SM_STATUS_BUILD_ERROR = 'Build Error'
 
     # RackConnect automation statuses
     RC_STATUS_DEPLOYING = 'DEPLOYING'
@@ -55,22 +55,59 @@ class CloudServer(server.Server):
     RC_STATUS_FAILED = 'FAILED'
     RC_STATUS_UNPROCESSABLE = 'UNPROCESSABLE'
 
+    # Nova Extra specs
+    FLAVOR_EXTRA_SPECS = 'OS-FLV-WITH-EXT-SPECS:extra_specs'
+    FLAVOR_CLASSES_KEY = 'flavor_classes'
+    FLAVOR_ACCEPT_ANY = '*'
+    FLAVOR_CLASS = 'class'
+    DISK_IO_INDEX = 'disk_io_index'
+    FLAVOR_CLASSES = (
+        GENERAL1, MEMORY1, PERFORMANCE2, PERFORMANCE1, STANDARD1, IO1,
+        ONMETAL, COMPUTE1
+    ) = (
+        'general1', 'memory1', 'performance2', 'performance1',
+        'standard1', 'io1', 'onmetal', 'compute1',
+    )
+
+    # flavor classes that can be booted ONLY from volume
+    BFV_VOLUME_REQUIRED = {MEMORY1, COMPUTE1}
+
+    # flavor classes that can NOT be booted from volume
+    NON_BFV = {STANDARD1, ONMETAL}
+
     properties_schema = copy.deepcopy(server.Server.properties_schema)
     properties_schema.update(
         {
             server.Server.USER_DATA_FORMAT: properties.Schema(
                 properties.Schema.STRING,
-                _('How the user_data should be formatted for the server. For '
-                  'HEAT_CFNTOOLS, the user_data is bundled as part of the '
-                  'heat-cfntools cloud-init boot configuration data. For RAW '
-                  'the user_data is passed to Nova unmodified. '
+                _('How the user_data should be formatted for the server. '
+                  'For RAW the user_data is passed to Nova unmodified. '
                   'For SOFTWARE_CONFIG user_data is bundled as part of the '
                   'software config data, and metadata is derived from any '
                   'associated SoftwareDeployment resources.'),
                 default=server.Server.RAW,
                 constraints=[
-                    constraints.AllowedValues(
-                        server.Server._SOFTWARE_CONFIG_FORMATS),
+                    constraints.AllowedValues([
+                        server.Server.RAW, server.Server.SOFTWARE_CONFIG
+                    ])
+                ]
+            ),
+        }
+    )
+    properties_schema.update(
+        {
+            server.Server.SOFTWARE_CONFIG_TRANSPORT: properties.Schema(
+                properties.Schema.STRING,
+                _('How the server should receive the metadata required for '
+                  'software configuration. POLL_TEMP_URL is the only '
+                  'supported transport on Rackspace Cloud. This property is '
+                  'retained for compatability.'),
+                default=server.Server.POLL_TEMP_URL,
+                update_allowed=True,
+                constraints=[
+                    constraints.AllowedValues([
+                        server.Server.POLL_TEMP_URL
+                    ])
                 ]
             ),
         }
@@ -82,40 +119,42 @@ class CloudServer(server.Server):
         self._rack_connect_started_event_sent = False
 
     def _config_drive(self):
+        user_data_format = self.properties.get(self.USER_DATA_FORMAT, "")
+        is_sw_config = user_data_format == self.SOFTWARE_CONFIG
         user_data = self.properties.get(self.USER_DATA)
         config_drive = self.properties.get(self.CONFIG_DRIVE)
-        if user_data or config_drive:
+        if config_drive or is_sw_config or user_data:
             return True
         else:
             return False
 
-    def _check_managed_cloud_complete(self, server):
+    def _check_rax_automation_complete(self, server):
         if not self._managed_cloud_started_event_sent:
-            msg = _("Waiting for Managed Cloud automation to complete")
+            msg = _("Waiting for Rackspace Cloud automation to complete")
             self._add_event(self.action, self.status, msg)
             self._managed_cloud_started_event_sent = True
 
         if 'rax_service_level_automation' not in server.metadata:
-            LOG.debug("Managed Cloud server does not have the "
+            LOG.debug("Cloud server does not have the "
                       "rax_service_level_automation metadata tag yet")
             return False
 
         mc_status = server.metadata['rax_service_level_automation']
-        LOG.debug("Managed Cloud automation status: %s" % mc_status)
+        LOG.debug("Rackspace Cloud automation status: %s" % mc_status)
 
-        if mc_status == self.MC_STATUS_IN_PROGRESS:
+        if mc_status == self.SM_STATUS_IN_PROGRESS:
             return False
 
-        elif mc_status == self.MC_STATUS_COMPLETE:
-            msg = _("Managed Cloud automation has completed")
+        elif mc_status == self.SM_STATUS_COMPLETE:
+            msg = _("Rackspace Cloud automation has completed")
             self._add_event(self.action, self.status, msg)
             return True
 
-        elif mc_status == self.MC_STATUS_BUILD_ERROR:
-            raise exception.Error(_("Managed Cloud automation failed"))
+        elif mc_status == self.SM_STATUS_BUILD_ERROR:
+            raise exception.Error(_("Rackspace Cloud automation failed"))
 
         else:
-            raise exception.Error(_("Unknown Managed Cloud automation "
+            raise exception.Error(_("Unknown Rackspace Cloud automation "
                                     "status: %s") % mc_status)
 
     def _check_rack_connect_complete(self, server):
@@ -147,7 +186,8 @@ class CloudServer(server.Server):
             reason = server.metadata.get('rackconnect_unprocessable_reason',
                                          None)
             if reason is not None:
-                LOG.warn(_LW("RackConnect unprocessable reason: %s"), reason)
+                LOG.warning(_LW("RackConnect unprocessable reason: %s"),
+                            reason)
 
             msg = _("RackConnect automation has completed")
             self._add_event(self.action, self.status, msg)
@@ -173,8 +213,7 @@ class CloudServer(server.Server):
                 self._check_rack_connect_complete(server)):
             return False
 
-        if ('rax_managed' in self.context.roles and not
-                self._check_managed_cloud_complete(server)):
+        if not self._check_rax_automation_complete(server):
             return False
 
         return True
@@ -200,6 +239,53 @@ class CloudServer(server.Server):
 
         return self._extend_networks(nets)
 
+    def _image_flavor_class_match(self, flavor_type, image_obj):
+        flavor_class_string = image_obj.get(self.FLAVOR_CLASSES_KEY, '')
+        flavor_class_excluded = "!{0}".format(flavor_type)
+        flavor_classes_accepted = flavor_class_string.split(',')
+
+        if flavor_type in flavor_classes_accepted:
+            return True
+
+        if (self.FLAVOR_ACCEPT_ANY in flavor_classes_accepted and
+           flavor_class_excluded not in flavor_classes_accepted):
+            return True
+
+        return False
+
+    def validate(self):
+        """Validate for Rackspace Cloud specific parameters"""
+        super(CloudServer, self).validate()
+
+        # check if image, flavor combination is valid
+        flavor = self.properties[self.FLAVOR]
+        flavor_obj = self.client_plugin().get_flavor(flavor)
+        fl_xtra_specs = flavor_obj.to_dict().get(self.FLAVOR_EXTRA_SPECS, {})
+        flavor_type = fl_xtra_specs.get(self.FLAVOR_CLASS, None)
+
+        image = self.properties.get(self.IMAGE)
+        if not image:
+            if flavor_type in self.NON_BFV:
+                msg = _('Flavor %s cannot be booted from volume.') % flavor
+                raise exception.StackValidationFailed(message=msg)
+            else:
+                # we cannot determine details of the attached volume, so this
+                # is all the validation possible
+                return
+
+        image_obj = self.client_plugin('glance').get_image(image)
+
+        if not self._image_flavor_class_match(flavor_type, image_obj):
+            msg = _('Flavor %(flavor)s cannot be used with image '
+                    '%(image)s.') % {'image': image, 'flavor': flavor}
+            raise exception.StackValidationFailed(message=msg)
+
+        if flavor_type in self.BFV_VOLUME_REQUIRED:
+            msg = _('Flavor %(flavor)s must be booted from volume, '
+                    'but image %(image)s was also specified.') % {
+                'flavor': flavor, 'image': image}
+            raise exception.StackValidationFailed(message=msg)
+
 
 def resource_mapping():
     return {'OS::Nova::Server': CloudServer}
