@@ -14,15 +14,17 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-"""A middleware that turns exceptions into parsable string. Inspired by
-Cinder's faultwrapper
+"""A middleware that turns exceptions into parsable string.
+
+Inspired by Cinder's faultwrapper.
 """
-import six
 
 import sys
 import traceback
 
 from oslo_config import cfg
+from oslo_utils import reflection
+import six
 import webob
 
 from heat.common import exception
@@ -56,15 +58,10 @@ class FaultWrapper(wsgi.Middleware):
         'ActionInProgress': webob.exc.HTTPConflict,
         'ValueError': webob.exc.HTTPBadRequest,
         'EntityNotFound': webob.exc.HTTPNotFound,
-        'StackNotFound': webob.exc.HTTPNotFound,
         'NotFound': webob.exc.HTTPNotFound,
         'ResourceActionNotSupported': webob.exc.HTTPBadRequest,
-        'ResourceNotFound': webob.exc.HTTPNotFound,
-        'ResourceTypeNotFound': webob.exc.HTTPNotFound,
         'InvalidGlobalResource': webob.exc.HTTPInternalServerError,
-        'SnapshotNotFound': webob.exc.HTTPNotFound,
         'ResourceNotAvailable': webob.exc.HTTPNotFound,
-        'PhysicalResourceNotFound': webob.exc.HTTPNotFound,
         'PhysicalResourceNameAmbiguity': webob.exc.HTTPBadRequest,
         'InvalidTenant': webob.exc.HTTPForbidden,
         'Forbidden': webob.exc.HTTPForbidden,
@@ -95,6 +92,7 @@ class FaultWrapper(wsgi.Middleware):
         'UnsupportedObjectError': webob.exc.HTTPBadRequest,
         'ResourceTypeUnavailable': webob.exc.HTTPBadRequest,
         'InvalidBreakPointHook': webob.exc.HTTPBadRequest,
+        'ImmutableParameterModified': webob.exc.HTTPBadRequest
     }
 
     def _map_exception_to_error(self, class_exception):
@@ -119,7 +117,7 @@ class FaultWrapper(wsgi.Middleware):
             ex = ex.exc
             webob_exc = ex
 
-        ex_type = ex.__class__.__name__
+        ex_type = reflection.get_class_name(ex, fully_qualified=False)
 
         is_remote = ex_type.endswith('_Remote')
         if is_remote:
