@@ -37,7 +37,9 @@ summary_keys = [
 
 
 def format_event(req, event, keys=None):
-    include_key = lambda k: k in keys if keys else True
+
+    def include_key(k):
+        return k in keys if keys else True
 
     def transform(key, value):
         if not include_key(key):
@@ -72,9 +74,9 @@ def format_event(req, event, keys=None):
 
 
 class EventController(object):
-    """
-    WSGI controller for Events in Heat v1 API
-    Implements the API actions
+    """WSGI controller for Events in Heat v1 API.
+
+    Implements the API actions.
     """
     # Define request scope (must match what is in policy.json)
     REQUEST_SCOPE = 'events'
@@ -98,20 +100,18 @@ class EventController(object):
 
     @util.identified_stack
     def index(self, req, identity, resource_name=None):
-        """
-        Lists summary information for all events
-        """
+        """Lists summary information for all events."""
         whitelist = {
-            'limit': 'single',
-            'marker': 'single',
-            'sort_dir': 'single',
-            'sort_keys': 'multi',
+            'limit': util.PARAM_TYPE_SINGLE,
+            'marker': util.PARAM_TYPE_SINGLE,
+            'sort_dir': util.PARAM_TYPE_SINGLE,
+            'sort_keys': util.PARAM_TYPE_MULTI,
         }
         filter_whitelist = {
-            'resource_status': 'mixed',
-            'resource_action': 'mixed',
-            'resource_name': 'mixed',
-            'resource_type': 'mixed',
+            'resource_status': util.PARAM_TYPE_MIXED,
+            'resource_action': util.PARAM_TYPE_MIXED,
+            'resource_name': util.PARAM_TYPE_MIXED,
+            'resource_type': util.PARAM_TYPE_MIXED,
         }
         params = util.get_allowed_params(req.params, whitelist)
         filter_params = util.get_allowed_params(req.params, filter_whitelist)
@@ -141,11 +141,9 @@ class EventController(object):
 
     @util.identified_stack
     def show(self, req, identity, resource_name, event_id):
-        """
-        Gets detailed information for an event
-        """
+        """Gets detailed information for an event."""
 
-        filters = {"resource_name": resource_name}
+        filters = {"resource_name": resource_name, "uuid": event_id}
         events = self._event_list(req, identity, filters=filters, detail=True)
         if not events:
             raise exc.HTTPNotFound(_('No event %s found') % event_id)
@@ -154,9 +152,7 @@ class EventController(object):
 
 
 def create_resource(options):
-    """
-    Events resource factory method.
-    """
+    """Events resource factory method."""
     deserializer = wsgi.JSONRequestDeserializer()
     serializer = serializers.JSONResponseSerializer()
     return wsgi.Resource(EventController(options), deserializer, serializer)
