@@ -12,10 +12,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""
-Heat API Server.  This implements an approximation of the Amazon
-CloudFormation API and translates it into a native representation.  It then
-calls the heat-engine via AMQP RPC to implement them.
+"""Heat API Server.
+
+This implements an approximation of the Amazon CloudFormation API and
+translates it into a native representation.  It then calls the heat-engine via
+AMQP RPC to implement them.
 """
 
 import eventlet
@@ -50,6 +51,7 @@ def main():
                  version=version.version_info.version_string())
         logging.setup(cfg.CONF, 'heat-api-cfn')
         logging.set_defaults()
+        config.set_config_defaults()
         messaging.setup()
 
         app = config.load_paste_app()
