@@ -367,9 +367,11 @@ def list_opts():
     yield profiler.list_opts()[0]
     yield 'clients', default_clients_opts
 
-    for client in ('nova', 'swift', 'neutron', 'cinder',
-                   'ceilometer', 'keystone', 'heat', 'glance', 'trove',
-                   'sahara'):
+    for client in ('barbican', 'ceilometer', 'cinder', 'designate', 'glance',
+                   'heat', 'keystone', 'magnum', 'manila', 'mistral',
+                   'neutron', 'nova', 'sahara', 'senlin', 'swift', 'trove',
+                   'zaqar'
+                   ):
         client_specific_group = 'clients_' + client
         yield client_specific_group, clients_opts
 
