@@ -16,7 +16,6 @@ from keystoneclient import auth
 from keystoneclient.auth.identity import access as access_plugin
 from keystoneclient.auth.identity import v3
 from keystoneclient.auth import token_endpoint
-from keystoneclient import discover as ks_discover
 from oslo_config import cfg
 from oslo_context import context
 from oslo_log import log as logging
@@ -25,6 +24,7 @@ from oslo_middleware import request_id as oslo_request_id
 from oslo_utils import importutils
 import six
 
+from heat.common import endpoint_utils
 from heat.common import exception
 from heat.common.i18n import _LE, _LW
 from heat.common import policy
@@ -34,14 +34,33 @@ from heat.engine import clients
 
 LOG = logging.getLogger(__name__)
 
+
+# Note, we yield the options via list_opts to enable generation of the
+# sample heat.conf, but we don't register these options directly via
+# cfg.CONF.register*, it's done via auth.register_conf_options
+# Note, only auth_plugin = v3password is expected to work, example config:
+# [trustee]
+# auth_plugin = password
+# auth_url = http://192.168.1.2:35357
+# username = heat
+# password = password
+# user_domain_id = default
+V3_PASSWORD_PLUGIN = 'v3password'
 TRUSTEE_CONF_GROUP = 'trustee'
 auth.register_conf_options(cfg.CONF, TRUSTEE_CONF_GROUP)
 
 
+def list_opts():
+    trustee_opts = auth.conf.get_common_conf_options()
+    trustee_opts.extend(auth.conf.get_plugin_options(V3_PASSWORD_PLUGIN))
+    yield TRUSTEE_CONF_GROUP, trustee_opts
+
+
 class RequestContext(context.RequestContext):
-    """
-    Stores information about the security context under which the user
-    accesses the system, as well as additional request information.
+    """Stores information about the security context.
+
+    Under the security context the user accesses the system, as well as
+    additional request information.
     """
 
     def __init__(self, auth_token=None, username=None, password=None,
@@ -50,8 +69,10 @@ class RequestContext(context.RequestContext):
                  read_only=False, show_deleted=False,
                  overwrite=True, trust_id=None, trustor_user_id=None,
                  request_id=None, auth_token_info=None, region_name=None,
-                 auth_plugin=None, trusts_auth_plugin=None, **kwargs):
-        """
+                 auth_plugin=None, trusts_auth_plugin=None,
+                 user_domain_id=None, project_domain_id=None, **kwargs):
+        """Initialisation of the request context.
+
         :param overwrite: Set to False to ensure that the greenthread local
             copy of the index is not overwritten.
 
@@ -63,7 +84,9 @@ class RequestContext(context.RequestContext):
                                              is_admin=is_admin,
                                              read_only=read_only,
                                              show_deleted=show_deleted,
-                                             request_id=request_id)
+                                             request_id=request_id,
+                                             user_domain=user_domain_id,
+                                             project_domain=project_domain_id)
 
         self.username = username
         self.user_id = user_id
@@ -120,7 +143,9 @@ class RequestContext(context.RequestContext):
                 'request_id': self.request_id,
                 'show_deleted': self.show_deleted,
                 'region_name': self.region_name,
-                'user_identity': user_idt}
+                'user_identity': user_idt,
+                'user_domain_id': self.user_domain,
+                'project_domain_id': self.project_domain}
 
     @classmethod
     def from_dict(cls, values):
@@ -129,27 +154,16 @@ class RequestContext(context.RequestContext):
     @property
     def keystone_v3_endpoint(self):
         if self.auth_url:
-            auth_uri = self.auth_url.replace('v2.0', 'v3')
+            return self.auth_url.replace('v2.0', 'v3')
         else:
-            # Look for the keystone auth_uri in the configuration. First we
-            # check the [clients_keystone] section, and if it is not set we
-            # look in [keystone_authtoken]
-            if cfg.CONF.clients_keystone.auth_uri:
-                discover = ks_discover.Discover(
-                    auth_url=cfg.CONF.clients_keystone.auth_uri)
-                auth_uri = discover.url_for('3.0')
+            auth_uri = endpoint_utils.get_auth_uri()
+            if auth_uri:
+                return auth_uri
             else:
-                # Import auth_token to have keystone_authtoken settings setup.
-                importutils.import_module('keystonemiddleware.auth_token')
-                if cfg.CONF.keystone_authtoken.auth_uri:
-                    auth_uri = cfg.CONF.keystone_authtoken.auth_uri.replace(
-                        'v2.0', 'v3')
-                else:
-                    LOG.error('Keystone API endpoint not provided. Set '
-                              'auth_uri in section [clients_keystone] '
-                              'of the configuration file.')
-                    raise exception.AuthorizationFailure()
-        return auth_uri
+                LOG.error('Keystone API endpoint not provided. Set '
+                          'auth_uri in section [clients_keystone] '
+                          'of the configuration file.')
+                raise exception.AuthorizationFailure()
 
     @property
     def trusts_auth_plugin(self):
@@ -162,18 +176,22 @@ class RequestContext(context.RequestContext):
         if self._trusts_auth_plugin:
             return self._trusts_auth_plugin
 
-        LOG.warn(_LW('Using the keystone_authtoken user as the heat '
-                     'trustee user directly is deprecated. Please add the '
-                     'trustee credentials you need to the %s section of '
-                     'your heat.conf file.') % TRUSTEE_CONF_GROUP)
+        LOG.warning(_LW('Using the keystone_authtoken user as the heat '
+                        'trustee user directly is deprecated. Please add the '
+                        'trustee credentials you need to the %s section of '
+                        'your heat.conf file.') % TRUSTEE_CONF_GROUP)
 
         cfg.CONF.import_group('keystone_authtoken',
                               'keystonemiddleware.auth_token')
 
+        trustee_user_domain = 'default'
+        if 'user_domain_id' in cfg.CONF.keystone_authtoken:
+            trustee_user_domain = cfg.CONF.keystone_authtoken.user_domain_id
+
         self._trusts_auth_plugin = v3.Password(
             username=cfg.CONF.keystone_authtoken.admin_user,
             password=cfg.CONF.keystone_authtoken.admin_password,
-            user_domain_id='default',
+            user_domain_id=trustee_user_domain,
             auth_url=self.keystone_v3_endpoint,
             trust_id=self.trust_id)
         return self._trusts_auth_plugin
@@ -198,7 +216,7 @@ class RequestContext(context.RequestContext):
             return v3.Password(username=self.username,
                                password=self.password,
                                project_id=self.tenant_id,
-                               user_domain_id='default',
+                               user_domain_id=self.user_domain,
                                auth_url=self.keystone_v3_endpoint)
 
         LOG.error(_LE("Keystone v3 API connection failed, no password "
@@ -234,15 +252,14 @@ class ContextMiddleware(wsgi.Middleware):
         super(ContextMiddleware, self).__init__(app)
 
     def make_context(self, *args, **kwargs):
-        """
-        Create a context with the given arguments.
-        """
+        """Create a context with the given arguments."""
         return self.ctxcls(*args, **kwargs)
 
     def process_request(self, req):
-        """
-        Extract any authentication information in the request and
-        construct an appropriate context from it.
+        """Constructs an appropriate context from extracted auth information.
+
+        Extract any authentication information in the request and construct an
+        appropriate context from it.
         """
         headers = req.headers
         environ = req.environ
@@ -259,11 +276,14 @@ class ContextMiddleware(wsgi.Middleware):
                 aws_creds = headers.get('X-Auth-EC2-Creds')
 
             user_id = headers.get('X-User-Id')
+            user_domain_id = headers.get('X_User_Domain_Id')
             token = headers.get('X-Auth-Token')
             tenant = headers.get('X-Project-Name')
             tenant_id = headers.get('X-Project-Id')
+            project_domain_id = headers.get('X_Project_Domain_Id')
             region_name = headers.get('X-Region-Name')
             auth_url = headers.get('X-Auth-Url')
+
             roles = headers.get('X-Roles')
             if roles is not None:
                 roles = roles.split(',')
@@ -274,24 +294,25 @@ class ContextMiddleware(wsgi.Middleware):
         except Exception:
             raise exception.NotAuthenticated()
 
-        req.context = self.make_context(auth_token=token,
-                                        tenant=tenant, tenant_id=tenant_id,
-                                        aws_creds=aws_creds,
-                                        username=username,
-                                        user_id=user_id,
-                                        password=password,
-                                        auth_url=auth_url,
-                                        roles=roles,
-                                        request_id=req_id,
-                                        auth_token_info=token_info,
-                                        region_name=region_name,
-                                        auth_plugin=auth_plugin)
+        req.context = self.make_context(
+            auth_token=token,
+            tenant=tenant, tenant_id=tenant_id,
+            aws_creds=aws_creds,
+            username=username,
+            user_id=user_id,
+            password=password,
+            auth_url=auth_url,
+            roles=roles,
+            request_id=req_id,
+            auth_token_info=token_info,
+            region_name=region_name,
+            auth_plugin=auth_plugin,
+            user_domain_id=user_domain_id,
+            project_domain_id=project_domain_id)
 
 
 def ContextMiddleware_filter_factory(global_conf, **local_conf):
-    """
-    Factory method for paste.deploy
-    """
+    """Factory method for paste.deploy."""
     conf = global_conf.copy()
     conf.update(local_conf)
 
