@@ -20,7 +20,6 @@ import sys
 
 from oslo_log import log as logging
 import six
-from six.moves.urllib import parse as urlparse
 from six import reraise as raise_
 
 from heat.common.i18n import _
@@ -32,36 +31,41 @@ _FATAL_EXCEPTION_FORMAT_ERRORS = False
 LOG = logging.getLogger(__name__)
 
 
-class RedirectException(Exception):
-    def __init__(self, url):
-        self.url = urlparse.urlparse(url)
-
-
-class KeystoneError(Exception):
-    def __init__(self, code, message):
-        self.code = code
-        self.message = message
-
-    def __str__(self):
-        return "Code: %s, message: %s" % (self.code, self.message)
+# TODO(kanagaraj-manickam): Expose this to user via REST API
+ERROR_CODE_MAP = {
+    '99001': _("Service %(service_name)s is not available for resource "
+               "type %(resource_type)s, reason: %(reason)s")
+}
 
 
 @six.python_2_unicode_compatible
 class HeatException(Exception):
-    """Base Heat Exception
-
-    To correctly use this class, inherit from it and define
-    a 'msg_fmt' property. That msg_fmt will get printf'd
-    with the keyword arguments provided to the constructor.
+    """Base Heat Exception.
 
+    To correctly use this class, inherit from it and define a 'msg_fmt'
+    property. That msg_fmt will get formatted with the keyword arguments
+    provided to the constructor.
     """
     message = _("An unknown exception occurred.")
+    # error_code helps to provide an unique number for a given exception
+    # and is encoded in XXYYY format.
+    # Here, XX - For each of the entity type like stack, resource, etc
+    # an unique number will be provided. All exceptions for a entity will
+    # have same XX code.
+    # YYY - Specific error code for a given exception.
+    error_code = None
 
     def __init__(self, **kwargs):
         self.kwargs = kwargs
 
         try:
+            if self.error_code in ERROR_CODE_MAP:
+                self.msg_fmt = ERROR_CODE_MAP[self.error_code]
+
             self.message = self.msg_fmt % kwargs
+
+            if self.error_code:
+                self.message = 'HEAT-E%s %s' % (self.error_code, self.message)
         except KeyError:
             exc_info = sys.exc_info()
             # kwargs doesn't match a variable in the message
@@ -85,19 +89,6 @@ class MissingCredentialError(HeatException):
     msg_fmt = _("Missing required credential: %(required)s")
 
 
-class BadAuthStrategy(HeatException):
-    msg_fmt = _('Incorrect auth strategy, expected "%(expected)s" but '
-                'received "%(received)s"')
-
-
-class AuthBadRequest(HeatException):
-    msg_fmt = _("Connect error/bad request to Auth service at URL %(url)s.")
-
-
-class AuthUrlNotFound(HeatException):
-    msg_fmt = _("Auth service at URL %(url)s not found.")
-
-
 class AuthorizationFailure(HeatException):
     msg_fmt = _("Authorization failed.")
 
@@ -122,28 +113,6 @@ class Invalid(HeatException):
     msg_fmt = _("Data supplied was not valid: %(reason)s")
 
 
-class AuthorizationRedirect(HeatException):
-    msg_fmt = _("Redirecting to %(uri)s for authorization.")
-
-
-class RequestUriTooLong(HeatException):
-    msg_fmt = _("The URI was too long.")
-
-
-class MaxRedirectsExceeded(HeatException):
-    msg_fmt = _("Maximum redirects (%(redirects)s) was exceeded.")
-
-
-class InvalidRedirect(HeatException):
-    msg_fmt = _("Received invalid HTTP redirect.")
-
-
-class RegionAmbiguity(HeatException):
-    msg_fmt = _("Multiple 'image' service matches for region %(region)s. This "
-                "generally means that a region is required and you have not "
-                "supplied one.")
-
-
 class UserParameterMissing(HeatException):
     msg_fmt = _("The Parameter (%(key)s) was not provided.")
 
@@ -164,6 +133,16 @@ class InvalidTemplateParameter(HeatException):
     msg_fmt = _("The Parameter (%(key)s) has no attributes.")
 
 
+class ImmutableParameterModified(HeatException):
+    msg_fmt = _("The following parameters are immutable and may not be "
+                "updated: %(keys)s")
+
+    def __init__(self, *args, **kwargs):
+        if args:
+            kwargs.update({'keys': ", ".join(args)})
+        super(ImmutableParameterModified, self).__init__(**kwargs)
+
+
 class InvalidTemplateAttribute(HeatException):
     msg_fmt = _("The Referenced Attribute (%(resource)s %(key)s)"
                 " is incorrect.")
@@ -174,20 +153,14 @@ class InvalidTemplateReference(HeatException):
                 ' is incorrect.')
 
 
-class UserKeyPairMissing(HeatException):
-    msg_fmt = _("The Key (%(key_name)s) could not be found.")
-
-
-class FlavorMissing(HeatException):
-    msg_fmt = _("The Flavor ID (%(flavor_id)s) could not be found.")
-
-
 class EntityNotFound(HeatException):
     msg_fmt = _("The %(entity)s (%(name)s) could not be found.")
 
-
-class NovaNetworkNotFound(HeatException):
-    msg_fmt = _("The Nova network (%(network)s) could not be found.")
+    def __init__(self, entity=None, name=None, **kwargs):
+        self.entity = entity
+        self.name = name
+        super(EntityNotFound, self).__init__(entity=entity, name=name,
+                                             **kwargs)
 
 
 class PhysicalResourceNameAmbiguity(HeatException):
@@ -200,10 +173,6 @@ class InvalidTenant(HeatException):
                 "from Tenant %(actual)s forbidden.")
 
 
-class StackNotFound(HeatException):
-    msg_fmt = _("The Stack (%(stack_name)s) could not be found.")
-
-
 class StackExists(HeatException):
     msg_fmt = _("The Stack (%(stack_name)s) already exists.")
 
@@ -254,38 +223,43 @@ class InvalidSchemaError(HeatException):
     msg_fmt = _("%(message)s")
 
 
-class ResourceNotFound(HeatException):
+class ResourceNotFound(EntityNotFound):
     msg_fmt = _("The Resource (%(resource_name)s) could not be found "
                 "in Stack %(stack_name)s.")
 
 
-class SnapshotNotFound(HeatException):
+class SnapshotNotFound(EntityNotFound):
     msg_fmt = _("The Snapshot (%(snapshot)s) for Stack (%(stack)s) "
                 "could not be found.")
 
 
-class ResourceTypeNotFound(HeatException):
-    msg_fmt = _("The Resource Type (%(type_name)s) could not be found.")
-
-
 class InvalidGlobalResource(HeatException):
     msg_fmt = _("There was an error loading the definition of the global "
                 "resource type %(type_name)s.")
 
 
+class ResourceTypeUnavailable(HeatException):
+    error_code = '99001'
+
+
 class InvalidBreakPointHook(HeatException):
     msg_fmt = _("%(message)s")
 
 
+class InvalidRestrictedAction(HeatException):
+    msg_fmt = _("%(message)s")
+
+
 class ResourceNotAvailable(HeatException):
     msg_fmt = _("The Resource (%(resource_name)s) is not available.")
 
 
-class PhysicalResourceNotFound(HeatException):
-    msg_fmt = _("The Resource (%(resource_id)s) could not be found.")
+class ClientNotAvailable(HeatException):
+    msg_fmt = _("The client (%(client_name)s) is not available.")
 
 
-class WatchRuleNotFound(HeatException):
+class WatchRuleNotFound(EntityNotFound):
+    """Keep this for AWS compatiblility."""
     msg_fmt = _("The Watch Rule (%(watch_name)s) could not be found.")
 
 
@@ -356,6 +330,10 @@ class ResourceActionNotSupported(HeatException):
     msg_fmt = _("%(action)s is not supported for resource.")
 
 
+class ResourceActionRestricted(HeatException):
+    msg_fmt = _("%(action)s is restricted for resource.")
+
+
 class ResourcePropertyConflict(HeatException):
     msg_fmt = _('Cannot define the following properties '
                 'at the same time: %(props)s.')
@@ -389,7 +367,7 @@ class PropertyUnspecifiedError(HeatException):
 # correct (and original) location in heat.engine.resource. Reference it as
 # heat.engine.resource.UpdateReplace instead.
 class UpdateReplace(Exception):
-    '''Raised when resource update requires replacement.'''
+    """Raised when resource update requires replacement."""
     def __init__(self, resource_name='Unknown'):
         msg = _("The Resource %s requires replacement.") % resource_name
         super(Exception, self).__init__(six.text_type(msg))
@@ -421,8 +399,9 @@ class UpdateInProgress(Exception):
 
 
 class HTTPExceptionDisguise(Exception):
-    """Disguises HTTP exceptions so they can be handled by the webob fault
-    application in the wsgi pipeline.
+    """Disguises HTTP exceptions.
+
+    They can be handled by the webob fault application in the wsgi pipeline.
     """
 
     def __init__(self, exception):
@@ -475,10 +454,6 @@ class EventSendFailed(HeatException):
                 "on other engine (%(engine_id)s)")
 
 
-class ServiceNotFound(HeatException):
-    msg_fmt = _("Service %(service_id)s not found")
-
-
 class InterfaceAttachFailed(HeatException):
     msg_fmt = _("Failed to attach interface (%(port)s) "
                 "to server (%(server)s)")
@@ -526,10 +501,5 @@ class SIGHUPInterrupt(HeatException):
     msg_fmt = _("System SIGHUP signal received.")
 
 
-class ResourceTypeUnavailable(HeatException):
-    msg_fmt = _("Service %(service_name)s does not have required endpoint in "
-                "service catalog for the resource type %(resource_type)s")
-
-
 class NoActionRequired(Exception):
     pass
