@@ -111,6 +111,10 @@ class KeystoneClientV3(object):
             self._client = self._v3_client_init()
         return self._client
 
+    @property
+    def region_name(self):
+        return self.context.region_name or cfg.CONF.region_name_for_services
+
     @property
     def domain_admin_auth(self):
         if not self._domain_admin_auth:
@@ -140,13 +144,15 @@ class KeystoneClientV3(object):
         if not self._domain_admin_client:
             self._domain_admin_client = kc_v3.Client(
                 session=self.session,
-                auth=self.domain_admin_auth)
+                auth=self.domain_admin_auth,
+                region_name=self.region_name)
 
         return self._domain_admin_client
 
     def _v3_client_init(self):
         client = kc_v3.Client(session=self.session,
-                              auth=self.context.auth_plugin)
+                              auth=self.context.auth_plugin,
+                              region_name=self.region_name)
 
         if hasattr(self.context.auth_plugin, 'get_access'):
             # NOTE(jamielennox): get_access returns the current token without
