@@ -16,13 +16,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""
-Utility methods for working with WSGI servers
-"""
+"""Utility methods for working with WSGI servers."""
 
 import abc
 import errno
-import logging as std_logging
 import os
 import signal
 import sys
@@ -39,6 +36,7 @@ from oslo_config import cfg
 import oslo_i18n as i18n
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
+from oslo_utils import encodeutils
 from oslo_utils import importutils
 from paste import deploy
 import routes
@@ -60,13 +58,13 @@ LOG = logging.getLogger(__name__)
 URL_LENGTH_LIMIT = 50000
 
 api_opts = [
-    cfg.StrOpt('bind_host', default='0.0.0.0',
-               help=_('Address to bind the server. Useful when '
-                      'selecting a particular network interface.'),
-               deprecated_group='DEFAULT'),
-    cfg.IntOpt('bind_port', default=8004, min=1, max=65535,
-               help=_('The port on which the server will listen.'),
-               deprecated_group='DEFAULT'),
+    cfg.IPOpt('bind_host', default='0.0.0.0',
+              help=_('Address to bind the server. Useful when '
+                     'selecting a particular network interface.'),
+              deprecated_group='DEFAULT'),
+    cfg.PortOpt('bind_port', default=8004,
+                help=_('The port on which the server will listen.'),
+                deprecated_group='DEFAULT'),
     cfg.IntOpt('backlog', default=4096,
                help=_("Number of backlog requests "
                       "to configure the socket with."),
@@ -79,8 +77,10 @@ api_opts = [
                help=_("Location of the SSL key file to use "
                       "for enabling SSL mode."),
                deprecated_group='DEFAULT'),
-    cfg.IntOpt('workers', default=processutils.get_worker_count(),
-               help=_("Number of workers for Heat service."),
+    cfg.IntOpt('workers', default=0,
+               help=_("Number of workers for Heat service. "
+                      "Default value 0 means, that service will start number "
+                      "of workers equal number of cores on server."),
                deprecated_group='DEFAULT'),
     cfg.IntOpt('max_header_line', default=16384,
                help=_('Maximum line size of message headers to be accepted. '
@@ -98,13 +98,13 @@ cfg.CONF.register_opts(api_opts,
                        group=api_group)
 
 api_cfn_opts = [
-    cfg.StrOpt('bind_host', default='0.0.0.0',
-               help=_('Address to bind the server. Useful when '
-                      'selecting a particular network interface.'),
-               deprecated_group='DEFAULT'),
-    cfg.IntOpt('bind_port', default=8000, min=1, max=65535,
-               help=_('The port on which the server will listen.'),
-               deprecated_group='DEFAULT'),
+    cfg.IPOpt('bind_host', default='0.0.0.0',
+              help=_('Address to bind the server. Useful when '
+                     'selecting a particular network interface.'),
+              deprecated_group='DEFAULT'),
+    cfg.PortOpt('bind_port', default=8000,
+                help=_('The port on which the server will listen.'),
+                deprecated_group='DEFAULT'),
     cfg.IntOpt('backlog', default=4096,
                help=_("Number of backlog requests "
                       "to configure the socket with."),
@@ -117,7 +117,7 @@ api_cfn_opts = [
                help=_("Location of the SSL key file to use "
                       "for enabling SSL mode."),
                deprecated_group='DEFAULT'),
-    cfg.IntOpt('workers', default=0,
+    cfg.IntOpt('workers', default=1,
                help=_("Number of workers for Heat service."),
                deprecated_group='DEFAULT'),
     cfg.IntOpt('max_header_line', default=16384,
@@ -136,13 +136,13 @@ cfg.CONF.register_opts(api_cfn_opts,
                        group=api_cfn_group)
 
 api_cw_opts = [
-    cfg.StrOpt('bind_host', default='0.0.0.0',
-               help=_('Address to bind the server. Useful when '
-                      'selecting a particular network interface.'),
-               deprecated_group='DEFAULT'),
-    cfg.IntOpt('bind_port', default=8003, min=1, max=65535,
-               help=_('The port on which the server will listen.'),
-               deprecated_group='DEFAULT'),
+    cfg.IPOpt('bind_host', default='0.0.0.0',
+              help=_('Address to bind the server. Useful when '
+                     'selecting a particular network interface.'),
+              deprecated_group='DEFAULT'),
+    cfg.PortOpt('bind_port', default=8003,
+                help=_('The port on which the server will listen.'),
+                deprecated_group='DEFAULT'),
     cfg.IntOpt('backlog', default=4096,
                help=_("Number of backlog requests "
                       "to configure the socket with."),
@@ -155,7 +155,7 @@ api_cw_opts = [
                help=_("Location of the SSL key file to use "
                       "for enabling SSL mode."),
                deprecated_group='DEFAULT'),
-    cfg.IntOpt('workers', default=0,
+    cfg.IntOpt('workers', default=1,
                help=_("Number of workers for Heat service."),
                deprecated_group='DEFAULT'),
     cfg.IntOpt('max_header_line', default=16384,
@@ -209,10 +209,9 @@ def get_bind_addr(conf, default_port=None):
 
 
 def get_socket(conf, default_port):
-    """
-    Bind socket to bind ip:port in conf
+    """Bind socket to bind ip:port in conf.
 
-    note: Mostly comes from Swift with a few small changes...
+    Note: Mostly comes from Swift with a few small changes...
 
     :param conf: a cfg.ConfigOpts object
     :param default_port: port to bind to if none is specified in conf
@@ -256,24 +255,12 @@ def get_socket(conf, default_port):
     return sock
 
 
-class WritableLogger(object):
-    """A thin wrapper that responds to `write` and logs."""
-
-    def __init__(self, LOG, level=std_logging.DEBUG):
-        self.LOG = LOG
-        self.level = level
-
-    def write(self, msg):
-        self.LOG.log(self.level, msg.rstrip("\n"))
-
-
 class Server(object):
     """Server class to manage multiple WSGI sockets and applications."""
 
     def __init__(self, name, conf, threads=1000):
         os.umask(0o27)  # ensure files are created with the correct privileges
         self._logger = logging.getLogger("eventlet.wsgi.server")
-        self._wsgi_logger = WritableLogger(self._logger)
         self.name = name
         self.threads = threads
         self.children = set()
@@ -295,16 +282,13 @@ class Server(object):
         os.killpg(0, signal.SIGTERM)
 
     def hup(self, *args):
-        """
-        Reloads configuration files with zero down time.
-        """
+        """Reloads configuration files with zero down time."""
         LOG.error(_LE('SIGHUP received'))
         signal.signal(signal.SIGHUP, signal.SIG_IGN)
         raise exception.SIGHUPInterrupt
 
     def start(self, application, default_port):
-        """
-        Run a WSGI server with the given application.
+        """Run a WSGI server with the given application.
 
         :param application: The application to run in the WSGI server
         :param default_port: Port to bind to if none is specified in conf
@@ -317,17 +301,28 @@ class Server(object):
         self.start_wsgi()
 
     def start_wsgi(self):
-        if self.conf.workers == 0:
+        workers = self.conf.workers
+        # raise error if workers is incorrect value
+        if workers < 0:
+            raise ValueError("Number of workers should be more or equal '0'!")
+        # childs == num of cores
+        if workers == 0:
+            childs_num = processutils.get_worker_count()
+        # launch only one GreenPool without childs
+        elif workers == 1:
             # Useful for profiling, test, debug etc.
             self.pool = eventlet.GreenPool(size=self.threads)
             self.pool.spawn_n(self._single_run, self.application, self.sock)
             return
+        # childs equal specified value of workers
+        else:
+            childs_num = workers
 
-        LOG.info(_LI("Starting %d workers"), self.conf.workers)
+        LOG.info(_LI("Starting %d workers"), workers)
         signal.signal(signal.SIGTERM, self.kill_children)
         signal.signal(signal.SIGINT, self.kill_children)
         signal.signal(signal.SIGHUP, self.hup)
-        while len(self.children) < self.conf.workers:
+        while len(self.children) < childs_num:
             self.run_child()
 
     def wait_on_children(self):
@@ -352,8 +347,7 @@ class Server(object):
         LOG.debug('Exited')
 
     def configure_socket(self, old_conf=None, has_changed=None):
-        """
-        Ensure a socket exists and is appropriately configured.
+        """Ensure a socket exists and is appropriately configured.
 
         This function is called on start up, and can also be
         called in the event of a configuration reload.
@@ -431,7 +425,7 @@ class Server(object):
             self.stale_children.remove(pid)
             LOG.info(_LI('Removed stale child %s'), pid)
         else:
-            LOG.warn(_LW('Unrecognised child %s'), pid)
+            LOG.warning(_LW('Unrecognised child %s'), pid)
 
     def _verify_and_respawn_children(self, pid, status):
         if len(self.stale_children) == 0:
@@ -448,10 +442,10 @@ class Server(object):
                 self.run_child()
 
     def stash_conf_values(self):
-        """
-        Make a copy of some of the current global CONF's settings.
-        Allows determining if any of these values have changed
-        when the config is reloaded.
+        """Make a copy of some of the current global CONF's settings.
+
+        Allows determining if any of these values have changed when the config
+        is reloaded.
         """
         conf = {}
         conf['bind_host'] = self.conf.bind_host
@@ -462,8 +456,7 @@ class Server(object):
         return conf
 
     def reload(self):
-        """
-        Reload and re-apply configuration settings
+        """Reload and re-apply configuration settings.
 
         Existing child processes are sent a SIGHUP signal
         and will exit after completing existing requests.
@@ -540,7 +533,7 @@ class Server(object):
                 self.application,
                 custom_pool=self.pool,
                 url_length_limit=URL_LENGTH_LIMIT,
-                log=self._wsgi_logger,
+                log=self._logger,
                 debug=cfg.CONF.debug,
                 keepalive=cfg.CONF.eventlet_opts.wsgi_keep_alive,
                 socket_timeout=socket_timeout)
@@ -555,29 +548,27 @@ class Server(object):
         eventlet.wsgi.server(sock, application,
                              custom_pool=self.pool,
                              url_length_limit=URL_LENGTH_LIMIT,
-                             log=self._wsgi_logger,
+                             log=self._logger,
                              debug=cfg.CONF.debug)
 
 
 class Middleware(object):
-    """
-    Base WSGI middleware wrapper. These classes require an application to be
-    initialized that will be called next.  By default the middleware will
-    simply call its wrapped app, or you can override __call__ to customize its
-    behavior.
+    """Base WSGI middleware wrapper.
+
+    These classes require an application to be initialized that will be called
+    next. By default the middleware will simply call its wrapped app, or you
+    can override __call__ to customize its behavior.
     """
 
     def __init__(self, application):
         self.application = application
 
     def process_request(self, req):
-        """
-        Called on each request.
+        """Called on each request.
 
         If this returns None, the next application down the stack will be
         executed. If it returns a response then that response will be returned
         and execution will stop here.
-
         """
         return None
 
@@ -595,7 +586,8 @@ class Middleware(object):
 
 
 class Debug(Middleware):
-    """
+    """Helper class to get information about the request and response.
+
     Helper class that can be inserted into any WSGI application chain
     to get information about the request and response.
     """
@@ -619,10 +611,7 @@ class Debug(Middleware):
 
     @staticmethod
     def print_generator(app_iter):
-        """
-        Iterator that prints the contents of a wrapper string iterator
-        when iterated.
-        """
+        """Prints the contents of a wrapper string iterator when iterated."""
         print(("*" * 40) + " BODY")
         for part in app_iter:
             sys.stdout.write(part)
@@ -636,35 +625,34 @@ def debug_filter(app, conf, **local_conf):
 
 
 class DefaultMethodController(object):
-    """
-    This controller handles the OPTIONS request method and any of the
-    HTTP methods that are not explicitly implemented by the application.
+    """Controller that handles the OPTIONS request method.
+
+    This controller handles the OPTIONS request method and any of the HTTP
+    methods that are not explicitly implemented by the application.
     """
     def options(self, req, allowed_methods, *args, **kwargs):
-        """
+        """Return a response that includes the 'Allow' header.
+
         Return a response that includes the 'Allow' header listing the methods
         that are implemented. A 204 status code is used for this response.
         """
         raise webob.exc.HTTPNoContent(headers=[('Allow', allowed_methods)])
 
     def reject(self, req, allowed_methods, *args, **kwargs):
-        """
-        Return a 405 method not allowed error. As a convenience, the 'Allow'
-        header with the list of implemented methods is included in the
-        response as well.
+        """Return a 405 method not allowed error.
+
+        As a convenience, the 'Allow' header with the list of implemented
+        methods is included in the response as well.
         """
         raise webob.exc.HTTPMethodNotAllowed(
             headers=[('Allow', allowed_methods)])
 
 
 class Router(object):
-    """
-    WSGI middleware that maps incoming requests to WSGI apps.
-    """
+    """WSGI middleware that maps incoming requests to WSGI apps."""
 
     def __init__(self, mapper):
-        """
-        Create a router for the given routes.Mapper.
+        """Create a router for the given routes.Mapper.
 
         Each route in `mapper` must specify a 'controller', which is a
         WSGI app to call.  You'll probably want to specify an 'action' as
@@ -692,8 +680,8 @@ class Router(object):
 
     @webob.dec.wsgify
     def __call__(self, req):
-        """
-        Route the incoming request to a controller based on self.map.
+        """Route the incoming request to a controller based on self.map.
+
         If no match, return a 404.
         """
         return self._router
@@ -701,10 +689,11 @@ class Router(object):
     @staticmethod
     @webob.dec.wsgify
     def _dispatch(req):
-        """
+        """Returns controller after matching the incoming request to a route.
+
         Called by self._router after matching the incoming request to a route
-        and putting the information into req.environ.  Either returns 404
-        or the routed WSGI app's response.
+        and putting the information into req.environ. Either returns 404 or the
+        routed WSGI app's response.
         """
         match = req.environ['wsgiorg.routing_args'][1]
         if not match:
@@ -768,8 +757,7 @@ def is_json_content_type(request):
 
 class JSONRequestDeserializer(object):
     def has_body(self, request):
-        """
-        Returns whether a Webob.Request object will possess an entity body.
+        """Returns whether a Webob.Request object will possess an entity body.
 
         :param request:  Webob.Request object
         """
@@ -799,8 +787,7 @@ class JSONRequestDeserializer(object):
 
 
 class Resource(object):
-    """
-    WSGI app that handles (de)serialization and controller dispatch.
+    """WSGI app that handles (de)serialization and controller dispatch.
 
     Reads routing information supplied by RoutesMiddleware and calls
     the requested action method upon its deserializer, controller,
@@ -816,7 +803,8 @@ class Resource(object):
     serialized by requested content type.
     """
     def __init__(self, controller, deserializer, serializer=None):
-        """
+        """Initialisation of the WSGI app.
+
         :param controller: object that implement methods created by routes lib
         :param deserializer: object that supports webob request deserialization
                              through controller-like actions
@@ -958,7 +946,8 @@ def translate_exception(exc, locale):
     if isinstance(exc, exception.HeatException):
         exc.message = i18n.translate(exc.message, locale)
     else:
-        exc.message = i18n.translate(six.text_type(exc), locale)
+        err_msg = encodeutils.exception_to_unicode(exc)
+        exc.message = i18n.translate(err_msg, locale)
 
     if isinstance(exc, webob.exc.HTTPError):
         exc.explanation = i18n.translate(exc.explanation, locale)
@@ -968,7 +957,6 @@ def translate_exception(exc, locale):
 
 @six.add_metaclass(abc.ABCMeta)
 class BasePasteFactory(object):
-
     """A base class for paste app and filter factories.
 
     Sub-classes must override the KEY class attribute and provide
@@ -1006,7 +994,6 @@ class BasePasteFactory(object):
 
 
 class AppFactory(BasePasteFactory):
-
     """A Generic paste.deploy app factory.
 
     This requires heat.app_factory to be set to a callable which returns a
@@ -1029,7 +1016,6 @@ class AppFactory(BasePasteFactory):
 
 
 class FilterFactory(AppFactory):
-
     """A Generic paste.deploy filter factory.
 
     This requires heat.filter_factory to be set to a callable which returns a
