@@ -19,7 +19,7 @@ Usage:
     >>> db.event_get(context, event_id)
     # Event object received
 
-The underlying driver is loaded . SQLAlchemy is currently the only
+The underlying driver is loaded. SQLAlchemy is currently the only
 supported backend.
 """
 
@@ -113,8 +113,8 @@ def resource_exchange_stacks(context, resource_id1, resource_id2):
     return IMPL.resource_exchange_stacks(context, resource_id1, resource_id2)
 
 
-def resource_get_all_by_stack(context, stack_id, key_id=False):
-    return IMPL.resource_get_all_by_stack(context, stack_id, key_id)
+def resource_get_all_by_stack(context, stack_id, key_id=False, filters=None):
+    return IMPL.resource_get_all_by_stack(context, stack_id, key_id, filters)
 
 
 def resource_get_by_name_and_stack(context, resource_name, stack_id):
@@ -134,6 +134,10 @@ def stack_get(context, stack_id, show_deleted=False, tenant_safe=True,
                           eager_load=eager_load)
 
 
+def stack_get_status(context, stack_id):
+    return IMPL.stack_get_status(context, stack_id)
+
+
 def stack_get_by_name_and_owner_id(context, stack_name, owner_id):
     return IMPL.stack_get_by_name_and_owner_id(context, stack_name,
                                                owner_id=owner_id)
