@@ -16,13 +16,16 @@ import datetime
 import sys
 
 from oslo_config import cfg
+from oslo_db import api as oslo_db_api
 from oslo_db.sqlalchemy import session as db_session
 from oslo_db.sqlalchemy import utils
 from oslo_serialization import jsonutils
+from oslo_utils import encodeutils
 from oslo_utils import timeutils
 import osprofiler.sqlalchemy
 import six
 import sqlalchemy
+from sqlalchemy import func
 from sqlalchemy import orm
 from sqlalchemy.orm import aliased as orm_aliased
 from sqlalchemy.orm import session as orm_session
@@ -33,6 +36,7 @@ from heat.common.i18n import _
 from heat.db.sqlalchemy import filters as db_filters
 from heat.db.sqlalchemy import migration
 from heat.db.sqlalchemy import models
+from heat.db.sqlalchemy import utils as db_utils
 from heat.rpc import api as rpc_api
 
 CONF = cfg.CONF
@@ -48,7 +52,7 @@ def get_facade():
 
     if not _facade:
         _facade = db_session.EngineFacade.from_config(CONF)
-        if CONF.profiler.profiler_enabled:
+        if CONF.profiler.enabled:
             if CONF.profiler.trace_sqlalchemy:
                 osprofiler.sqlalchemy.add_tracing(sqlalchemy,
                                                   _facade.get_engine(),
@@ -56,8 +60,13 @@ def get_facade():
 
     return _facade
 
-get_engine = lambda: get_facade().get_engine()
-get_session = lambda: get_facade().get_session()
+
+def get_engine():
+    return get_facade().get_engine()
+
+
+def get_session():
+    return get_facade().get_session()
 
 
 def get_backend():
@@ -213,7 +222,7 @@ def resource_data_get(resource, key):
 
 
 def stack_tags_set(context, stack_id, tags):
-    session = get_session()
+    session = _session(context)
     with session.begin():
         stack_tags_delete(context, stack_id)
         result = []
@@ -227,8 +236,8 @@ def stack_tags_set(context, stack_id, tags):
 
 
 def stack_tags_delete(context, stack_id):
-    session = get_session()
-    with session.begin():
+    session = _session(context)
+    with session.begin(subtransactions=True):
         result = stack_tags_get(context, stack_id)
         if result:
             for tag in result:
@@ -300,12 +309,15 @@ def resource_create(context, values):
     return resource_ref
 
 
-def resource_get_all_by_stack(context, stack_id, key_id=False):
-    results = model_query(
+def resource_get_all_by_stack(context, stack_id, key_id=False, filters=None):
+    query = model_query(
         context, models.Resource
     ).filter_by(
         stack_id=stack_id
-    ).options(orm.joinedload("data")).all()
+    ).options(orm.joinedload("data"))
+
+    query = db_filters.exact_filter(query, models.Resource, filters)
+    results = query.all()
 
     if not results:
         raise exception.NotFound(_("no resources for stack_id %s were found")
@@ -356,6 +368,18 @@ def stack_get(context, stack_id, show_deleted=False, tenant_safe=True,
     return result
 
 
+def stack_get_status(context, stack_id):
+    query = model_query(context, models.Stack)
+    query = query.options(
+        orm.load_only("action", "status", "status_reason", "updated_at"))
+    result = query.filter_by(id=stack_id).first()
+    if result is None:
+        raise exception.NotFound(_('Stack with id %s not found') % stack_id)
+
+    return (result.action, result.status, result.status_reason,
+            result.updated_at)
+
+
 def stack_get_all_by_owner_id(context, owner_id):
     results = soft_delete_aware_query(
         context, models.Stack).filter_by(owner_id=owner_id).all()
@@ -393,7 +417,8 @@ def _paginate_query(context, query, model, limit=None, sort_keys=None,
         query = utils.paginate_query(query, model, limit, sort_keys,
                                      model_marker, sort_dir)
     except utils.InvalidSortKey as exc:
-        raise exception.Invalid(reason=exc.message)
+        err_msg = encodeutils.exception_to_unicode(exc)
+        raise exception.Invalid(reason=err_msg)
     return query
 
 
@@ -514,13 +539,14 @@ def stack_update(context, stack_id, values, exp_trvsl=None):
         return False
 
     session = _session(context)
-    rows_updated = (session.query(models.Stack)
-                    .filter(models.Stack.id == stack.id)
-                    .filter(models.Stack.current_traversal
-                            == stack.current_traversal)
-                    .update(values, synchronize_session=False))
-    session.expire_all()
 
+    with session.begin():
+        rows_updated = (session.query(models.Stack)
+                        .filter(models.Stack.id == stack.id)
+                        .filter(models.Stack.current_traversal
+                                == stack.current_traversal)
+                        .update(values, synchronize_session=False))
+    session.expire_all()
     return (rows_updated is not None and rows_updated > 0)
 
 
@@ -532,14 +558,14 @@ def stack_delete(context, stack_id):
                                      'id': stack_id,
                                      'msg': 'that does not exist'})
     session = orm_session.Session.object_session(s)
-
-    for r in s.resources:
-        session.delete(r)
-
-    s.soft_delete(session=session)
-    session.flush()
+    with session.begin():
+        for r in s.resources:
+            session.delete(r)
+        s.soft_delete(session=session)
 
 
+@oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
+                           retry_interval=0.5, inc_retry_interval=True)
 def stack_lock_create(stack_id, engine_id):
     session = get_session()
     with session.begin():
@@ -597,6 +623,8 @@ def stack_lock_release(stack_id, engine_id):
 
 def stack_get_root_id(context, stack_id):
     s = stack_get(context, stack_id)
+    if not s:
+        return None
     while s.owner_id:
         s = stack_get(context, s.owner_id)
     return s.id
@@ -658,6 +686,7 @@ def user_creds_get(user_creds_id):
     return result
 
 
+@db_utils.retry_on_stale_data_error
 def user_creds_delete(context, user_creds_id):
     creds = model_query(context, models.UserCreds).get(user_creds_id)
     if not creds:
@@ -665,8 +694,8 @@ def user_creds_delete(context, user_creds_id):
             _('Attempt to delete user creds with id '
               '%(id)s that does not exist') % {'id': user_creds_id})
     session = orm_session.Session.object_session(creds)
-    session.delete(creds)
-    session.flush()
+    with session.begin():
+        session.delete(creds)
 
 
 def event_get(context, event_id):
@@ -729,7 +758,8 @@ def _events_paginate_query(context, query, model, limit=None, sort_keys=None,
         query = utils.paginate_query(query, model, limit, sort_keys,
                                      model_marker, sort_dir)
     except utils.InvalidSortKey as exc:
-        raise exception.Invalid(reason=exc.message)
+        err_msg = encodeutils.exception_to_unicode(exc)
+        raise exception.Invalid(reason=err_msg)
 
     return query
 
@@ -752,7 +782,8 @@ def _events_filter_and_page_query(context, query,
 
 
 def event_count_all_by_stack(context, stack_id):
-    return _query_all_by_stack(context, stack_id).count()
+    query = model_query(context, func.count(models.Event.id))
+    return query.filter_by(stack_id=stack_id).scalar()
 
 
 def _delete_event_rows(context, stack_id, limit):
@@ -832,12 +863,10 @@ def watch_rule_delete(context, watch_id):
                                      'id': watch_id,
                                      'msg': 'that does not exist'})
     session = orm_session.Session.object_session(wr)
-
-    for d in wr.watch_data:
-        session.delete(d)
-
-    session.delete(wr)
-    session.flush()
+    with session.begin():
+        for d in wr.watch_data:
+            session.delete(d)
+        session.delete(wr)
 
 
 def watch_data_create(context, values):
@@ -889,8 +918,8 @@ def software_config_get_all(context, limit=None, marker=None,
 def software_config_delete(context, config_id):
     config = software_config_get(context, config_id)
     session = orm_session.Session.object_session(config)
-    session.delete(config)
-    session.flush()
+    with session.begin():
+        session.delete(config)
 
 
 def software_deployment_create(context, values):
@@ -978,8 +1007,8 @@ def snapshot_update(context, snapshot_id, values):
 def snapshot_delete(context, snapshot_id):
     snapshot = snapshot_get(context, snapshot_id)
     session = orm_session.Session.object_session(snapshot)
-    session.delete(snapshot)
-    session.flush()
+    with session.begin():
+        session.delete(snapshot)
 
 
 def snapshot_get_all(context, stack_id):
@@ -1005,17 +1034,17 @@ def service_update(context, service_id, values):
 def service_delete(context, service_id, soft_delete=True):
     service = service_get(context, service_id)
     session = orm_session.Session.object_session(service)
-    if soft_delete:
-        service.soft_delete(session=session)
-    else:
-        session.delete(service)
-    session.flush()
+    with session.begin():
+        if soft_delete:
+            service.soft_delete(session=session)
+        else:
+            session.delete(service)
 
 
 def service_get(context, service_id):
     result = model_query(context, models.Service).get(service_id)
     if result is None:
-        raise exception.ServiceNotFound(service_id=service_id)
+        raise exception.EntityNotFound(entity='Service', name=service_id)
     return result
 
 
@@ -1050,13 +1079,14 @@ def purge_deleted(age, granularity='days'):
     elif granularity == 'minutes':
         age = age * 60
 
-    time_line = datetime.datetime.now() - datetime.timedelta(seconds=age)
+    time_line = timeutils.utcnow() - datetime.timedelta(seconds=age)
     engine = get_engine()
     meta = sqlalchemy.MetaData()
     meta.bind = engine
 
     stack = sqlalchemy.Table('stack', meta, autoload=True)
     stack_lock = sqlalchemy.Table('stack_lock', meta, autoload=True)
+    stack_tag = sqlalchemy.Table('stack_tag', meta, autoload=True)
     resource = sqlalchemy.Table('resource', meta, autoload=True)
     resource_data = sqlalchemy.Table('resource_data', meta, autoload=True)
     event = sqlalchemy.Table('event', meta, autoload=True)
@@ -1066,47 +1096,68 @@ def purge_deleted(age, granularity='days'):
     syncpoint = sqlalchemy.Table('sync_point', meta, autoload=True)
 
     # find the soft-deleted stacks that are past their expiry
-    stack_where = sqlalchemy.select([stack.c.id]).where(
-        stack.c.deleted_at < time_line)
-    # delete stack locks (just in case some got stuck)
-    stack_lock_del = stack_lock.delete().where(
-        stack_lock.c.stack_id.in_(stack_where))
-    engine.execute(stack_lock_del)
-    # delete resource_data
-    res_where = sqlalchemy.select([resource.c.id]).where(
-        resource.c.stack_id.in_(stack_where))
-    res_data_del = resource_data.delete().where(
-        resource_data.c.resource_id.in_(res_where))
-    engine.execute(res_data_del)
-    # delete resources
-    res_del = resource.delete().where(resource.c.stack_id.in_(stack_where))
-    engine.execute(res_del)
-    # delete events
-    event_del = event.delete().where(event.c.stack_id.in_(stack_where))
-    engine.execute(event_del)
-    # clean up any sync_points that may have lingered
-    sync_del = syncpoint.delete().where(syncpoint.c.stack_id.in_(stack_where))
-    engine.execute(sync_del)
-    # delete the stacks
-    stack_del = stack.delete().where(stack.c.deleted_at < time_line)
-    engine.execute(stack_del)
-    # delete orphaned raw templates
-    raw_templ_sel = raw_template.c.id.in_(
-        sqlalchemy.select([raw_template.c.id]).select_from(
-            sqlalchemy.join(
-                raw_template,
-                stack,
-                sqlalchemy.or_(
-                    stack.c.prev_raw_template_id == raw_template.c.id,
-                    stack.c.raw_template_id == raw_template.c.id),
-                isouter=True)).where(stack.c.id == None))  # noqa
-    raw_templ_del = raw_template.delete().where(raw_templ_sel)
-    engine.execute(raw_templ_del)
-    # purge any user creds that are no longer referenced
-    stack_creds_sel = sqlalchemy.select([stack.c.user_creds_id])
-    user_creds_sel = sqlalchemy.not_(user_creds.c.id.in_(stack_creds_sel))
-    usr_creds_del = user_creds.delete().where(user_creds_sel)
-    engine.execute(usr_creds_del)
+    stack_where = sqlalchemy.select([stack.c.id, stack.c.raw_template_id,
+                                     stack.c.prev_raw_template_id,
+                                     stack.c.user_creds_id]).where(
+                                         stack.c.deleted_at < time_line)
+    stacks = list(engine.execute(stack_where))
+    if stacks:
+        stack_ids = [i[0] for i in stacks]
+        # delete stack locks (just in case some got stuck)
+        stack_lock_del = stack_lock.delete().where(
+            stack_lock.c.stack_id.in_(stack_ids))
+        engine.execute(stack_lock_del)
+        # delete stack tags
+        stack_tag_del = stack_tag.delete().where(
+            stack_tag.c.stack_id.in_(stack_ids))
+        engine.execute(stack_tag_del)
+        # delete resource_data
+        res_where = sqlalchemy.select([resource.c.id]).where(
+            resource.c.stack_id.in_(stack_ids))
+        res_data_del = resource_data.delete().where(
+            resource_data.c.resource_id.in_(res_where))
+        engine.execute(res_data_del)
+        # delete resources
+        res_del = resource.delete().where(resource.c.stack_id.in_(stack_ids))
+        engine.execute(res_del)
+        # delete events
+        event_del = event.delete().where(event.c.stack_id.in_(stack_ids))
+        engine.execute(event_del)
+        # clean up any sync_points that may have lingered
+        sync_del = syncpoint.delete().where(
+            syncpoint.c.stack_id.in_(stack_ids))
+        engine.execute(sync_del)
+        # delete the stacks
+        stack_del = stack.delete().where(stack.c.id.in_(stack_ids))
+        engine.execute(stack_del)
+        # delete orphaned raw templates
+        raw_template_ids = [i[1] for i in stacks if i[1] is not None]
+        raw_template_ids.extend(i[2] for i in stacks if i[2] is not None)
+        if raw_template_ids:
+            # keep those still referenced
+            raw_tmpl_sel = sqlalchemy.select([stack.c.raw_template_id]).where(
+                stack.c.raw_template_id.in_(raw_template_ids))
+            raw_tmpl = [i[0] for i in engine.execute(raw_tmpl_sel)]
+            raw_template_ids = set(raw_template_ids) - set(raw_tmpl)
+            raw_tmpl_sel = sqlalchemy.select(
+                [stack.c.prev_raw_template_id]).where(
+                stack.c.prev_raw_template_id.in_(raw_template_ids))
+            raw_tmpl = [i[0] for i in engine.execute(raw_tmpl_sel)]
+            raw_template_ids = raw_template_ids - set(raw_tmpl)
+            raw_templ_del = raw_template.delete().where(
+                raw_template.c.id.in_(raw_template_ids))
+            engine.execute(raw_templ_del)
+        # purge any user creds that are no longer referenced
+        user_creds_ids = [i[3] for i in stacks if i[3] is not None]
+        if user_creds_ids:
+            # keep those still referenced
+            user_sel = sqlalchemy.select([stack.c.user_creds_id]).where(
+                stack.c.user_creds_id.in_(user_creds_ids))
+            users = [i[0] for i in engine.execute(user_sel)]
+            user_creds_ids = set(user_creds_ids) - set(users)
+            usr_creds_del = user_creds.delete().where(
+                user_creds.c.id.in_(user_creds_ids))
+            engine.execute(usr_creds_del)
     # Purge deleted services
     srvc_del = service.delete().where(service.c.deleted_at < time_line)
     engine.execute(srvc_del)
@@ -1119,6 +1170,8 @@ def sync_point_delete_all_by_stack_and_traversal(context, stack_id,
     return rows_deleted
 
 
+@oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
+                           retry_interval=0.5, inc_retry_interval=True)
 def sync_point_create(context, values):
     values['entity_id'] = str(values['entity_id'])
     sync_point_ref = models.SyncPoint()
@@ -1178,21 +1231,17 @@ def db_encrypt_parameters_and_properties(ctxt, encryption_key, batch_size=50):
                 session=session, ctxt=ctxt, query=query,
                 model=models.RawTemplate, batch_size=batch_size):
             tmpl = template.Template.load(ctxt, raw_template.id, raw_template)
+            param_schemata = tmpl.param_schemata()
             env = raw_template.environment
 
             if 'encrypted_param_names' in env:
                 encrypted_params = env['encrypted_param_names']
             else:
                 encrypted_params = []
-            for param_name, param in tmpl.param_schemata().items():
-                if (param_name in encrypted_params) or (not param.hidden):
-                    continue
-
-                try:
-                    param_val = env['parameters'][param_name]
-                except KeyError:
-                    param_val = param.default
-
+            for param_name, param_val in env['parameters'].items():
+                if ((param_name in encrypted_params) or
+                   (not param_schemata[param_name].hidden)):
+                        continue
                 encrypted_val = crypt.encrypt(param_val, encryption_key)
                 env['parameters'][param_name] = encrypted_val
                 encrypted_params.append(param_name)
