@@ -60,7 +60,7 @@ class HeatBase(models.ModelBase, models.TimestampMixin):
             session = orm_session.Session.object_session(self)
             if not session:
                 session = get_session()
-        session.begin()
+        session.begin(subtransactions=True)
         session.delete(self)
         session.commit()
 
@@ -69,7 +69,7 @@ class HeatBase(models.ModelBase, models.TimestampMixin):
             session = orm_session.Session.object_session(self)
             if not session:
                 session = get_session()
-        session.begin()
+        session.begin(subtransactions=True)
         for k, v in six.iteritems(values):
             setattr(self, k, v)
         session.commit()
@@ -117,7 +117,7 @@ class StackTag(BASE, HeatBase):
 
 
 class SyncPoint(BASE, HeatBase):
-    """Represents an syncpoint for an stack that is being worked on."""
+    """Represents a syncpoint for a stack that is being worked on."""
 
     __tablename__ = 'sync_point'
     __table_args__ = (
@@ -166,7 +166,7 @@ class Stack(BASE, HeatBase, SoftDelete, StateAware):
     user_creds_id = sqlalchemy.Column(
         sqlalchemy.Integer,
         sqlalchemy.ForeignKey('user_creds.id'))
-    owner_id = sqlalchemy.Column(sqlalchemy.String(36))
+    owner_id = sqlalchemy.Column(sqlalchemy.String(36), index=True)
     parent_resource_name = sqlalchemy.Column(sqlalchemy.String(255))
     timeout = sqlalchemy.Column(sqlalchemy.Integer)
     disable_rollback = sqlalchemy.Column(sqlalchemy.Boolean, nullable=False)
@@ -215,7 +215,8 @@ class UserCreds(BASE, HeatBase):
     tenant_id = sqlalchemy.Column(sqlalchemy.String(256))
     trust_id = sqlalchemy.Column(sqlalchemy.String(255))
     trustor_user_id = sqlalchemy.Column(sqlalchemy.String(64))
-    stack = relationship(Stack, backref=backref('user_creds'))
+    stack = relationship(Stack, backref=backref('user_creds'),
+                         cascade_backrefs=False)
 
 
 class Event(BASE, HeatBase):
