@@ -12,6 +12,7 @@
 #    under the License.
 
 import abc
+import functools
 import sys
 import weakref
 
@@ -21,11 +22,71 @@ from keystoneclient.auth.identity import v3
 from keystoneclient import exceptions
 from keystoneclient import session
 from oslo_config import cfg
+import requests
 import six
 
 from heat.common import config
 from heat.common.i18n import _
 
+cfg.CONF.import_opt('client_retry_limit', 'heat.common.config')
+
+
+class ExceptionFilter(object):
+    """A context manager that prevents some exceptions from being raised.
+
+    For backwards compatibility, these objects can also be called with the
+    exception value as an argument - any non-matching exception will be
+    re-raised from this call. We attempt but cannot guarantee to keep the same
+    traceback; the context manager method is preferred for this reason except
+    in cases where the ignored exception affects control flow.
+
+    Use this class as a decorator for a function that returns whether a given
+    exception should be ignored. e.g.
+
+    >>> @ExceptionFilter
+    >>> def ignore_assertions(ex):
+    ...     return isinstance(ex, AssertionError)
+
+    and then use it as a context manager:
+
+    >>> with ignore_assertions:
+    ...     assert False
+
+    or call it:
+
+    >>> try:
+    ...     assert False
+    ... except Exception as ex:
+    ...     ignore_assertions(ex)
+    """
+
+    def __init__(self, should_ignore_ex):
+        self._should_ignore_ex = should_ignore_ex
+        functools.update_wrapper(self, should_ignore_ex)
+
+    def __get__(self, obj, owner):
+        return type(self)(six.create_bound_method(self._should_ignore_ex, obj))
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        if exc_val is not None:
+            return self._should_ignore_ex(exc_val)
+
+    def __call__(self, ex):
+        """Re-raise any exception value not being filtered out.
+
+        If the exception was the last to be raised, it will be re-raised with
+        its original traceback.
+        """
+        if not self._should_ignore_ex(ex):
+            exc_type, exc_val, traceback = sys.exc_info()
+            if exc_val is ex:
+                six.reraise(exc_type, exc_val, traceback)
+            else:
+                raise ex
+
 
 @six.add_metaclass(abc.ABCMeta)
 class ClientPlugin(object):
@@ -116,7 +177,7 @@ class ClientPlugin(object):
 
         reg = self.context.region_name or cfg.CONF.region_name_for_services
         kwargs.setdefault('region_name', reg)
-
+        url = None
         try:
             url = get_endpoint()
         except exceptions.EmptyCatalog:
@@ -179,19 +240,15 @@ class ClientPlugin(object):
         """Returns True if the exception is a conflict."""
         return False
 
+    @ExceptionFilter
     def ignore_not_found(self, ex):
         """Raises the exception unless it is a not-found."""
-        if not self.is_not_found(ex):
-            exc_info = sys.exc_info()
-            six.reraise(*exc_info)
+        return self.is_not_found(ex)
 
+    @ExceptionFilter
     def ignore_conflict_and_not_found(self, ex):
         """Raises the exception unless it is a conflict or not-found."""
-        if self.is_conflict(ex) or self.is_not_found(ex):
-            return
-        else:
-            exc_info = sys.exc_info()
-            six.reraise(*exc_info)
+        return self.is_conflict(ex) or self.is_not_found(ex)
 
     def _get_client_args(self,
                          service_name,
@@ -230,5 +287,9 @@ class ClientPlugin(object):
             return False
 
 
+def retry_if_connection_err(exception):
+    return isinstance(exception, requests.ConnectionError)
+
+
 def retry_if_result_is_false(result):
     return result is False
