@@ -11,10 +11,19 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from magnumclient.openstack.common.apiclient import exceptions
 from magnumclient.v1 import client as magnum_client
 
+from heat.common import exception
 from heat.engine.clients import client_plugin
+from heat.engine import constraints
+
+try:
+    from magnumclient.common.apiclient import exceptions as mc_exc
+except ImportError:
+    # NOTE (duvarenkov): For magnumclient versions before 2.0.0.
+    from magnumclient.openstack.common.apiclient import exceptions as mc_exc
+
+CLIENT_NAME = 'magnum'
 
 
 class MagnumClientPlugin(client_plugin.ClientPlugin):
@@ -22,7 +31,7 @@ class MagnumClientPlugin(client_plugin.ClientPlugin):
     service_types = [CONTAINER] = ['container']
 
     def _create(self):
-        endpoint_type = self._get_client_option('magnum', 'endpoint_type')
+        endpoint_type = self._get_client_option(CLIENT_NAME, 'endpoint_type')
         endpoint = self.url_for(service_type=self.CONTAINER,
                                 endpoint_type=endpoint_type)
 
@@ -34,10 +43,23 @@ class MagnumClientPlugin(client_plugin.ClientPlugin):
         return client
 
     def is_not_found(self, ex):
-        return isinstance(ex, exceptions.NotFound)
+        return isinstance(ex, mc_exc.NotFound)
 
     def is_over_limit(self, ex):
-        return isinstance(ex, exceptions.RequestEntityTooLarge)
+        return isinstance(ex, mc_exc.RequestEntityTooLarge)
 
     def is_conflict(self, ex):
-        return isinstance(ex, exceptions.Conflict)
+        return isinstance(ex, mc_exc.Conflict)
+
+    def get_baymodel(self, value):
+        try:
+            self.client().baymodels.get(value)
+        except mc_exc.NotFound:
+            raise exception.EntityNotFound(entity='BayModel',
+                                           name=value)
+
+
+class BaymodelConstraint(constraints.BaseCustomConstraint):
+
+    resource_client_name = CLIENT_NAME
+    resource_getter_name = 'get_baymodel'
