@@ -311,7 +311,15 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
                 subtype = os.path.splitext(filename)[0]
             if content is None:
                 content = ''
-            msg = text.MIMEText(content, _subtype=subtype)
+
+            try:
+                content.encode('us-ascii')
+                charset = 'us-ascii'
+            except UnicodeEncodeError:
+                charset = 'utf-8'
+            msg = (text.MIMEText(content, _subtype=subtype, _charset=charset)
+                   if subtype else text.MIMEText(content, _charset=charset))
+
             msg.add_header('Content-Disposition', 'attachment',
                            filename=filename)
             return msg
@@ -496,12 +504,13 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
                 result=msg, resource_status=status)
 
     def rebuild(self, server_id, image_id, password=None,
-                preserve_ephemeral=False):
+                preserve_ephemeral=False, meta=None, files=None):
         """Rebuild the server and call check_rebuild to verify."""
         server = self.fetch_server(server_id)
         if server:
             server.rebuild(image_id, password=password,
-                           preserve_ephemeral=preserve_ephemeral)
+                           preserve_ephemeral=preserve_ephemeral,
+                           meta=meta, files=files)
             return True
         else:
             return False
