@@ -19,6 +19,8 @@ from heat.common.i18n import _
 from heat.engine.clients import client_plugin
 from heat.engine import constraints
 
+CLIENT_NAME = 'trove'
+
 
 class TroveClientPlugin(client_plugin.ClientPlugin):
 
@@ -29,22 +31,22 @@ class TroveClientPlugin(client_plugin.ClientPlugin):
     def _create(self):
 
         con = self.context
-        endpoint_type = self._get_client_option('trove', 'endpoint_type')
+        endpoint_type = self._get_client_option(CLIENT_NAME, 'endpoint_type')
         args = {
             'service_type': self.DATABASE,
             'auth_url': con.auth_url or '',
             'proxy_token': con.auth_token,
             'username': None,
             'password': None,
-            'cacert': self._get_client_option('trove', 'ca_file'),
-            'insecure': self._get_client_option('trove', 'insecure'),
+            'cacert': self._get_client_option(CLIENT_NAME, 'ca_file'),
+            'insecure': self._get_client_option(CLIENT_NAME, 'insecure'),
             'endpoint_type': endpoint_type
         }
 
         client = tc.Client('1.0', **args)
         management_url = self.url_for(service_type=self.DATABASE,
                                       endpoint_type=endpoint_type)
-        client.client.auth_token = con.auth_token
+        client.client.auth_token = self.auth_token
         client.client.management_url = management_url
 
         return client
@@ -65,15 +67,6 @@ class TroveClientPlugin(client_plugin.ClientPlugin):
                                 'dsversion': datastore_version,
                                 'allowed': ', '.join(allowed_version_names)}
                     raise exception.StackValidationFailed(message=msg)
-            else:
-                if len(allowed_versions) > 1:
-                    msg = _("Multiple active datastore versions exist for "
-                            "datastore type %(dstype)s. "
-                            "Explicit datastore version must be provided. "
-                            "Allowed versions are %(allowed)s.") % {
-                                'dstype': datastore_type,
-                                'allowed': ', '.join(allowed_version_names)}
-                    raise exception.StackValidationFailed(message=msg)
         else:
             if datastore_version:
                 msg = _("Not allowed - %(dsver)s without %(dstype)s.") % {
@@ -90,32 +83,21 @@ class TroveClientPlugin(client_plugin.ClientPlugin):
     def is_conflict(self, ex):
         return isinstance(ex, exceptions.Conflict)
 
-    def get_flavor_id(self, flavor):
-        """Get the ID for the specified flavor name.
-
-        If the specified value is flavor id, just return it.
+    def find_flavor_by_name_or_id(self, flavor):
+        """Find the specified flavor by name or id.
 
         :param flavor: the name of the flavor to find
         :returns: the id of :flavor:
-        :raises: exception.FlavorMissing
         """
-        flavor_id = None
-        flavor_list = self.client().flavors.list()
-        for o in flavor_list:
-            if o.name == flavor:
-                flavor_id = o.id
-                break
-            if o.id == flavor:
-                flavor_id = o.id
-                break
-        if flavor_id is None:
-            raise exception.FlavorMissing(flavor_id=flavor)
-        return flavor_id
+        try:
+            return self.client().flavors.get(flavor).id
+        except exceptions.NotFound:
+            return self.client().flavors.find(name=flavor).id
 
 
 class FlavorConstraint(constraints.BaseCustomConstraint):
 
-    expected_exceptions = (exception.FlavorMissing,)
+    expected_exceptions = (exceptions.NotFound,)
 
-    def validate_with_client(self, client, flavor):
-        client.client_plugin('trove').get_flavor_id(flavor)
+    resource_client_name = CLIENT_NAME
+    resource_getter_name = 'find_flavor_by_name_or_id'
