@@ -18,12 +18,14 @@ import six
 
 from heat.common import exception
 from heat.common.i18n import _
+from heat.common.i18n import repr_wraper
 
 
 class CircularDependencyException(exception.HeatException):
     msg_fmt = _("Circular Dependency Found: %(cycle)s")
 
 
+@repr_wraper
 @six.python_2_unicode_compatible
 class Node(object):
     """A node in a dependency graph."""
@@ -88,7 +90,7 @@ class Node(object):
 
     def __str__(self):
         """Return a human-readable string representation of the node."""
-        text = '{%s}' % ', '.join(str(n) for n in self)
+        text = '{%s}' % ', '.join(six.text_type(n) for n in self)
         return six.text_type(text)
 
     def __repr__(self):
@@ -104,7 +106,7 @@ class Graph(collections.defaultdict):
         super(Graph, self).__init__(Node, *args)
 
     def map(self, func):
-        """A dict mapping the supplied function onto each node in the graph.
+        """Map the supplied function onto each node in the graph.
 
         Return a dictionary derived from mapping the supplied function onto
         each node in the graph.
@@ -143,7 +145,8 @@ class Graph(collections.defaultdict):
 
     def __str__(self):
         """Convert the graph to a human-readable string."""
-        pairs = ('%s: %s' % (str(k), str(v)) for k, v in six.iteritems(self))
+        pairs = ('%s: %s' % (six.text_type(k), six.text_type(v))
+                 for k, v in six.iteritems(self))
         text = '{%s}' % ', '.join(pairs)
         return six.text_type(text)
 
@@ -165,6 +168,7 @@ class Graph(collections.defaultdict):
                 raise CircularDependencyException(cycle=six.text_type(graph))
 
 
+@repr_wraper
 @six.python_2_unicode_compatible
 class Dependencies(object):
     """Helper class for calculating a dependency graph."""
@@ -172,7 +176,7 @@ class Dependencies(object):
     def __init__(self, edges=None):
         """Initialise, optionally with a list of edges.
 
-        Each edge has the form of (requirer, required) tuple.
+        Each edge takes the form of a (requirer, required) tuple.
         """
         edges = edges or []
         self._graph = Graph()
@@ -207,10 +211,10 @@ class Dependencies(object):
         return self._graph[target].requires()
 
     def __getitem__(self, last):
-        """Partial dependency graph consisting of the specified node.
+        """Return a partial dependency graph starting with the specified node.
 
-        Return a partial dependency graph consisting of the specified node and
-        all those that require it only.
+        Return a subset of the dependency graph consisting of the specified
+        node and all those that require it only.
         """
         if last not in self._graph:
             raise KeyError
