@@ -17,7 +17,6 @@ import glob
 import itertools
 import os.path
 import re
-import warnings
 
 from oslo_config import cfg
 from oslo_log import log
@@ -35,13 +34,25 @@ from heat.engine import support
 LOG = log.getLogger(__name__)
 
 
-HOOK_TYPES = (HOOK_PRE_CREATE, HOOK_PRE_UPDATE) = ('pre-create', 'pre-update')
+HOOK_TYPES = (
+    HOOK_PRE_CREATE, HOOK_PRE_UPDATE, HOOK_PRE_DELETE, HOOK_POST_CREATE,
+    HOOK_POST_UPDATE, HOOK_POST_DELETE
+) = (
+    'pre-create', 'pre-update', 'pre-delete', 'post-create',
+    'post-update', 'post-delete'
+)
+
+RESTRICTED_ACTIONS = (UPDATE, REPLACE) = ('update', 'replace')
 
 
 def valid_hook_type(hook):
     return hook in HOOK_TYPES
 
 
+def valid_restricted_actions(action):
+    return action in RESTRICTED_ACTIONS
+
+
 def is_hook_definition(key, value):
     is_valid_hook = False
     if key == 'hooks':
@@ -59,6 +70,25 @@ def is_hook_definition(key, value):
     return is_valid_hook
 
 
+def is_valid_restricted_action(key, value):
+    valid_action = False
+    if key == 'restricted_actions':
+        if isinstance(value, six.string_types):
+            valid_action = valid_restricted_actions(value)
+        elif isinstance(value, collections.Sequence):
+            valid_action = all(valid_restricted_actions(
+                action) for action in value)
+
+        if not valid_action:
+            msg = (_('Invalid restricted_action type "%(value)s" for '
+                     'resource, acceptable restricted_action '
+                     'types are: %(types)s') %
+                   {'value': value, 'types': RESTRICTED_ACTIONS})
+            raise exception.InvalidRestrictedAction(message=msg)
+
+    return valid_action
+
+
 class ResourceInfo(object):
     """Base mapping of resource type to implementation."""
 
@@ -189,16 +219,30 @@ class GlobResourceInfo(MapResourceInfo):
     """Store the mapping (with wild cards) of one resource type to another.
 
     like: OS::Networking::* -> OS::Neutron::*
+
+    Also supports many-to-one mapping (mostly useful together with special
+    "OS::Heat::None" resource)
+
+    like: OS::* -> OS::Heat::None
     """
     description = 'Wildcard Mapping'
 
     def get_resource_info(self, resource_type=None, resource_name=None):
+        # NOTE(pas-ha) we end up here only when self.name already
+        # ends with * so truncate it
         orig_prefix = self.name[:-1]
-        new_type = self.value[:-1] + resource_type[len(orig_prefix):]
+        if self.value.endswith('*'):
+            new_type = self.value[:-1] + resource_type[len(orig_prefix):]
+        else:
+            new_type = self.value
+
         return self.registry.get_resource_info(new_type, resource_name)
 
     def matches(self, resource_type):
-        return resource_type.startswith(self.name[:-1])
+        # prevent self-recursion in case of many-to-one mapping
+        match = (resource_type != self.value and
+                 resource_type.startswith(self.name[:-1]))
+        return match
 
 
 class ResourceRegistry(object):
@@ -222,28 +266,27 @@ class ResourceRegistry(object):
         for k, v in iter(registry.items()):
             if v is None:
                 self._register_info(path + [k], None)
-            elif is_hook_definition(k, v):
-                self._register_hook(path + [k], v)
+            elif is_hook_definition(k, v) or is_valid_restricted_action(k, v):
+                self._register_item(path + [k], v)
             elif isinstance(v, dict):
                 self._load_registry(path + [k], v)
             else:
                 self._register_info(path + [k],
                                     ResourceInfo(self, path + [k], v))
 
-    def _register_hook(self, path, hook):
+    def _register_item(self, path, item):
         name = path[-1]
         registry = self._registry
         for key in path[:-1]:
             if key not in registry:
                 registry[key] = {}
             registry = registry[key]
-        registry[name] = hook
+        registry[name] = item
 
     def _register_info(self, path, info):
         """Place the new info in the correct location in the registry.
 
-        :param path: a list of keys ['resources', 'my_server',
-         'OS::Nova::Server']
+        :param path: a list of keys ['resources', 'my_srv', 'OS::Nova::Server']
         """
         descriptive_path = '/'.join(path)
         name = path[-1]
@@ -260,13 +303,13 @@ class ResourceRegistry(object):
                 for res_name in list(six.iterkeys(registry)):
                     if (isinstance(registry[res_name], ResourceInfo) and
                             res_name.startswith(name[:-1])):
-                        LOG.warn(_LW('Removing %(item)s from %(path)s'), {
+                        LOG.warning(_LW('Removing %(item)s from %(path)s'), {
                             'item': res_name,
                             'path': descriptive_path})
                         del registry[res_name]
             else:
                 # delete this entry.
-                LOG.warn(_LW('Removing %(item)s from %(path)s'), {
+                LOG.warning(_LW('Removing %(item)s from %(path)s'), {
                     'item': name,
                     'path': descriptive_path})
                 registry.pop(name, None)
@@ -279,24 +322,34 @@ class ResourceRegistry(object):
                 'path': descriptive_path,
                 'was': str(registry[name].value),
                 'now': str(info.value)}
-            LOG.warn(_LW('Changing %(path)s from %(was)s to %(now)s'),
-                     details)
+            LOG.warning(_LW('Changing %(path)s from %(was)s to %(now)s'),
+                        details)
 
         if isinstance(info, ClassResourceInfo):
             if info.value.support_status.status != support.SUPPORTED:
                 if info.value.support_status.message is not None:
-                    warnings.warn(six.text_type(
-                        info.value.support_status.message))
+                    details = {
+                        'name': info.name,
+                        'status': six.text_type(
+                            info.value.support_status.status),
+                        'message': six.text_type(
+                            info.value.support_status.message)
+                        }
+                    LOG.warning(_LW('%(name)s is %(status)s. %(message)s'),
+                                details)
 
         info.user_resource = (self.global_registry is not None)
         registry[name] = info
 
     def log_resource_info(self, show_all=False, prefix=None):
         registry = self._registry
+        prefix = '%s ' % prefix if prefix is not None else ''
         for name in registry:
+            if name == 'resources':
+                continue
             if show_all or isinstance(registry[name], TemplateResourceInfo):
-                msg = (_('%(p)s Registered: %(t)s') %
-                       {'p': prefix or '',
+                msg = (_LI('%(p)sRegistered: %(t)s') %
+                       {'p': prefix,
                         't': six.text_type(registry[name])})
                 LOG.info(msg)
 
@@ -310,6 +363,33 @@ class ResourceRegistry(object):
         if info.path[-1] in registry:
             registry.pop(info.path[-1])
 
+    def get_rsrc_restricted_actions(self, resource_name):
+        """Returns a set of restricted actions.
+
+        For a given resource we get the set of restricted actions.
+
+        Actions are set in this format via `resources`:
+
+            {
+                "restricted_actions": [update, replace]
+            }
+
+        A restricted_actions value is either `update`, `replace` or a list
+        of those values. Resources support wildcard matching. The asterisk
+        sign matches everything.
+        """
+        ress = self._registry['resources']
+        restricted_actions = set()
+        for name_pattern, resource in six.iteritems(ress):
+            if fnmatch.fnmatchcase(resource_name, name_pattern):
+                if 'restricted_actions' in resource:
+                    actions = resource['restricted_actions']
+                    if isinstance(actions, six.string_types):
+                        restricted_actions.add(actions)
+                    elif isinstance(actions, collections.Sequence):
+                        restricted_actions |= set(actions)
+        return restricted_actions
+
     def matches_hook(self, resource_name, hook):
         """Return whether a resource have a hook set in the environment.
 
@@ -420,8 +500,12 @@ class ResourceRegistry(object):
                                   giter)
 
         for info in sorted(matches):
-            match = info.get_resource_info(resource_type,
-                                           resource_name)
+            try:
+                match = info.get_resource_info(resource_type,
+                                               resource_name)
+            except exception.EntityNotFound:
+                continue
+
             if registry_type is None or isinstance(match, registry_type):
                 if ignore is not None and match == ignore:
                     continue
@@ -436,11 +520,12 @@ class ResourceRegistry(object):
                     self._register_info([resource_type], info)
                 return match
 
+        raise exception.EntityNotFound(entity='Resource Type',
+                                       name=resource_type)
+
     def get_class(self, resource_type, resource_name=None, files=None):
         info = self.get_resource_info(resource_type,
                                       resource_name=resource_name)
-        if info is None:
-            raise exception.ResourceTypeNotFound(type_name=resource_type)
         return info.get_class(files=files)
 
     def get_class_to_instantiate(self, resource_type, resource_name=None):
@@ -455,11 +540,12 @@ class ResourceRegistry(object):
             msg = _('Resource "%s" type is not a string') % resource_name
             raise exception.StackValidationFailed(message=msg)
 
-        info = self.get_resource_info(resource_type,
-                                      resource_name=resource_name)
-        if info is None:
-            msg = _("Unknown resource Type : %s") % resource_type
-            raise exception.StackValidationFailed(message=msg)
+        try:
+            info = self.get_resource_info(resource_type,
+                                          resource_name=resource_name)
+        except exception.EntityNotFound as exc:
+            raise exception.StackValidationFailed(message=six.text_type(exc))
+
         return info.get_class_to_instantiate()
 
     def as_dict(self):
@@ -469,7 +555,8 @@ class ResourceRegistry(object):
             for k, v in iter(level.items()):
                 if isinstance(v, dict):
                     tmp[k] = _as_dict(v)
-                elif is_hook_definition(k, v):
+                elif is_hook_definition(
+                        k, v) or is_valid_restricted_action(k, v):
                     tmp[k] = v
                 elif v.user_resource:
                     tmp[k] = v.value
@@ -504,7 +591,10 @@ class ResourceRegistry(object):
             if cnxt is None:
                 return True
 
-            return cls.get_class().is_service_available(cnxt)
+            try:
+                return cls.get_class().is_service_available(cnxt)
+            except Exception:
+                return False
 
         def not_hidden_matches(cls):
             return cls.get_class().support_status.status != support.HIDDEN
@@ -544,30 +634,30 @@ class ResourceRegistry(object):
 class Environment(object):
 
     def __init__(self, env=None, user_env=True):
-        """Create an Environment from a dict of varying format.
+        """Create an Environment from an input dict.
 
-        Next formats are available:
-          1) old-school flat parameters
-          2) or newer {resource_registry: bla, parameters: foo}
+        The dict may be in one of two formats:
+          1) old-school flat parameters; or
+          2) newer {resource_registry: bla, parameters: foo}
 
         :param env: the json environment
-        :param user_env: boolean, if false then we manage python resources too.
+        :param user_env: boolean, if False then we manage python resources too.
         """
         if env is None:
             env = {}
         if user_env:
             from heat.engine import resources
-            global_registry = resources.global_env().registry
+            global_env = resources.global_env()
+            global_registry = global_env.registry
+            event_sink_classes = global_env.event_sink_classes
         else:
             global_registry = None
+            event_sink_classes = {}
 
         self.registry = ResourceRegistry(global_registry, self)
         self.registry.load(env.get(env_fmt.RESOURCE_REGISTRY, {}))
 
-        if env_fmt.PARAMETER_DEFAULTS in env:
-            self.param_defaults = env[env_fmt.PARAMETER_DEFAULTS]
-        else:
-            self.param_defaults = {}
+        self.param_defaults = env.get(env_fmt.PARAMETER_DEFAULTS, {})
 
         self.encrypted_param_names = env.get(env_fmt.ENCRYPTED_PARAM_NAMES, [])
 
@@ -576,7 +666,13 @@ class Environment(object):
         else:
             self.params = dict((k, v) for (k, v) in six.iteritems(env)
                                if k not in (env_fmt.PARAMETER_DEFAULTS,
+                                            env_fmt.ENCRYPTED_PARAM_NAMES,
+                                            env_fmt.EVENT_SINKS,
                                             env_fmt.RESOURCE_REGISTRY))
+        self.event_sink_classes = event_sink_classes
+        self._event_sinks = []
+        self._built_event_sinks = []
+        self._update_event_sinks(env.get(env_fmt.EVENT_SINKS, []))
         self.constraints = {}
         self.stack_lifecycle_plugins = []
 
@@ -585,13 +681,15 @@ class Environment(object):
         self.params.update(env_snippet.get(env_fmt.PARAMETERS, {}))
         self.param_defaults.update(
             env_snippet.get(env_fmt.PARAMETER_DEFAULTS, {}))
+        self._update_event_sinks(env_snippet.get(env_fmt.EVENT_SINKS, []))
 
     def user_env_as_dict(self):
         """Get the environment as a dict, ready for storing in the db."""
         return {env_fmt.RESOURCE_REGISTRY: self.registry.as_dict(),
                 env_fmt.PARAMETERS: self.params,
                 env_fmt.PARAMETER_DEFAULTS: self.param_defaults,
-                env_fmt.ENCRYPTED_PARAM_NAMES: self.encrypted_param_names}
+                env_fmt.ENCRYPTED_PARAM_NAMES: self.encrypted_param_names,
+                env_fmt.EVENT_SINKS: self._event_sinks}
 
     def register_class(self, resource_type, resource_class, path=None):
         self.registry.register_class(resource_type, resource_class, path=path)
@@ -604,6 +702,9 @@ class Environment(object):
         self.stack_lifecycle_plugins.append((stack_lifecycle_name,
                                              stack_lifecycle_class))
 
+    def register_event_sink(self, event_sink_name, event_sink_class):
+        self.event_sink_classes[event_sink_name] = event_sink_class
+
     def get_class(self, resource_type, resource_name=None, files=None):
         return self.registry.get_class(resource_type, resource_name,
                                        files=files)
@@ -633,6 +734,17 @@ class Environment(object):
     def get_stack_lifecycle_plugins(self):
         return self.stack_lifecycle_plugins
 
+    def _update_event_sinks(self, sinks):
+        self._event_sinks.extend(sinks)
+        for sink in sinks:
+            sink = sink.copy()
+            sink_class = sink.pop('type')
+            sink_class = self.event_sink_classes[sink_class]
+            self._built_event_sinks.append(sink_class(**sink))
+
+    def get_event_sinks(self):
+        return self._built_event_sinks
+
 
 def get_child_environment(parent_env, child_params, item_to_remove=None,
                           child_resource_name=None):
@@ -643,7 +755,7 @@ def get_child_environment(parent_env, child_params, item_to_remove=None,
     environment.
 
     1. resource_registry must be merged (child env should be loaded after the
-       parent env to take presence).
+       parent env to take precedence).
     2. child parameters must overwrite the parent's as they won't be relevant
        in the child template.
 
