@@ -80,6 +80,15 @@ class GetParam(function.Function):
                 raise TypeError(_('Path components in "%s" '
                                   'must be strings') % self.fn_name)
 
+            if isinstance(collection, collections.Sequence
+                          ) and isinstance(key, six.string_types):
+                try:
+                    key = int(key)
+                except ValueError:
+                    raise TypeError(_("Path components in '%s' "
+                                      "must be a string that can be "
+                                      "parsed into an "
+                                      "integer.") % self.fn_name)
             return collection[key]
 
         try:
@@ -238,8 +247,9 @@ class Replace(cfn_funcs.Replace):
 
         "<value_1> <value_2>"
 
-    This is implemented using Python's str.replace on each key. The order in
-    which replacements are performed is undefined.
+    This is implemented using python str.replace on each key. Longer keys are
+    substituted before shorter ones, but the order in which replacements are
+    performed is otherwise undefined.
     """
 
     def _parse_args(self):
@@ -265,9 +275,25 @@ class Replace(cfn_funcs.Replace):
 class ReplaceJson(Replace):
     """A function for performing string substitutions.
 
-    Behaves the same as Replace, but tolerates non-string parameter
-    values, e.g map/list - these are serialized as json before doing
-    the string substitution.
+    Takes the form::
+
+        str_replace:
+          template: <key_1> <key_2>
+          params:
+            <key_1>: <value_1>
+            <key_2>: <value_2>
+            ...
+
+    And resolves to::
+
+        "<value_1> <value_2>"
+
+    This is implemented using python str.replace on each key. Longer keys are
+    substituted before shorter ones, but the order in which replacements are
+    performed is otherwise undefined.
+
+    Non-string param values (e.g maps or lists) are serialized as JSON before
+    being substituted in.
     """
 
     def result(self):
@@ -309,6 +335,9 @@ class ReplaceJson(Replace):
 
             return string.replace(placeholder, six.text_type(value))
 
+        mapping = collections.OrderedDict(sorted(mapping.items(),
+                                                 key=lambda t: len(t[0]),
+                                                 reverse=True))
         return six.moves.reduce(replace, six.iteritems(mapping), template)
 
 
@@ -667,6 +696,10 @@ class StrSplit(function.Function):
         except (AttributeError, IndexError):
             raise ValueError(_('Incorrect arguments to "%(fn_name)s" '
                                'should be: %(example)s') % self.fmt_data)
+
+        if str_to_split is None:
+            return None
+
         split_list = str_to_split.split(delim)
 
         # Optionally allow an index to be specified
