@@ -351,9 +351,10 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
         self.ipaddress = None
 
     def _set_ipaddress(self, networks):
-        '''
-        Read the server's IP address from a list of networks provided by Nova
-        '''
+        """Set IP address to self.ipaddress from a list of networks.
+
+        Read the server's IP address from a list of networks provided by Nova.
+        """
         # Just record the first ipaddress
         for n in sorted(networks, reverse=True):
             if len(networks[n]) > 0:
@@ -361,9 +362,10 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
                 break
 
     def _ipaddress(self):
-        '''
-        Return the server's IP address, fetching it from Nova if necessary
-        '''
+        """Return the server's IP address.
+
+        Fetching it from Nova if necessary.
+        """
         if self.ipaddress is None:
             self.ipaddress = self.client_plugin().server_to_ipaddress(
                 self.resource_id)
@@ -371,9 +373,10 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
         return self.ipaddress or '0.0.0.0'
 
     def _availability_zone(self):
-        '''
-        Return Server's Availability Zone, fetching it from Nova if necessary.
-        '''
+        """Return Server's Availability Zone.
+
+        Fetching it from Nova if necessary.
+        """
         availability_zone = self.properties[self.AVAILABILITY_ZONE]
         if availability_zone is None:
             try:
@@ -401,10 +404,8 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
         # delete the port data which implicit-created
         port_id = self.data().get('port_id')
         if port_id:
-            try:
+            with self.client_plugin('neutron').ignore_not_found:
                 self.neutron().delete_port(port_id)
-            except Exception as ex:
-                self.client_plugin('neutron').ignore_not_found(ex)
             self.data_delete('port_id')
 
     def _build_nics(self, network_interfaces,
@@ -509,9 +510,10 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
 
         image_name = self.properties[self.IMAGE_ID]
 
-        image_id = self.client_plugin('glance').get_image_id(image_name)
+        image_id = self.client_plugin(
+            'glance').find_image_by_name_or_id(image_name)
 
-        flavor_id = self.client_plugin().get_flavor_id(flavor)
+        flavor_id = self.client_plugin().find_flavor_by_name_or_id(flavor)
 
         scheduler_hints = {}
         if self.properties[self.NOVA_SCHEDULER_HINTS]:
@@ -600,9 +602,10 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
         return out
 
     def volumes(self):
-        """
-        Return an iterator over (volume_id, device) tuples for all volumes
-        that should be attached to this instance.
+        """Return an iterator for all volumes that should be attached.
+
+        Return an iterator over (volume_id, device) tuples for all volumes that
+        should be attached to this instance.
         """
         volumes = self.properties[self.VOLUMES]
 
@@ -625,9 +628,9 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
 
     def _update_instance_type(self, prop_diff):
         flavor = prop_diff[self.INSTANCE_TYPE]
-        flavor_id = self.client_plugin().get_flavor_id(flavor)
+        flavor_id = self.client_plugin().find_flavor_by_name_or_id(flavor)
         handler_args = {'args': (flavor_id,)}
-        checker_args = {'args': (flavor_id, flavor)}
+        checker_args = {'args': (flavor_id,)}
 
         prg_resize = progress.ServerUpdateProgress(self.resource_id,
                                                    'resize',
@@ -676,11 +679,11 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
         # keep the behavior as creation
         elif (old_network_ifaces and
                 (self.NETWORK_INTERFACES not in prop_diff)):
-            LOG.warn(_LW('There is no change of "%(net_interfaces)s" '
-                         'for instance %(server)s, do nothing '
-                         'when updating.'),
-                     {'net_interfaces': self.NETWORK_INTERFACES,
-                      'server': self.resource_id})
+            LOG.warning(_LW('There is no change of "%(net_interfaces)s" '
+                            'for instance %(server)s, do nothing '
+                            'when updating.'),
+                        {'net_interfaces': self.NETWORK_INTERFACES,
+                         'server': self.resource_id})
         # if the interfaces not come from property 'NetworkInterfaces',
         # the situation is somewhat complex, so to detach the old ifaces,
         # and then attach the new ones.
@@ -749,7 +752,7 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
         return updaters
 
     def check_update_complete(self, updaters):
-        '''Push all updaters to completion in list order.'''
+        """Push all updaters to completion in list order."""
         for prg in updaters:
             if not prg.called:
                 handler = getattr(self.client_plugin(), prg.handler)
@@ -764,16 +767,12 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
         return all(prg.complete for prg in updaters)
 
     def metadata_update(self, new_metadata=None):
-        '''
-        Refresh the metadata if new_metadata is None
-        '''
+        """Refresh the metadata if new_metadata is None."""
         if new_metadata is None:
             self.metadata_set(self.t.metadata())
 
     def validate(self):
-        '''
-        Validate any of the provided params
-        '''
+        """Validate any of the provided params."""
         res = super(Instance, self).validate()
         if res:
             return res
@@ -807,12 +806,12 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
         if network_interfaces and subnet_id:
             # consider the old templates, we only to log to warn user
             # NetworkInterfaces has higher priority than SubnetId
-            LOG.warn(_LW('"%(subnet)s" will be ignored if specified '
-                         '"%(net_interfaces)s". So if you specified the '
-                         '"%(net_interfaces)s" property, '
-                         'do not specify "%(subnet)s" property.'),
-                     {'subnet': self.SUBNET_ID,
-                      'net_interfaces': self.NETWORK_INTERFACES})
+            LOG.warning(_LW('"%(subnet)s" will be ignored if specified '
+                            '"%(net_interfaces)s". So if you specified the '
+                            '"%(net_interfaces)s" property, '
+                            'do not specify "%(subnet)s" property.'),
+                        {'subnet': self.SUBNET_ID,
+                         'net_interfaces': self.NETWORK_INTERFACES})
 
     def handle_delete(self):
         # make sure to delete the port which implicit-created by heat
@@ -833,11 +832,12 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
         return self.client_plugin().check_delete_server_complete(server_id)
 
     def handle_suspend(self):
-        '''
-        Suspend an instance - note we do not wait for the SUSPENDED state,
-        this is polled for by check_suspend_complete in a similar way to the
-        create logic so we can take advantage of coroutines
-        '''
+        """Suspend an instance.
+
+        Note we do not wait for the SUSPENDED state, this is polled for by
+        check_suspend_complete in a similar way to the create logic so we can
+        take advantage of coroutines.
+        """
         if self.resource_id is None:
             raise exception.Error(_('Cannot suspend %s, resource_id not set') %
                                   self.name)
@@ -875,11 +875,12 @@ class Instance(resource.Resource, sh.SchedulerHintsMixin):
             raise exc
 
     def handle_resume(self):
-        '''
-        Resume an instance - note we do not wait for the ACTIVE state,
-        this is polled for by check_resume_complete in a similar way to the
-        create logic so we can take advantage of coroutines
-        '''
+        """Resume an instance.
+
+        Note we do not wait for the ACTIVE state, this is polled for by
+        check_resume_complete in a similar way to the create logic so we can
+        take advantage of coroutines.
+        """
         if self.resource_id is None:
             raise exception.Error(_('Cannot resume %s, resource_id not set') %
                                   self.name)
