@@ -20,6 +20,13 @@ from heat.engine import support
 
 
 class Secret(resource.Resource):
+    """The resource provides access to the secret/keying stored material.
+
+    A secret is a singular item that stored within Barbican. A secret is
+    anything you want it to be; however, the formal use case is a key that you
+    wish to store away from prying eyes. Secret may include private keys,
+    passwords and so on.
+    """
 
     support_status = support.SupportStatus(version='2014.2')
 
@@ -41,6 +48,12 @@ class Secret(resource.Resource):
         'status', 'decrypted_payload',
     )
 
+    _SECRET_TYPES = (
+        SYMMETRIC, PUBLIC, PRIVATE, CERTIFICATE, PASSPHRASE, OPAQUE
+    ) = (
+        'symmetric', 'public', 'private', 'certificate', 'passphrase', 'opaque'
+    )
+
     properties_schema = {
         NAME: properties.Schema(
             properties.Schema.STRING,
@@ -54,12 +67,10 @@ class Secret(resource.Resource):
             properties.Schema.STRING,
             _('The type of the secret.'),
             constraints=[
-                constraints.AllowedValues([
-                    'symmetric', 'public', 'private', 'certificate',
-                    'passphrase', 'opaque'
-                ]),
+                constraints.AllowedValues(_SECRET_TYPES),
             ],
             support_status=support.SupportStatus(version='5.0.0'),
+            default=OPAQUE
         ),
         PAYLOAD_CONTENT_TYPE: properties.Schema(
             properties.Schema.STRING,
