@@ -25,32 +25,39 @@ from heat.engine import properties
 from heat.engine.resources import scheduler_hints as sh
 from heat.engine.resources import volume_base as vb
 from heat.engine import support
+from heat.engine import translation
 
 LOG = logging.getLogger(__name__)
 
 
 class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
+    """A resource that implements Cinder volumes.
+
+    Cinder volume is a storage in the form of block devices. It can be used,
+    for example, for providing storage to instance. Volume supports creation
+    from snapshot, backup or image. Also volume can be created only by size.
+    """
 
     PROPERTIES = (
         AVAILABILITY_ZONE, SIZE, SNAPSHOT_ID, BACKUP_ID, NAME,
         DESCRIPTION, VOLUME_TYPE, METADATA, IMAGE_REF, IMAGE,
-        SOURCE_VOLID, CINDER_SCHEDULER_HINTS, READ_ONLY,
+        SOURCE_VOLID, CINDER_SCHEDULER_HINTS, READ_ONLY, MULTI_ATTACH,
     ) = (
         'availability_zone', 'size', 'snapshot_id', 'backup_id', 'name',
         'description', 'volume_type', 'metadata', 'imageRef', 'image',
-        'source_volid', 'scheduler_hints', 'read_only',
+        'source_volid', 'scheduler_hints', 'read_only', 'multiattach',
     )
 
     ATTRIBUTES = (
         AVAILABILITY_ZONE_ATTR, SIZE_ATTR, SNAPSHOT_ID_ATTR, DISPLAY_NAME_ATTR,
         DISPLAY_DESCRIPTION_ATTR, VOLUME_TYPE_ATTR, METADATA_ATTR,
         SOURCE_VOLID_ATTR, STATUS, CREATED_AT, BOOTABLE, METADATA_VALUES_ATTR,
-        ENCRYPTED_ATTR, ATTACHMENTS,
+        ENCRYPTED_ATTR, ATTACHMENTS, MULTI_ATTACH_ATTR,
     ) = (
         'availability_zone', 'size', 'snapshot_id', 'display_name',
         'display_description', 'volume_type', 'metadata',
         'source_volid', 'status', 'created_at', 'bootable', 'metadata_values',
-        'encrypted', 'attachments',
+        'encrypted', 'attachments', 'multiattach',
     )
 
     properties_schema = {
@@ -146,6 +153,12 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
             support_status=support.SupportStatus(version='5.0.0'),
             update_allowed=True,
         ),
+        MULTI_ATTACH: properties.Schema(
+            properties.Schema.BOOLEAN,
+            _('Whether allow the volume to be attached more than once. '
+              'This property is only supported from Cinder API v2.'),
+            support_status=support.SupportStatus(version='6.0.0'),
+        ),
     }
 
     attributes_schema = {
@@ -205,6 +218,12 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
             _('The list of attachments of the volume.'),
             type=attributes.Schema.STRING
         ),
+        MULTI_ATTACH_ATTR: attributes.Schema(
+            _('Boolean indicating whether allow the volume to be attached '
+              'more than once.'),
+            type=attributes.Schema.BOOLEAN,
+            support_status=support.SupportStatus(version='6.0.0'),
+        ),
     }
 
     _volume_creating_status = ['creating', 'restoring-backup', 'downloading']
@@ -213,9 +232,9 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
 
     def translation_rules(self, props):
         return [
-            properties.TranslationRule(
+            translation.TranslationRule(
                 props,
-                properties.TranslationRule.REPLACE,
+                translation.TranslationRule.REPLACE,
                 [self.IMAGE],
                 value_path=[self.IMAGE_REF]
             )
@@ -242,15 +261,17 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
             arguments[self.CINDER_SCHEDULER_HINTS] = scheduler_hints
 
         if self.properties[self.IMAGE]:
-            arguments['imageRef'] = self.client_plugin('glance').get_image_id(
+            arguments['imageRef'] = self.client_plugin(
+                'glance').find_image_by_name_or_id(
                 self.properties[self.IMAGE])
         elif self.properties[self.IMAGE_REF]:
             arguments['imageRef'] = self.properties[self.IMAGE_REF]
 
         optionals = (self.SNAPSHOT_ID, self.VOLUME_TYPE, self.SOURCE_VOLID,
-                     self.METADATA)
+                     self.METADATA, self.MULTI_ATTACH)
+
         arguments.update((prop, self.properties[prop]) for prop in optionals
-                         if self.properties[prop])
+                         if self.properties[prop] is not None)
 
         return arguments
 
@@ -273,14 +294,15 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
         volume = self.client().volumes.get(self.resource_id)
         return volume._info
 
-    def handle_create(self):
-        vol_id = super(CinderVolume, self).handle_create()
-        read_only_flag = self.properties.get(self.READ_ONLY)
-        if read_only_flag is not None:
-            self.client().volumes.update_readonly_flag(vol_id,
-                                                       read_only_flag)
+    def check_create_complete(self, vol_id):
+        complete = super(CinderVolume, self).check_create_complete(vol_id)
+        # Cinder just supports update read only for volume in available,
+        # if we update in handle_create(), maybe the volume still in
+        # creating, then cinder will raise an exception
+        if complete:
+            self._update_read_only(self.properties[self.READ_ONLY])
 
-        return vol_id
+        return complete
 
     def _extend_volume(self, new_size):
         try:
@@ -289,11 +311,18 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
             if self.client_plugin().is_client_exception(ex):
                 raise exception.Error(_(
                     "Failed to extend volume %(vol)s - %(err)s") % {
-                        'vol': self.resource_id, 'err': str(ex)})
+                        'vol': self.resource_id, 'err': six.text_type(ex)})
             else:
                 raise
         return True
 
+    def _update_read_only(self, read_only_flag):
+        if read_only_flag is not None:
+            self.client().volumes.update_readonly_flag(self.resource_id,
+                                                       read_only_flag)
+
+        return True
+
     def _check_extend_volume_complete(self):
         vol = self.client().volumes.get(self.resource_id)
         if vol.status == 'extending':
@@ -347,7 +376,9 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
         prg_resize = None
         prg_attach = None
         prg_detach = None
-        prg_backup_restore = None
+        prg_restore = None
+        prg_access = None
+
         # update the name and description for cinder volume
         if self.NAME in prop_diff or self.DESCRIPTION in prop_diff:
             vol = cinder.volumes.get(self.resource_id)
@@ -378,11 +409,15 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
                 cinder.volumes.retype(vol, new_vol_type, 'never')
         # update read_only access mode
         if self.READ_ONLY in prop_diff:
+            if not vol:
+                vol = cinder.volumes.get(self.resource_id)
             flag = prop_diff.get(self.READ_ONLY)
-            cinder.volumes.update_readonly_flag(self.resource_id, flag)
+            prg_access = progress.VolumeUpdateAccessModeProgress(
+                read_only=flag)
+            prg_detach, prg_attach = self._detach_attach_progress(vol)
         # restore the volume from backup
         if self.BACKUP_ID in prop_diff:
-            prg_backup_restore = progress.VolumeBackupRestoreProgress(
+            prg_restore = progress.VolumeBackupRestoreProgress(
                 vol_id=self.resource_id,
                 backup_id=prop_diff.get(self.BACKUP_ID))
         # extend volume size
@@ -396,23 +431,30 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
 
             elif new_size > vol.size:
                 prg_resize = progress.VolumeResizeProgress(size=new_size)
-                if vol.attachments:
-                    # NOTE(pshchelo):
-                    # this relies on current behavior of cinder attachments,
-                    # i.e. volume attachments is a list with len<=1,
-                    # so the volume can be attached only to single instance,
-                    # and id of attachment is the same as id of the volume
-                    # it describes, so detach/attach the same volume
-                    # will not change volume attachment id.
-                    server_id = vol.attachments[0]['server_id']
-                    device = vol.attachments[0]['device']
-                    attachment_id = vol.attachments[0]['id']
-                    prg_detach = progress.VolumeDetachProgress(
-                        server_id, vol.id, attachment_id)
-                    prg_attach = progress.VolumeAttachProgress(
-                        server_id, vol.id, device)
+                prg_detach, prg_attach = self._detach_attach_progress(vol)
 
-        return prg_backup_restore, prg_detach, prg_resize, prg_attach
+        return prg_restore, prg_detach, prg_resize, prg_access, prg_attach
+
+    def _detach_attach_progress(self, vol):
+        prg_attach = None
+        prg_detach = None
+        if vol.attachments:
+            # NOTE(pshchelo):
+            # this relies on current behavior of cinder attachments,
+            # i.e. volume attachments is a list with len<=1,
+            # so the volume can be attached only to single instance,
+            # and id of attachment is the same as id of the volume
+            # it describes, so detach/attach the same volume
+            # will not change volume attachment id.
+            server_id = vol.attachments[0]['server_id']
+            device = vol.attachments[0]['device']
+            attachment_id = vol.attachments[0]['id']
+            prg_detach = progress.VolumeDetachProgress(
+                server_id, vol.id, attachment_id)
+            prg_attach = progress.VolumeAttachProgress(
+                server_id, vol.id, device)
+
+        return prg_detach, prg_attach
 
     def _detach_volume_to_complete(self, prg_detach):
         if not prg_detach.called:
@@ -442,18 +484,17 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
             return prg_attach.complete
 
     def check_update_complete(self, checkers):
-        prg_backup_restore, prg_detach, prg_resize, prg_attach = checkers
-        if prg_backup_restore:
-            if not prg_backup_restore.called:
-                prg_backup_restore.called = self._backup_restore(
-                    prg_backup_restore.vol_id,
-                    prg_backup_restore.backup_id)
+        prg_restore, prg_detach, prg_resize, prg_access, prg_attach = checkers
+        if prg_restore:
+            if not prg_restore.called:
+                prg_restore.called = self._backup_restore(
+                    prg_restore.vol_id,
+                    prg_restore.backup_id)
                 return False
-            if not prg_backup_restore.complete:
-                prg_backup_restore.complete = \
-                    self._check_backup_restore_complete()
-                return prg_backup_restore.complete and not prg_resize
-        if not prg_resize:
+            if not prg_restore.complete:
+                prg_restore.complete = self._check_backup_restore_complete()
+                return prg_restore.complete and not prg_resize
+        if not prg_resize and not prg_access:
             return True
         # detach volume
         if prg_detach:
@@ -461,12 +502,19 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
                 self._detach_volume_to_complete(prg_detach)
                 return False
         # resize volume
-        if not prg_resize.called:
-            prg_resize.called = self._extend_volume(prg_resize.size)
-            return False
-        if not prg_resize.complete:
-            prg_resize.complete = self._check_extend_volume_complete()
-            return prg_resize.complete and not prg_attach
+        if prg_resize:
+            if not prg_resize.called:
+                prg_resize.called = self._extend_volume(prg_resize.size)
+                return False
+            if not prg_resize.complete:
+                prg_resize.complete = self._check_extend_volume_complete()
+                return prg_resize.complete and not prg_attach
+        # update read_only access mode
+        if prg_access:
+            if not prg_access.called:
+                prg_access.called = self._update_read_only(
+                    prg_access.read_only)
+                return False
         # reattach volume back
         if prg_attach:
             return self._attach_volume_to_complete(prg_attach)
@@ -562,6 +610,13 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
             raise exception.StackValidationFailed(
                 message=_('Scheduler hints are not supported by the current '
                           'volume API.'))
+        # Multi attach is only supported from Cinder API v2
+        if (self.properties[self.MULTI_ATTACH]
+                and self.client().volume_api_version == 1):
+            raise exception.StackValidationFailed(
+                message=_('Multiple attach is not supported by the current '
+                          'volume API. Use this property since '
+                          'Cinder API v2.'))
         # can not specify both image and imageRef
         image = self.properties.get(self.IMAGE)
         imageRef = self.properties.get(self.IMAGE_REF)
@@ -588,6 +643,11 @@ class CinderVolume(vb.BaseVolume, sh.SchedulerHintsMixin):
 
 
 class CinderVolumeAttachment(vb.BaseVolumeAttachment):
+    """Resource for associating volume to instance.
+
+    Resource for associating existing volume to instance. Also, the location
+    where the volume is exposed on the instance can be specified.
+    """
 
     PROPERTIES = (
         INSTANCE_ID, VOLUME_ID, DEVICE,
