@@ -18,12 +18,19 @@ from heat.engine import attributes
 from heat.engine import constraints
 from heat.engine import properties
 from heat.engine.resources.aws.autoscaling import autoscaling_group as aws_asg
-from heat.engine import rsrc_defn
 from heat.engine import support
+from heat.engine import template
 
 
 class AutoScalingResourceGroup(aws_asg.AutoScalingGroup):
-    """An autoscaling group that can scale arbitrary resources."""
+    """An autoscaling group that can scale arbitrary resources.
+
+    A resource allows to create desired count of similar resources, which
+    defined with resource property in HOT format. If there's necessity of
+    creating many of the same resources (e.g. one hundred sets of Server,
+    WaitCondition and WaitConditionHandle or even Neutron Nets),
+    AutoScalingGroup is the most convenient and easy way to do that.
+    """
 
     PROPERTIES = (
         RESOURCE, MAX_SIZE, MIN_SIZE, COOLDOWN, DESIRED_CAPACITY,
@@ -133,12 +140,12 @@ class AutoScalingResourceGroup(aws_asg.AutoScalingGroup):
     }
     update_policy_schema = {}
 
-    def _get_instance_definition(self):
-        rsrc = self.properties[self.RESOURCE]
-        return rsrc_defn.ResourceDefinition(
-            None, rsrc['type'],
-            properties=rsrc.get('properties'),
-            metadata=rsrc.get('metadata'))
+    def _get_resource_definition(self,
+                                 template_version=('heat_template_version',
+                                                   '2015-04-30')):
+        tmpl = template.Template(dict([template_version]))
+        return tmpl.rsrc_defn_from_snippet(None,
+                                           self.properties[self.RESOURCE])
 
     def _try_rolling_update(self, prop_diff):
         if self.RESOURCE in prop_diff:
@@ -155,7 +162,7 @@ class AutoScalingResourceGroup(aws_asg.AutoScalingGroup):
                      self)._create_template(num_instances, num_replace,
                                             template_version=template_version)
 
-    def FnGetAtt(self, key, *path):
+    def get_attribute(self, key, *path):
         if key == self.CURRENT_SIZE:
             return grouputils.get_size(self)
         if path:
