@@ -31,9 +31,12 @@ LOG = logging.getLogger(__name__)
 
 
 class RemoteStack(resource.Resource):
-    """
-    A Resource representing a stack which can be created using specified
-    context.
+    """A Resource representing a stack.
+
+    A resource that allowing for the creating stack, where should be defined
+    stack template in HOT format, parameters (if template has any parameters
+    with no default value), and timeout of creating. After creating current
+    stack will have remote stack.
     """
     default_client_name = 'heat'
 
@@ -173,10 +176,8 @@ class RemoteStack(resource.Resource):
 
     def handle_delete(self):
         if self.resource_id is not None:
-            try:
+            with self.client_plugin().ignore_not_found:
                 self.heat().stacks.delete(stack_id=self.resource_id)
-            except Exception as e:
-                self.client_plugin().ignore_not_found(e)
 
     def handle_resume(self):
         if self.resource_id is None:
@@ -213,6 +214,11 @@ class RemoteStack(resource.Resource):
 
     def _needs_update(self, after, before, after_props, before_props,
                       prev_resource, check_init_complete=True):
+        # If resource is in CHECK_FAILED state, raise UpdateReplace
+        # to replace the failed stack.
+        if self.state == (self.CHECK, self.FAILED):
+            raise exception.UpdateReplace(self)
+
         # Always issue an update to the remote stack and let the individual
         # resources in it decide if they need updating.
         return True
@@ -300,7 +306,7 @@ class RemoteStack(resource.Resource):
             return dict((output['output_key'], output['output_value'])
                         for output in outputs)
 
-    def FnGetRefId(self):
+    def get_reference_id(self):
         return self.resource_id
 
 
