@@ -16,7 +16,6 @@ import copy
 import itertools
 
 import six
-import six.moves
 
 from heat.common import exception
 from heat.common import grouputils
@@ -24,28 +23,22 @@ from heat.common.i18n import _
 from heat.common import timeutils
 from heat.engine import attributes
 from heat.engine import constraints
+from heat.engine import function
+from heat.engine.hot import template
 from heat.engine import properties
 from heat.engine.resources import stack_resource
 from heat.engine import rsrc_defn
 from heat.engine import scheduler
 from heat.engine import support
-from heat.engine import template
 from heat.scaling import rolling_update
-from heat.scaling import template as scale_template
-
-template_template = {
-    "heat_template_version": "2015-04-30",
-    "resources": {}
-}
+from heat.scaling import template as scl_template
 
 
 class ResourceGroup(stack_resource.StackResource):
-    """
-    A resource that creates one or more identically configured nested
-    resources.
+    """Creates one or more identically configured nested resources.
 
     In addition to the `refs` attribute, this resource implements synthetic
-    attributes that mirror those of the resources in the group.  When
+    attributes that mirror those of the resources in the group. When
     getting an attribute from this resource, however, a list of attribute
     values for each resource in the group is returned. To get attribute values
     for a single resource in the group, synthetic attributes of the form
@@ -151,16 +144,16 @@ class ResourceGroup(stack_resource.StackResource):
             schema={
                 RESOURCE_DEF_TYPE: properties.Schema(
                     properties.Schema.STRING,
-                    _('The type of the resources in the group'),
+                    _('The type of the resources in the group.'),
                     required=True
                 ),
                 RESOURCE_DEF_PROPERTIES: properties.Schema(
                     properties.Schema.MAP,
-                    _('Property values for the resources in the group')
+                    _('Property values for the resources in the group.')
                 ),
                 RESOURCE_DEF_METADATA: properties.Schema(
                     properties.Schema.MAP,
-                    _('Supplied metadata for the resources in the group'),
+                    _('Supplied metadata for the resources in the group.'),
                     support_status=support.SupportStatus(version='5.0.0')
                 ),
 
@@ -170,7 +163,7 @@ class ResourceGroup(stack_resource.StackResource):
         ),
         REMOVAL_POLICIES: properties.Schema(
             properties.Schema.LIST,
-            _('Policies for removal of resources on update'),
+            _('Policies for removal of resources on update.'),
             schema=properties.Schema(
                 properties.Schema.MAP,
                 _('Policy to be processed when doing an update which '
@@ -185,11 +178,11 @@ class ResourceGroup(stack_resource.StackResource):
                           "(1) The resource name, as in the nested stack, "
                           "(2) The resource reference returned from "
                           "get_resource in a template, as available via "
-                          "the 'refs' attribute "
+                          "the 'refs' attribute. "
                           "Note this is destructive on update when specified; "
                           "even if the count is not being reduced, and once "
                           "a resource name is removed, it's name is never "
-                          "reused in subsequent updates"
+                          "reused in subsequent updates."
                           ),
                         default=[]
                     ),
@@ -203,12 +196,12 @@ class ResourceGroup(stack_resource.StackResource):
 
     attributes_schema = {
         REFS: attributes.Schema(
-            _("A list of resource IDs for the resources in the group"),
+            _("A list of resource IDs for the resources in the group."),
             type=attributes.Schema.LIST
         ),
         ATTR_ATTRIBUTES: attributes.Schema(
             _("A map of resource names to the specified attribute of each "
-              "individual resource.  "
+              "individual resource. "
               "Requires heat_template_version: 2014-10-16."),
             support_status=support.SupportStatus(version='2014.2'),
             type=attributes.Schema.MAP
@@ -263,11 +256,6 @@ class ResourceGroup(stack_resource.StackResource):
         )
     }
 
-    def __init__(self, name, json_snippet, stack):
-        super(ResourceGroup, self).__init__(name, json_snippet, stack)
-        self.update_policy = self.t.update_policy(self.update_policy_schema,
-                                                  self.context)
-
     def get_size(self):
         return self.properties.get(self.COUNT)
 
@@ -279,8 +267,8 @@ class ResourceGroup(stack_resource.StackResource):
             return
 
         test_tmpl = self._assemble_nested(["0"], include_all=True)
-        val_templ = template.Template(test_tmpl)
-        res_def = val_templ.resource_definitions(self.stack)["0"]
+        res_def = next(six.itervalues(
+            test_tmpl.resource_definitions(self.stack)))
         # make sure we can resolve the nested resource type
         self.stack.env.get_class_to_instantiate(res_def.resource_type)
 
@@ -345,13 +333,8 @@ class ResourceGroup(stack_resource.StackResource):
                                                       candidates),
                                 size)
 
-    def _get_resources(self):
-        """Get templates for resources."""
-        return [(resource.name, resource.t.render_hot())
-                for resource in grouputils.get_members(self)]
-
     def _count_black_listed(self):
-        """Return the number of current resource names that are blacklisted"""
+        """Return the number of current resource names that are blacklisted."""
         existing_members = grouputils.get_member_names(self)
         return len(self._name_blacklist() & set(existing_members))
 
@@ -361,16 +344,15 @@ class ResourceGroup(stack_resource.StackResource):
             max_batch_size = batch_create[self.MAX_BATCH_SIZE]
             pause_sec = batch_create[self.PAUSE_TIME]
             checkers = self._replace(0, max_batch_size, pause_sec)
+            checkers[0].start()
+            return checkers
         else:
             names = self._resource_names()
-            checkers = [(scheduler.TaskRunner(
-                self._run_to_completion,
-                self._assemble_nested(names),
-                self.stack.timeout_secs()))]
-        checkers[0].start()
-        return checkers
+            self.create_with_template(self._assemble_nested(names),
+                                      self.child_params(),
+                                      self.stack.timeout_secs())
 
-    def check_create_complete(self, checkers):
+    def check_create_complete(self, checkers=None):
         if checkers is None:
             return super(ResourceGroup, self).check_create_complete()
         for checker in checkers:
@@ -427,7 +409,7 @@ class ResourceGroup(stack_resource.StackResource):
         checkers[0].start()
         return checkers
 
-    def FnGetAtt(self, key, *path):
+    def get_attribute(self, key, *path):
         if key.startswith("resource."):
             return grouputils.get_nested_attrs(self, key, False, *path)
 
@@ -446,52 +428,93 @@ class ResourceGroup(stack_resource.StackResource):
         return [grouputils.get_rsrc_attr(self, key, False, n, *path)
                 for n in names]
 
-    def _build_resource_definition(self, include_all=False):
+    def build_resource_definition(self, res_name, res_defn):
+        res_def = copy.deepcopy(res_defn)
+        props = res_def.get(self.RESOURCE_DEF_PROPERTIES)
+        if props:
+            repl_props = self._handle_repl_val(res_name, props)
+            res_def[self.RESOURCE_DEF_PROPERTIES] = repl_props
+        return template.HOTemplate20130523.rsrc_defn_from_snippet(res_name,
+                                                                  res_def)
+
+    def get_resource_def(self, include_all=False):
+        """Returns the resource definition portion of the group.
+
+        :param include_all: if False, only properties for the resource
+               definition that are not empty will be included
+        :type include_all: bool
+        :return: resource definition for the group
+        :rtype: dict
+        """
+
+        # At this stage, we don't mind if all of the parameters have values
+        # assigned. Pass in a custom resolver to the properties to not
+        # error when a parameter does not have a user entered value.
+        def ignore_param_resolve(snippet):
+            while isinstance(snippet, function.Function):
+                try:
+                    snippet = snippet.result()
+                except exception.UserParameterMissing:
+                    return None
+
+            if isinstance(snippet, collections.Mapping):
+                return dict((k, ignore_param_resolve(v))
+                            for k, v in snippet.items())
+            elif (not isinstance(snippet, six.string_types) and
+                  isinstance(snippet, collections.Iterable)):
+                return [ignore_param_resolve(v) for v in snippet]
+
+            return snippet
+
+        self.properties.resolve = ignore_param_resolve
+
         res_def = self.properties[self.RESOURCE_DEF]
-        if res_def[self.RESOURCE_DEF_PROPERTIES] is None:
-            res_def[self.RESOURCE_DEF_PROPERTIES] = {}
-        if res_def[self.RESOURCE_DEF_METADATA] is None:
-            del res_def[self.RESOURCE_DEF_METADATA]
         if not include_all:
-            resource_def_props = res_def[self.RESOURCE_DEF_PROPERTIES]
-            clean = dict((k, v) for k, v in resource_def_props.items()
-                         if v is not None)
-            res_def[self.RESOURCE_DEF_PROPERTIES] = clean
+            return self._clean_props(res_def)
+        return res_def
+
+    def _clean_props(self, res_defn):
+        res_def = copy.deepcopy(res_defn)
+        props = res_def.get(self.RESOURCE_DEF_PROPERTIES)
+        if props:
+            clean = dict((k, v) for k, v in props.items() if v is not None)
+            props = clean
+            res_def[self.RESOURCE_DEF_PROPERTIES] = props
         return res_def
 
     def _handle_repl_val(self, res_name, val):
         repl_var = self.properties[self.INDEX_VAR]
-        recurse = lambda x: self._handle_repl_val(res_name, x)
+
+        def recurse(x):
+            return self._handle_repl_val(res_name, x)
+
         if isinstance(val, six.string_types):
             return val.replace(repl_var, res_name)
         elif isinstance(val, collections.Mapping):
-            return dict(zip(val, map(recurse, six.itervalues(val))))
+            return {k: recurse(v) for k, v in val.items()}
         elif isinstance(val, collections.Sequence):
-            return map(recurse, val)
+            return [recurse(v) for v in val]
         return val
 
-    def _do_prop_replace(self, res_name, res_def_template):
-        res_def = copy.deepcopy(res_def_template)
-        props = res_def[self.RESOURCE_DEF_PROPERTIES]
-        if props:
-            props = self._handle_repl_val(res_name, props)
-            res_def[self.RESOURCE_DEF_PROPERTIES] = props
-        return res_def
+    def _assemble_nested(self, names, include_all=False,
+                         template_version=('heat_template_version',
+                                           '2015-04-30')):
 
-    def _assemble_nested(self, names, include_all=False):
-        res_def = self._build_resource_definition(include_all)
-        resources = dict((k, self._do_prop_replace(k, res_def))
-                         for k in names)
-        child_template = copy.deepcopy(template_template)
-        child_template['resources'] = resources
-        return child_template
+        def_dict = self.get_resource_def(include_all)
+        definitions = [(k, self.build_resource_definition(k, def_dict))
+                       for k in names]
+        return scl_template.make_template(definitions,
+                                          version=template_version)
 
     def _assemble_for_rolling_update(self, total_capacity, max_updates,
-                                     include_all=False):
+                                     include_all=False,
+                                     template_version=('heat_template_version',
+                                                       '2015-04-30')):
         names = list(self._resource_names(total_capacity))
         name_blacklist = self._name_blacklist()
 
-        valid_resources = [(n, d) for n, d in self._get_resources()
+        valid_resources = [(n, d) for n, d in
+                           grouputils.get_member_definitions(self)
                            if n not in name_blacklist]
 
         targ_cap = self.get_size()
@@ -512,21 +535,18 @@ class ResourceGroup(stack_resource.StackResource):
                     return total_capacity
 
         old_resources = sorted(valid_resources, key=replace_priority)
-
         existing_names = set(n for n, d in valid_resources)
         new_names = six.moves.filterfalse(lambda n: n in existing_names,
                                           names)
-
-        res_def = self._build_resource_definition(include_all)
-        resources = scale_template.member_definitions(old_resources, res_def,
-                                                      total_capacity,
-                                                      max_updates,
-                                                      lambda: next(new_names),
-                                                      self._do_prop_replace)
-
-        child_template = copy.deepcopy(template_template)
-        child_template['resources'] = dict(resources)
-        return child_template
+        res_def = self.get_resource_def(include_all)
+        definitions = scl_template.member_definitions(
+            old_resources, res_def,
+            total_capacity,
+            max_updates,
+            lambda: next(new_names),
+            self.build_resource_definition)
+        return scl_template.make_template(definitions,
+                                          version=template_version)
 
     def _try_rolling_update(self):
         if self.update_policy[self.ROLLING_UPDATE]:
