@@ -36,6 +36,9 @@ class AutoScalingPolicy(signal_responder.SignalResponder,
     **Note** while it may incidentally support
     `AWS::AutoScaling::AutoScalingGroup` for now, please don't use it for that
     purpose and use `AWS::AutoScaling::ScalingPolicy` instead.
+
+    Resource to manage scaling for `OS::Heat::AutoScalingGroup`, i.e. define
+    which metric should be scaled and scaling adjustment, set cooldown etc.
     """
     PROPERTIES = (
         AUTO_SCALING_GROUP_NAME, SCALING_ADJUSTMENT, ADJUSTMENT_TYPE,
@@ -110,9 +113,7 @@ class AutoScalingPolicy(signal_responder.SignalResponder,
     }
 
     def validate(self):
-        """
-        Add validation for min_adjustment_step
-        """
+        """Add validation for min_adjustment_step."""
         super(AutoScalingPolicy, self).validate()
         self._validate_min_adjustment_step()
 
@@ -131,7 +132,8 @@ class AutoScalingPolicy(signal_responder.SignalResponder,
         self.resource_id_set(self._get_user_id())
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        """
+        """Updates self.properties, if Properties has changed.
+
         If Properties has changed, update self.properties, so we get the new
         values during any subsequent adjustment.
         """
@@ -192,13 +194,15 @@ class AutoScalingPolicy(signal_responder.SignalResponder,
                 changed_size=changed_size)
 
     def _resolve_attribute(self, name):
+        if self.resource_id is None:
+            return
         if name == self.ALARM_URL:
             return six.text_type(self._get_ec2_signed_url())
         elif name == self.SIGNAL_URL:
             return six.text_type(self._get_heat_signal_url())
 
-    def FnGetRefId(self):
-        return resource.Resource.FnGetRefId(self)
+    def get_reference_id(self):
+        return resource.Resource.get_reference_id(self)
 
 
 def resource_mapping():
