@@ -19,7 +19,14 @@ from heat.engine import support
 
 
 class KeystoneProject(resource.Resource):
-    """Heat Template Resource for Keystone Project."""
+    """Heat Template Resource for Keystone Project.
+
+    Projects represent the base unit of ownership in OpenStack, in that all
+    resources in OpenStack should be owned by a specific project. A project
+    itself must be owned by a specific domain, and hence all project names are
+    not globally unique, but unique to their domain. If the domain for a
+    project is not specified, then it is added to the default domain.
+    """
 
     support_status = support.SupportStatus(
         version='2015.1',
@@ -30,9 +37,9 @@ class KeystoneProject(resource.Resource):
     entity = 'projects'
 
     PROPERTIES = (
-        NAME, DOMAIN, DESCRIPTION, ENABLED
+        NAME, DOMAIN, DESCRIPTION, ENABLED, PARENT,
     ) = (
-        'name', 'domain', 'description', 'enabled'
+        'name', 'domain', 'description', 'enabled', 'parent',
     )
 
     properties_schema = {
@@ -59,77 +66,58 @@ class KeystoneProject(resource.Resource):
             _('This project is enabled or disabled.'),
             default=True,
             update_allowed=True
-        )
+        ),
+        PARENT: properties.Schema(
+            properties.Schema.STRING,
+            _('The name or ID of parent of this keystone project '
+              'in hierarchy.'),
+            support_status=support.SupportStatus(version='6.0.0'),
+            constraints=[constraints.CustomConstraint('keystone.project')]
+        ),
     }
 
     def client(self):
         return super(KeystoneProject, self).client().client
 
-    def _create_project(self,
-                        project_name,
-                        description,
-                        domain,
-                        enabled):
-        domain = self.client_plugin().get_domain_id(domain)
+    def handle_create(self):
+        project_name = (self.properties[self.NAME] or
+                        self.physical_resource_name())
+        description = self.properties[self.DESCRIPTION]
+        domain = self.client_plugin().get_domain_id(
+            self.properties[self.DOMAIN])
+        enabled = self.properties[self.ENABLED]
+        pp = self.properties[self.PARENT]
+        parent = self.client_plugin().get_project_id(pp)
 
-        return self.client().projects.create(
+        project = self.client().projects.create(
             name=project_name,
             domain=domain,
             description=description,
-            enabled=enabled)
-
-    def _update_project(self,
-                        project_id,
-                        domain,
-                        new_name=None,
-                        new_description=None,
-                        enabled=None):
-        values = dict()
-
-        if new_name is not None:
-            values['name'] = new_name
-        if new_description is not None:
-            values['description'] = new_description
-        if enabled is not None:
-            values['enabled'] = enabled
-
-        values['project'] = project_id
-        domain = self.client_plugin().get_domain_id(domain)
-
-        values['domain'] = domain
-
-        return self.client().projects.update(**values)
-
-    def handle_create(self):
-        project_name = (self.properties.get(self.NAME) or
-                        self.physical_resource_name())
-        description = self.properties.get(self.DESCRIPTION)
-        domain = self.properties.get(self.DOMAIN)
-        enabled = self.properties.get(self.ENABLED)
-
-        project = self._create_project(
-            project_name=project_name,
-            description=description,
-            domain=domain,
-            enabled=enabled
-        )
+            enabled=enabled,
+            parent=parent)
 
         self.resource_id_set(project.id)
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        name = prop_diff.get(self.NAME) or self.physical_resource_name()
-        description = prop_diff.get(self.DESCRIPTION)
-        enabled = prop_diff.get(self.ENABLED)
-        domain = (prop_diff.get(self.DOMAIN) or
-                  self._stored_properties_data.get(self.DOMAIN))
+        if prop_diff:
+            name = None
+            # Don't update the name if no change
+            if self.NAME in prop_diff:
+                name = prop_diff[self.NAME] or self.physical_resource_name()
 
-        self._update_project(
-            project_id=self.resource_id,
-            domain=domain,
-            new_name=name,
-            new_description=description,
-            enabled=enabled
-        )
+            description = prop_diff.get(self.DESCRIPTION)
+            enabled = prop_diff.get(self.ENABLED)
+            domain = (prop_diff.get(self.DOMAIN) or
+                      self._stored_properties_data.get(self.DOMAIN))
+            domain_id = self.client_plugin().get_domain_id(domain)
+
+            self.client().projects.update(
+                project=self.resource_id,
+                name=name,
+                description=description,
+                enabled=enabled,
+                domain=domain_id
+            )
 
 
 def resource_mapping():
