@@ -10,11 +10,8 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-from oslo_utils import uuidutils
 import six
 
-import warnings
-
 from heat.common import exception
 from heat.common.i18n import _
 from heat.engine import resource
@@ -24,15 +21,8 @@ class NeutronResource(resource.Resource):
 
     default_client_name = 'neutron'
 
-    # Subclasses provide a list of properties which, although
-    # update_allowed in the schema, should be excluded from the
-    # call to neutron, because they are handled in _needs_update
-    update_exclude_properties = []
-
     def validate(self):
-        '''
-        Validate any of the provided params
-        '''
+        """Validate any of the provided params."""
         res = super(NeutronResource, self).validate()
         if res:
             return res
@@ -40,13 +30,14 @@ class NeutronResource(resource.Resource):
 
     @staticmethod
     def validate_properties(properties):
-        '''
-        Validates to ensure nothing in value_specs overwrites
-        any key that exists in the schema.
+        """Validate properties for the resource.
+
+        Validates to ensure nothing in value_specs overwrites any key that
+        exists in the schema.
 
         Also ensures that shared and tenant_id is not specified
         in value_specs.
-        '''
+        """
         if 'value_specs' in six.iterkeys(properties):
             vs = properties.get('value_specs')
             banned_keys = set(['shared', 'tenant_id']).union(
@@ -56,41 +47,39 @@ class NeutronResource(resource.Resource):
 
     @staticmethod
     def prepare_properties(properties, name):
-        '''
+        """Prepares the property values for correct Neutron create call.
+
         Prepares the property values so that they can be passed directly to
         the Neutron create call.
 
         Removes None values and value_specs, merges value_specs with the main
         values.
-        '''
+        """
         props = dict((k, v) for k, v in properties.items()
-                     if v is not None and k != 'value_specs')
+                     if v is not None)
 
         if 'name' in six.iterkeys(properties):
             props.setdefault('name', name)
 
-        if 'value_specs' in six.iterkeys(properties):
-            props.update(properties.get('value_specs'))
-
+        if 'value_specs' in props:
+            NeutronResource.merge_value_specs(props)
         return props
 
-    def prepare_update_properties(self, definition):
-        '''
-        Prepares the property values so that they can be passed directly to
-        the Neutron update call.
+    @staticmethod
+    def merge_value_specs(props):
+        value_spec_props = props.pop('value_specs')
+        props.update(value_spec_props)
 
-        Removes any properties which are not update_allowed, then processes
-        as for prepare_properties.
-        '''
-        p = definition.properties(self.properties_schema, self.context)
-        update_props = dict((k, v) for k, v in p.items()
-                            if p.props.get(k).schema.update_allowed and
-                            k not in self.update_exclude_properties)
+    def prepare_update_properties(self, prop_diff):
+        """Prepares prop_diff values for correct neutron update call.
 
-        props = self.prepare_properties(
-            update_props,
-            self.physical_resource_name())
-        return props
+        1. Merges value_specs
+        2. Defaults resource name to physical resource name if None
+        """
+        if 'value_specs' in prop_diff and prop_diff['value_specs']:
+            NeutronResource.merge_value_specs(prop_diff)
+        if 'name' in prop_diff and prop_diff['name'] is None:
+            prop_diff['name'] = self.physical_resource_name()
 
     @staticmethod
     def is_built(attributes):
@@ -112,46 +101,7 @@ class NeutronResource(resource.Resource):
         return attributes[name]
 
     def get_reference_id(self):
-        return six.text_type(self.resource_id)
-
-    @staticmethod
-    def get_secgroup_uuids(security_groups, client, tenant_id):
-        '''
-        Returns a list of security group UUIDs.
-        Args:
-            security_groups: List of security group names or UUIDs
-            client: reference to neutronclient
-            tenant_id: the tenant id to match the security_groups
-        '''
-        warnings.warn('neutron.NeutronResource.get_secgroup_uuids is '
-                      'deprecated. Use '
-                      'self.client_plugin("neutron").get_secgroup_uuids')
-        seclist = []
-        all_groups = None
-        for sg in security_groups:
-            if uuidutils.is_uuid_like(sg):
-                seclist.append(sg)
-            else:
-                if not all_groups:
-                    response = client.list_security_groups()
-                    all_groups = response['security_groups']
-                same_name_groups = [g for g in all_groups if g['name'] == sg]
-                groups = [g['id'] for g in same_name_groups]
-                if len(groups) == 0:
-                    raise exception.PhysicalResourceNotFound(resource_id=sg)
-                elif len(groups) == 1:
-                    seclist.append(groups[0])
-                else:
-                    # for admin roles, can get the other users'
-                    # securityGroups, so we should match the tenant_id with
-                    # the groups, and return the own one
-                    own_groups = [g['id'] for g in same_name_groups
-                                  if g['tenant_id'] == tenant_id]
-                    if len(own_groups) == 1:
-                        seclist.append(own_groups[0])
-                    else:
-                        raise exception.PhysicalResourceNameAmbiguity(name=sg)
-        return seclist
+        return self.resource_id
 
     def _not_found_in_call(self, func, *args, **kwargs):
         try:
