@@ -11,8 +11,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import six
-
 from heat.common import exception
 from heat.common.i18n import _
 from heat.engine import attributes
@@ -23,15 +21,24 @@ from heat.engine import support
 
 
 class ProviderNet(net.Net):
+    """A resource for managing Neutron provider networks.
+
+    Provider networks specify details of physical realisation of the existing
+    network.
+    """
+
+    required_service_extension = 'provider'
 
     support_status = support.SupportStatus(version='2014.1')
 
     PROPERTIES = (
         NAME, PROVIDER_NETWORK_TYPE, PROVIDER_PHYSICAL_NETWORK,
         PROVIDER_SEGMENTATION_ID, ADMIN_STATE_UP, SHARED,
+        ROUTER_EXTERNAL,
     ) = (
         'name', 'network_type', 'physical_network',
         'segmentation_id', 'admin_state_up', 'shared',
+        'router_external',
     )
 
     ATTRIBUTES = (
@@ -45,7 +52,7 @@ class ProviderNet(net.Net):
         PROVIDER_NETWORK_TYPE: properties.Schema(
             properties.Schema.STRING,
             _('A string specifying the provider network type for the '
-                'network.'),
+              'network.'),
             update_allowed=True,
             required=True,
             constraints=[
@@ -55,14 +62,14 @@ class ProviderNet(net.Net):
         PROVIDER_PHYSICAL_NETWORK: properties.Schema(
             properties.Schema.STRING,
             _('A string specifying physical network mapping for the '
-                'network.'),
+              'network.'),
             update_allowed=True,
             required=True,
         ),
         PROVIDER_SEGMENTATION_ID: properties.Schema(
             properties.Schema.STRING,
             _('A string specifying the segmentation id for the '
-                'network.'),
+              'network.'),
             update_allowed=True
         ),
         ADMIN_STATE_UP: net.Net.properties_schema[ADMIN_STATE_UP],
@@ -72,6 +79,13 @@ class ProviderNet(net.Net):
             default=True,
             update_allowed=True
         ),
+        ROUTER_EXTERNAL: properties.Schema(
+            properties.Schema.BOOLEAN,
+            _('Whether the network contains an external router.'),
+            default=False,
+            update_allowed=True,
+            support_status=support.SupportStatus(version='6.0.0')
+        ),
     }
 
     attributes_schema = {
@@ -86,10 +100,11 @@ class ProviderNet(net.Net):
     }
 
     def validate(self):
-        '''
+        """Resource's validation.
+
         Validates to ensure that segmentation_id is not there for flat
         network type.
-        '''
+        """
         super(ProviderNet, self).validate()
 
         if (self.properties[self.PROVIDER_SEGMENTATION_ID] and
@@ -102,36 +117,47 @@ class ProviderNet(net.Net):
         props['provider:' + key] = props.pop(key)
 
     @staticmethod
-    def prepare_provider_properties(self, props):
-        self.add_provider_extension(props, self.PROVIDER_NETWORK_TYPE)
+    def prepare_provider_properties(props):
+        if ProviderNet.PROVIDER_NETWORK_TYPE in props:
+            ProviderNet.add_provider_extension(
+                props,
+                ProviderNet.PROVIDER_NETWORK_TYPE)
+        if ProviderNet.PROVIDER_PHYSICAL_NETWORK in props:
+            ProviderNet.add_provider_extension(
+                props,
+                ProviderNet.PROVIDER_PHYSICAL_NETWORK)
+        if ProviderNet.PROVIDER_SEGMENTATION_ID in props:
+            ProviderNet.add_provider_extension(
+                props,
+                ProviderNet.PROVIDER_SEGMENTATION_ID)
 
-        self.add_provider_extension(props, self.PROVIDER_PHYSICAL_NETWORK)
-
-        if self.PROVIDER_SEGMENTATION_ID in six.iterkeys(props):
-            self.add_provider_extension(props, self.PROVIDER_SEGMENTATION_ID)
+        if ProviderNet.ROUTER_EXTERNAL in props:
+            props['router:external'] = props.pop(ProviderNet.ROUTER_EXTERNAL)
 
     def handle_create(self):
-        '''
+        """Creates the resource with provided properties.
+
         Adds 'provider:' extension to the required properties during create.
-        '''
+        """
         props = self.prepare_properties(
             self.properties,
             self.physical_resource_name())
 
-        self.prepare_provider_properties(self, props)
+        ProviderNet.prepare_provider_properties(props)
 
         prov_net = self.client().create_network({'network': props})['network']
         self.resource_id_set(prov_net['id'])
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        '''
+        """Updates the resource with provided properties.
+
         Adds 'provider:' extension to the required properties during update.
-        '''
-        props = self.prepare_update_properties(json_snippet)
-
-        self.prepare_provider_properties(self, props)
-
-        self.client().update_network(self.resource_id, {'network': props})
+        """
+        if prop_diff:
+            ProviderNet.prepare_provider_properties(prop_diff)
+            self.prepare_update_properties(prop_diff)
+            self.client().update_network(self.resource_id,
+                                         {'network': prop_diff})
 
 
 def resource_mapping():
