@@ -21,9 +21,17 @@ from heat.engine import properties
 from heat.engine.resources.openstack.neutron import neutron
 from heat.engine.resources.openstack.neutron import subnet
 from heat.engine import support
+from heat.engine import translation
 
 
 class Router(neutron.NeutronResource):
+    """A resource that implements Neutron router.
+
+    Router is a physical or virtual network device that passes network traffic
+    between different networks.
+    """
+
+    required_service_extension = 'router'
 
     PROPERTIES = (
         NAME, EXTERNAL_GATEWAY, VALUE_SPECS, ADMIN_STATE_UP,
@@ -35,8 +43,15 @@ class Router(neutron.NeutronResource):
 
     _EXTERNAL_GATEWAY_KEYS = (
         EXTERNAL_GATEWAY_NETWORK, EXTERNAL_GATEWAY_ENABLE_SNAT,
+        EXTERNAL_GATEWAY_FIXED_IPS,
     ) = (
-        'network', 'enable_snat',
+        'network', 'enable_snat', 'external_fixed_ips',
+    )
+
+    _EXTERNAL_GATEWAY_FIXED_IPS_KEYS = (
+        IP_ADDRESS, SUBNET
+    ) = (
+        'ip_address', 'subnet'
     )
 
     ATTRIBUTES = (
@@ -70,6 +85,32 @@ class Router(neutron.NeutronResource):
                       'this property to administrative users only.'),
                     update_allowed=True
                 ),
+                EXTERNAL_GATEWAY_FIXED_IPS: properties.Schema(
+                    properties.Schema.LIST,
+                    _('External fixed IP addresses for the gateway.'),
+                    schema=properties.Schema(
+                        properties.Schema.MAP,
+                        schema={
+                            IP_ADDRESS: properties.Schema(
+                                properties.Schema.STRING,
+                                _('External fixed IP address.'),
+                                constraints=[
+                                    constraints.CustomConstraint('ip_addr'),
+                                ]
+                            ),
+                            SUBNET: properties.Schema(
+                                properties.Schema.STRING,
+                                _('Subnet of external fixed IP address.'),
+                                constraints=[
+                                    constraints.CustomConstraint(
+                                        'neutron.subnet')
+                                ]
+                            ),
+                        }
+                    ),
+                    update_allowed=True,
+                    support_status=support.SupportStatus(version='6.0.0')
+                ),
             },
             update_allowed=True
         ),
@@ -92,10 +133,15 @@ class Router(neutron.NeutronResource):
               'users only.'),
             update_allowed=True,
             support_status=support.SupportStatus(
-                status=support.DEPRECATED,
-                version='2015.1',
-                message=_('Use property %s.') % L3_AGENT_IDS,
-                previous_status=support.SupportStatus(version='2014.1')),
+                status=support.HIDDEN,
+                version='6.0.0',
+                previous_status=support.SupportStatus(
+                    status=support.DEPRECATED,
+                    version='2015.1',
+                    message=_('Use property %s.') % L3_AGENT_IDS,
+                    previous_status=support.SupportStatus(version='2014.1')
+                )
+            ),
         ),
         L3_AGENT_IDS: properties.Schema(
             properties.Schema.LIST,
@@ -150,6 +196,38 @@ class Router(neutron.NeutronResource):
         ),
     }
 
+    def translation_rules(self, props):
+        rules = [
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.EXTERNAL_GATEWAY, self.EXTERNAL_GATEWAY_NETWORK],
+                client_plugin=self.client_plugin(),
+                finder='find_resourceid_by_name_or_id',
+                entity='network'),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.EXTERNAL_GATEWAY, self.EXTERNAL_GATEWAY_FIXED_IPS,
+                 self.SUBNET],
+                client_plugin=self.client_plugin(),
+                finder='find_resourceid_by_name_or_id',
+                entity='subnet')
+            ]
+        if props.get(self.L3_AGENT_ID):
+            rules.extend([
+                translation.TranslationRule(
+                    props,
+                    translation.TranslationRule.ADD,
+                    [self.L3_AGENT_IDS],
+                    [props.get(self.L3_AGENT_ID)]),
+                translation.TranslationRule(
+                    props,
+                    translation.TranslationRule.DELETE,
+                    [self.L3_AGENT_ID]
+                )])
+        return rules
+
     def validate(self):
         super(Router, self).validate()
         is_distributed = self.properties[self.DISTRIBUTED]
@@ -182,14 +260,16 @@ class Router(neutron.NeutronResource):
                     if subnet_net == external_gw_net:
                         deps += (self, res)
 
-    def prepare_properties(self, properties, name):
-        props = super(Router, self).prepare_properties(properties, name)
+    def _resolve_gateway(self, props):
         gateway = props.get(self.EXTERNAL_GATEWAY)
         if gateway:
-            self.client_plugin().resolve_network(
-                gateway, self.EXTERNAL_GATEWAY_NETWORK, 'network_id')
+            gateway['network_id'] = gateway.pop(self.EXTERNAL_GATEWAY_NETWORK)
             if gateway[self.EXTERNAL_GATEWAY_ENABLE_SNAT] is None:
                 del gateway[self.EXTERNAL_GATEWAY_ENABLE_SNAT]
+            if gateway[self.EXTERNAL_GATEWAY_FIXED_IPS] is None:
+                del gateway[self.EXTERNAL_GATEWAY_FIXED_IPS]
+            else:
+                self._resolve_subnet(gateway)
         return props
 
     def _get_l3_agent_list(self, props):
@@ -200,11 +280,20 @@ class Router(neutron.NeutronResource):
 
         return l3_agent_ids
 
+    def _resolve_subnet(self, gateway):
+        external_gw_fixed_ips = gateway[self.EXTERNAL_GATEWAY_FIXED_IPS]
+        for fixed_ip in external_gw_fixed_ips:
+            for key, value in six.iteritems(fixed_ip):
+                if value is None:
+                    fixed_ip.pop(key)
+            if self.SUBNET in fixed_ip:
+                fixed_ip['subnet_id'] = fixed_ip.pop(self.SUBNET)
+
     def handle_create(self):
         props = self.prepare_properties(
             self.properties,
             self.physical_resource_name())
-
+        self._resolve_gateway(props)
         l3_agent_ids = self._get_l3_agent_list(props)
 
         router = self.client().create_router({'router': props})['router']
@@ -230,19 +319,17 @@ class Router(neutron.NeutronResource):
             return True
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        props = self.prepare_update_properties(json_snippet)
-        l3_agent_ids = self._get_l3_agent_list(props)
-        if l3_agent_ids:
+        if self.EXTERNAL_GATEWAY in prop_diff:
+            self._resolve_gateway(prop_diff)
+
+        if self.L3_AGENT_IDS in prop_diff or self.L3_AGENT_ID in prop_diff:
+            l3_agent_ids = self._get_l3_agent_list(prop_diff)
             self._replace_agent(l3_agent_ids)
 
-        if self.L3_AGENT_IDS in prop_diff:
-            del prop_diff[self.L3_AGENT_IDS]
-        if self.L3_AGENT_ID in prop_diff:
-            del prop_diff[self.L3_AGENT_ID]
-
-        if len(prop_diff) > 0:
+        if prop_diff:
+            self.prepare_update_properties(prop_diff)
             self.client().update_router(
-                self.resource_id, {'router': props})
+                self.resource_id, {'router': prop_diff})
 
     def _replace_agent(self, l3_agent_ids=None):
         ret = self.client().list_l3_agent_hosting_routers(
@@ -257,6 +344,13 @@ class Router(neutron.NeutronResource):
 
 
 class RouterInterface(neutron.NeutronResource):
+    """A resource for managing Neutron router interfaces.
+
+    Router interfaces associate routers with existing subnets or ports.
+    """
+
+    required_service_extension = 'router'
+
     PROPERTIES = (
         ROUTER, ROUTER_ID, SUBNET_ID, SUBNET, PORT_ID, PORT
     ) = (
@@ -276,10 +370,14 @@ class RouterInterface(neutron.NeutronResource):
             properties.Schema.STRING,
             _('ID of the router.'),
             support_status=support.SupportStatus(
-                status=support.DEPRECATED,
-                message=_('Use property %s.') % ROUTER,
-                version='2015.1',
-                previous_status=support.SupportStatus(version='2013.1')
+                status=support.HIDDEN,
+                version='6.0.0',
+                previous_status=support.SupportStatus(
+                    status=support.DEPRECATED,
+                    message=_('Use property %s.') % ROUTER,
+                    version='2015.1',
+                    previous_status=support.SupportStatus(version='2013.1')
+                )
             ),
             constraints=[
                 constraints.CustomConstraint('neutron.router')
@@ -312,10 +410,14 @@ class RouterInterface(neutron.NeutronResource):
             properties.Schema.STRING,
             _('The port id, either subnet or port_id should be specified.'),
             support_status=support.SupportStatus(
-                status=support.DEPRECATED,
-                message=_('Use property %s.') % PORT,
-                version='2015.1',
-                previous_status=support.SupportStatus(version='2014.1')
+                status=support.HIDDEN,
+                version='6.0.0',
+                previous_status=support.SupportStatus(
+                    status=support.DEPRECATED,
+                    message=_('Use property %s.') % PORT,
+                    version='2015.1',
+                    previous_status=support.SupportStatus(version='2014.1')
+                )
             ),
             constraints=[
                 constraints.CustomConstraint('neutron.port')
@@ -333,24 +435,50 @@ class RouterInterface(neutron.NeutronResource):
 
     def translation_rules(self, props):
         return [
-            properties.TranslationRule(
+            translation.TranslationRule(
                 props,
-                properties.TranslationRule.REPLACE,
+                translation.TranslationRule.REPLACE,
                 [self.PORT],
                 value_path=[self.PORT_ID]
             ),
-            properties.TranslationRule(
+            translation.TranslationRule(
                 props,
-                properties.TranslationRule.REPLACE,
+                translation.TranslationRule.REPLACE,
                 [self.ROUTER],
                 value_path=[self.ROUTER_ID]
             ),
-            properties.TranslationRule(
+            translation.TranslationRule(
                 props,
-                properties.TranslationRule.REPLACE,
+                translation.TranslationRule.REPLACE,
                 [self.SUBNET],
                 value_path=[self.SUBNET_ID]
+            ),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.PORT],
+                client_plugin=self.client_plugin(),
+                finder='find_resourceid_by_name_or_id',
+                entity='port'
+            ),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.ROUTER],
+                client_plugin=self.client_plugin(),
+                finder='find_resourceid_by_name_or_id',
+                entity='router'
+            ),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.SUBNET],
+                client_plugin=self.client_plugin(),
+                finder='find_resourceid_by_name_or_id',
+                entity='subnet'
             )
+
+
         ]
 
     def validate(self):
@@ -370,15 +498,12 @@ class RouterInterface(neutron.NeutronResource):
                                                      self.PORT)
 
     def handle_create(self):
-        router_id = self.client_plugin().resolve_router(
-            dict(self.properties), self.ROUTER, 'router_id')
+        router_id = dict(self.properties).get(self.ROUTER)
         key = 'subnet_id'
-        value = self.client_plugin().resolve_subnet(
-            dict(self.properties), self.SUBNET, key)
+        value = dict(self.properties).get(self.SUBNET)
         if not value:
             key = 'port_id'
-            value = self.client_plugin().resolve_port(
-                dict(self.properties), self.PORT, key)
+            value = dict(self.properties).get(self.PORT)
         self.client().add_interface_router(
             router_id,
             {key: value})
@@ -391,12 +516,10 @@ class RouterInterface(neutron.NeutronResource):
         if len(tokens) == 2:    # compatible with old data
             tokens.insert(1, 'subnet_id')
         (router_id, key, value) = tokens
-        try:
+        with self.client_plugin().ignore_not_found:
             self.client().remove_interface_router(
                 router_id,
                 {key: value})
-        except Exception as ex:
-            self.client_plugin().ignore_not_found(ex)
 
 
 class RouterGateway(neutron.NeutronResource):
@@ -449,12 +572,21 @@ class RouterGateway(neutron.NeutronResource):
 
     def translation_rules(self, props):
         return [
-            properties.TranslationRule(
+            translation.TranslationRule(
                 props,
-                properties.TranslationRule.REPLACE,
+                translation.TranslationRule.REPLACE,
                 [self.NETWORK],
                 value_path=[self.NETWORK_ID]
+            ),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.NETWORK],
+                client_plugin=self.client_plugin(),
+                finder='find_resourceid_by_name_or_id',
+                entity='network'
             )
+
         ]
 
     def add_dependencies(self, deps):
@@ -463,8 +595,13 @@ class RouterGateway(neutron.NeutronResource):
             # depend on any RouterInterface in this template with the same
             # router_id as this router_id
             if resource.has_interface('OS::Neutron::RouterInterface'):
-                dep_router_id = resource.properties.get(
-                    RouterInterface.ROUTER)
+                # Since RouterInterface translates router_id property to
+                # router, we should correctly resolve it for RouterGateway.
+                dep_router_id = self.client_plugin().resolve_router({
+                    RouterInterface.ROUTER: resource.properties.get(
+                        RouterInterface.ROUTER),
+                    RouterInterface.ROUTER_ID: None}, RouterInterface.ROUTER,
+                    RouterInterface.ROUTER_ID)
                 router_id = self.properties[self.ROUTER_ID]
                 if dep_router_id == router_id:
                     deps += (self, resource)
@@ -480,8 +617,7 @@ class RouterGateway(neutron.NeutronResource):
 
     def handle_create(self):
         router_id = self.properties[self.ROUTER_ID]
-        network_id = self.client_plugin().resolve_network(
-            dict(self.properties), self.NETWORK, 'network_id')
+        network_id = dict(self.properties).get(self.NETWORK)
         self.client().add_gateway_router(
             router_id,
             {'network_id': network_id})
@@ -492,10 +628,8 @@ class RouterGateway(neutron.NeutronResource):
             return
 
         (router_id, network_id) = self.resource_id.split(':')
-        try:
+        with self.client_plugin().ignore_not_found:
             self.client().remove_gateway_router(router_id)
-        except Exception as ex:
-            self.client_plugin().ignore_not_found(ex)
 
 
 def resource_mapping():
