@@ -17,12 +17,17 @@ from heat.engine import constraints
 from heat.engine import properties
 from heat.engine.resources.openstack.neutron import neutron
 from heat.engine import support
+from heat.engine import translation
 
 
 class VPNService(neutron.NeutronResource):
+    """A resource for VPN service in Neutron.
+
+    VPN service is a high level object that associates VPN with a specific
+    subnet and router.
     """
-    A resource for VPN service in Neutron.
-    """
+
+    required_service_extension = 'vpnaas'
 
     PROPERTIES = (
         NAME, DESCRIPTION, ADMIN_STATE_UP,
@@ -76,6 +81,7 @@ class VPNService(neutron.NeutronResource):
             properties.Schema.STRING,
             _('Subnet in which the vpn service will be created.'),
             support_status=support.SupportStatus(version='2014.2'),
+            required=True,
             constraints=[
                 constraints.CustomConstraint('neutron.subnet')
             ]
@@ -85,10 +91,14 @@ class VPNService(neutron.NeutronResource):
             _('Unique identifier for the router to which the vpn service '
               'will be inserted.'),
             support_status=support.SupportStatus(
-                status=support.DEPRECATED,
-                message=_('Use property %s') % ROUTER,
-                version='2015.1',
-                previous_status=support.SupportStatus(version='2013.2')),
+                status=support.HIDDEN,
+                version='6.0.0',
+                previous_status=support.SupportStatus(
+                    status=support.DEPRECATED,
+                    message=_('Use property %s') % ROUTER,
+                    version='2015.1',
+                    previous_status=support.SupportStatus(version='2013.2'))
+            ),
             constraints=[
                 constraints.CustomConstraint('neutron.router')
             ]
@@ -97,6 +107,7 @@ class VPNService(neutron.NeutronResource):
             properties.Schema.STRING,
             _('The router to which the vpn service will be inserted.'),
             support_status=support.SupportStatus(version='2015.1'),
+            required=True,
             constraints=[
                 constraints.CustomConstraint('neutron.router')
             ]
@@ -138,24 +149,23 @@ class VPNService(neutron.NeutronResource):
 
     def translation_rules(self, props):
         return [
-            properties.TranslationRule(
+            translation.TranslationRule(
                 props,
-                properties.TranslationRule.REPLACE,
+                translation.TranslationRule.REPLACE,
                 [self.SUBNET],
                 value_path=[self.SUBNET_ID]
+            ),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.REPLACE,
+                [self.ROUTER],
+                value_path=[self.ROUTER_ID]
             )
         ]
 
     def _show_resource(self):
         return self.client().show_vpnservice(self.resource_id)['vpnservice']
 
-    def validate(self):
-        super(VPNService, self).validate()
-        self._validate_depr_property_required(
-            self.properties, self.SUBNET, self.SUBNET_ID)
-        self._validate_depr_property_required(
-            self.properties, self.ROUTER, self.ROUTER_ID)
-
     def handle_create(self):
         props = self.prepare_properties(
             self.properties,
@@ -168,6 +178,7 @@ class VPNService(neutron.NeutronResource):
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         if prop_diff:
+            self.prepare_update_properties(prop_diff)
             self.client().update_vpnservice(self.resource_id,
                                             {'vpnservice': prop_diff})
 
@@ -181,9 +192,13 @@ class VPNService(neutron.NeutronResource):
 
 
 class IPsecSiteConnection(neutron.NeutronResource):
+    """A resource for IPsec site connection in Neutron.
+
+    This resource has details for the site-to-site IPsec connection, including
+    the peer CIDRs, MTU, peer address, DPD settings and status.
     """
-    A resource for IPsec site connection in Neutron.
-    """
+
+    required_service_extension = 'vpnaas'
 
     PROPERTIES = (
         NAME, DESCRIPTION, PEER_ADDRESS, PEER_ID, PEER_CIDRS, MTU,
@@ -425,9 +440,14 @@ class IPsecSiteConnection(neutron.NeutronResource):
 
 
 class IKEPolicy(neutron.NeutronResource):
+    """A resource for IKE policy in Neutron.
+
+    The Internet Key Exchange policy identifyies the authentication and
+    encryption algorithm used during phase one and phase two negotiation of a
+    VPN connection.
     """
-    A resource for IKE policy in Neutron.
-    """
+
+    required_service_extension = 'vpnaas'
 
     PROPERTIES = (
         NAME, DESCRIPTION, AUTH_ALGORITHM, ENCRYPTION_ALGORITHM,
@@ -593,9 +613,13 @@ class IKEPolicy(neutron.NeutronResource):
 
 
 class IPsecPolicy(neutron.NeutronResource):
+    """A resource for IPsec policy in Neutron.
+
+    The IP security policy specifying the authentication and encryption
+    algorithm, and encapsulation mode used for the established VPN connection.
     """
-    A resource for IPsec policy in Neutron.
-    """
+
+    required_service_extension = 'vpnaas'
 
     PROPERTIES = (
         NAME, DESCRIPTION, TRANSFORM_PROTOCOL, ENCAPSULATION_MODE,
