@@ -74,11 +74,12 @@ class ServerNetworkMixin(object):
                         'network': net}
                     raise exception.StackValidationFailed(message=msg)
 
-    def _create_internal_port(self, net_data, net_number):
+    def _create_internal_port(self, net_data, net_number,
+                              security_groups=None):
         name = _('%(server)s-port-%(number)s') % {'server': self.name,
                                                   'number': net_number}
 
-        kwargs = self._prepare_internal_port_kwargs(net_data)
+        kwargs = self._prepare_internal_port_kwargs(net_data, security_groups)
         kwargs['name'] = name
 
         port = self.client('neutron').create_port({'port': kwargs})['port']
@@ -89,7 +90,7 @@ class ServerNetworkMixin(object):
 
         return port['id']
 
-    def _prepare_internal_port_kwargs(self, net_data):
+    def _prepare_internal_port_kwargs(self, net_data, security_groups=None):
         kwargs = {'network_id': self._get_network_id(net_data)}
         fixed_ip = net_data.get(self.NETWORK_FIXED_IP)
         subnet = net_data.get(self.NETWORK_SUBNET)
@@ -102,14 +103,16 @@ class ServerNetworkMixin(object):
         if body:
             kwargs.update({'fixed_ips': [body]})
 
-        if net_data.get(self.SECURITY_GROUPS):
+        if security_groups:
             sec_uuids = self.client_plugin(
-                'neutron').get_secgroup_uuids(net_data.get(
-                    self.SECURITY_GROUPS))
+                'neutron').get_secgroup_uuids(security_groups)
             kwargs['security_groups'] = sec_uuids
 
         extra_props = net_data.get(self.NETWORK_PORT_EXTRA)
         if extra_props is not None:
+            specs = extra_props.pop(neutron_port.Port.VALUE_SPECS)
+            if specs:
+                kwargs.update(specs)
             port_extra_keys = list(neutron_port.Port.EXTRA_PROPERTIES)
             port_extra_keys.remove(neutron_port.Port.ALLOWED_ADDRESS_PAIRS)
             for key in port_extra_keys:
@@ -197,7 +200,7 @@ class ServerNetworkMixin(object):
         for port_id in new_ports:
             self._data_update_ports(port_id, 'add', port_type='external_ports')
 
-    def _build_nics(self, networks):
+    def _build_nics(self, networks, security_groups=None):
         if not networks:
             return None
         nics = []
@@ -208,7 +211,8 @@ class ServerNetworkMixin(object):
             if net.get(self.NETWORK_PORT):
                 nic_info['port-id'] = net[self.NETWORK_PORT]
             elif self.is_using_neutron() and net.get(self.NETWORK_SUBNET):
-                nic_info['port-id'] = self._create_internal_port(net, idx)
+                nic_info['port-id'] = self._create_internal_port(
+                    net, idx, security_groups)
 
             # if nic_info including 'port-id', do not set ip for nic
             if not nic_info.get('port-id'):
@@ -311,7 +315,8 @@ class ServerNetworkMixin(object):
             if net is not None:
                 net['port'] = props['port']
 
-    def calculate_networks(self, old_nets, new_nets, ifaces):
+    def calculate_networks(self, old_nets, new_nets, ifaces,
+                           security_groups=None):
         remove_ports = []
         add_nets = []
         attach_first_free_port = False
@@ -368,8 +373,8 @@ class ServerNetworkMixin(object):
             if net.get(self.NETWORK_PORT):
                 handler_kwargs['port_id'] = net.get(self.NETWORK_PORT)
             elif self.is_using_neutron() and net.get(self.NETWORK_SUBNET):
-                handler_kwargs['port_id'] = self._create_internal_port(net,
-                                                                       idx)
+                handler_kwargs['port_id'] = self._create_internal_port(
+                    net, idx, security_groups)
 
             if not handler_kwargs['port_id']:
                 handler_kwargs['net_id'] = self._get_network_id(net)
