@@ -27,6 +27,11 @@ LOG = logging.getLogger(__name__)
 
 
 class SwiftContainer(resource.Resource):
+    """A resource for managing Swift containers.
+
+    A container defines a namespace for objects. An object with the same
+    name in two different containers represents two different objects.
+    """
     PROPERTIES = (
         NAME, X_CONTAINER_READ, X_CONTAINER_WRITE, X_CONTAINER_META,
         X_ACCOUNT_META, PURGE_ON_DELETE,
@@ -123,10 +128,10 @@ class SwiftContainer(resource.Resource):
 
     @staticmethod
     def _build_meta_headers(obj_type, meta_props):
-        '''
-        Returns a new dict where each key is prepended with:
-        X-Container-Meta-
-        '''
+        """Returns a new dict.
+
+        Each key of new dict is prepended with "X-Container-Meta-".
+        """
         if meta_props is None:
             return {}
         return dict(
@@ -177,10 +182,9 @@ class SwiftContainer(resource.Resource):
             args.append(obj['name'])
         else:
             deleter = self.client().delete_container
-        try:
+
+        with self.client_plugin().ignore_not_found:
             deleter(*args)
-        except Exception as ex:
-            self.client_plugin().ignore_not_found(ex)
 
     def handle_delete(self):
         if self.resource_id is None:
@@ -220,7 +224,7 @@ class SwiftContainer(resource.Resource):
     def handle_check(self):
         self.client().get_container(self.resource_id)
 
-    def FnGetRefId(self):
+    def get_reference_id(self):
         return six.text_type(self.resource_id)
 
     def _resolve_attribute(self, key):
@@ -238,7 +242,7 @@ class SwiftContainer(resource.Resource):
                 headers = self.client().head_container(self.resource_id)
             except Exception as ex:
                 if self.client_plugin().is_client_exception(ex):
-                    LOG.warn(_LW("Head container failed: %s"), ex)
+                    LOG.warning(_LW("Head container failed: %s"), ex)
                     return None
                 raise
             else:
@@ -252,6 +256,40 @@ class SwiftContainer(resource.Resource):
     def _show_resource(self):
         return self.client().head_container(self.resource_id)
 
+    def get_live_resource_data(self):
+        resource_data = super(SwiftContainer, self).get_live_resource_data()
+        account_data = self.client().head_account()
+        resource_data['account_data'] = account_data
+        return resource_data
+
+    def parse_live_resource_data(self, resource_properties, resource_data):
+        swift_reality = {}
+        # swift container name can't be updated
+        swift_reality.update({self.NAME: resource_properties.get(self.NAME)})
+        # PURGE_ON_DELETE property is used only on the heat side and isn't
+        # passed to swift, so update it from existing resource properties
+        swift_reality.update({self.PURGE_ON_DELETE: resource_properties.get(
+            self.PURGE_ON_DELETE)})
+        swift_reality.update({self.X_CONTAINER_META: {}})
+        swift_reality.update({self.X_ACCOUNT_META: {}})
+        for key in [self.X_CONTAINER_READ, self.X_CONTAINER_WRITE]:
+            swift_reality.update({key: resource_data.get(key.lower())})
+        for key in resource_properties.get(self.X_CONTAINER_META):
+            prefixed_key = "%(prefix)s-%(key)s" % {
+                'prefix': self.X_CONTAINER_META.lower(),
+                'key': key.lower()}
+            if prefixed_key in resource_data:
+                swift_reality[self.X_CONTAINER_META].update(
+                    {key: resource_data[prefixed_key]})
+        for key in resource_properties.get(self.X_ACCOUNT_META):
+            prefixed_key = "%(prefix)s-%(key)s" % {
+                'prefix': self.X_ACCOUNT_META.lower(),
+                'key': key.lower()}
+            if prefixed_key in resource_data['account_data']:
+                swift_reality[self.X_ACCOUNT_META].update(
+                    {key: resource_data['account_data'][prefixed_key]})
+        return swift_reality
+
 
 def resource_mapping():
     return {
