@@ -11,7 +11,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from heat.common import exception
 from heat.common.i18n import _
 from heat.engine import attributes
 from heat.engine import properties
@@ -20,6 +19,13 @@ from heat.engine import support
 
 
 class ZaqarQueue(resource.Resource):
+    """A resource for managing Zaqar queues.
+
+    Queue is a logical entity that groups messages. Ideally a queue is created
+    per work type. For example, if you want to compress files, you would create
+    a queue dedicated for this job. Any application that reads from this queue
+    would only compress files.
+    """
 
     default_client_name = "zaqar"
 
@@ -54,11 +60,16 @@ class ZaqarQueue(resource.Resource):
             _("ID of the queue."),
             cache_mode=attributes.Schema.CACHE_NONE,
             support_status=support.SupportStatus(
-                status=support.DEPRECATED,
-                message=_("Use get_resource|Ref command instead. "
-                          "For example: { get_resource : <resource_name> }"),
-                version='2015.1',
-                previous_status=support.SupportStatus(version='2014.1')
+                status=support.HIDDEN,
+                version='6.0.0',
+                previous_status=support.SupportStatus(
+                    status=support.DEPRECATED,
+                    message=_("Use get_resource|Ref command instead. "
+                              "For example: { get_resource : "
+                              "<resource_name> }"),
+                    version='2015.1',
+                    previous_status=support.SupportStatus(version='2014.1')
+                )
             )
         ),
         HREF: attributes.Schema(
@@ -73,26 +84,10 @@ class ZaqarQueue(resource.Resource):
         """Create a zaqar message queue."""
         queue_name = self.physical_resource_name()
         queue = self.client().queue(queue_name, auto_create=False)
-        # Zaqar client doesn't report an error if an queue with the same
-        # id/name already exists, which can cause issue with stack update.
-        if queue.exists():
-            raise exception.Error(_('Message queue %s already exists.')
-                                  % queue_name)
-        queue.ensure_exists()
+        metadata = self.properties.get('metadata')
+        if metadata:
+            queue.metadata(new_meta=metadata)
         self.resource_id_set(queue_name)
-        return queue_name
-
-    def check_create_complete(self, queue_name):
-        """Set metadata of the newly created queue."""
-        queue = self.client().queue(queue_name, auto_create=False)
-        if queue.exists():
-            metadata = self.properties.get('metadata')
-            if metadata:
-                queue.metadata(new_meta=metadata)
-            return True
-        else:
-            raise exception.Error(_('Message queue %s creation failed.')
-                                  % queue_name)
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         """Update queue metadata."""
@@ -105,10 +100,8 @@ class ZaqarQueue(resource.Resource):
         """Delete a zaqar message queue."""
         if not self.resource_id:
             return
-        try:
+        with self.client_plugin().ignore_not_found:
             self.client().queue(self.resource_id, auto_create=False).delete()
-        except Exception as exc:
-            self.client_plugin().ignore_not_found(exc)
 
     def href(self):
         api_endpoint = self.client().api_url
@@ -124,6 +117,17 @@ class ZaqarQueue(resource.Resource):
         elif name == self.HREF:
             return self.href()
 
+    def _show_resource(self):
+        queue = self.client().queue(self.resource_id, auto_create=False)
+        metadata = queue.metadata()
+        return {self.METADATA: metadata}
+
+    def parse_live_resource_data(self, resource_properties, resource_data):
+        return {
+            self.NAME: self.resource_id,
+            self.METADATA: resource_data[self.METADATA]
+        }
+
 
 def resource_mapping():
     return {
