@@ -14,23 +14,23 @@ import collections
 import copy
 import itertools
 import operator
-import warnings
-
+from oslo_log import log
 import six
 
 from heat.common import exception
+from heat.common.i18n import _LW
+from heat.common.i18n import repr_wraper
 from heat.engine import function
 from heat.engine import properties
 
+LOG = log.getLogger(__name__)
 
 __all__ = ['ResourceDefinition']
 
 
+@repr_wraper
 class ResourceDefinitionCore(object):
-    """A definition of a resource.
-
-    Independent of any particular template format.
-    """
+    """A definition of a resource, independent of any template format."""
 
     DELETION_POLICIES = (
         DELETE, RETAIN, SNAPSHOT,
@@ -87,7 +87,7 @@ class ResourceDefinitionCore(object):
 
         if deletion_policy is not None:
             assert deletion_policy in self.DELETION_POLICIES
-            self._hash ^= hash(deletion_policy)
+            self._hash ^= _hash_data(deletion_policy)
 
         if update_policy is not None:
             assert isinstance(update_policy, (collections.Mapping,
@@ -141,7 +141,7 @@ class ResourceDefinitionCore(object):
             update_policy=reparse_snippet(self._update_policy))
 
     def dep_attrs(self, resource_name):
-        """Return an iterator over dependent attributes for resource_name.
+        """Iterate over attributes of a given resource that this references.
 
         Return an iterator over dependent attributes for specified
         resource_name in resources' properties and metadata fields.
@@ -295,7 +295,7 @@ class ResourceDefinition(ResourceDefinitionCore, collections.Mapping):
     ResourceDefinitionCore as soon as M release.
     """
 
-    _deprecation_msg = (
+    _deprecation_msg = _LW(
         'Reading the ResourceDefinition as if it were a snippet of a '
         'CloudFormation template is deprecated, and the ability to treat it '
         'as such will be removed in the future. Resource plugins should use '
@@ -330,7 +330,7 @@ class ResourceDefinition(ResourceDefinitionCore, collections.Mapping):
         This is for backwards compatibility with existing code that expects a
         parsed-JSON template snippet.
         """
-        warnings.warn(self._deprecation_msg, DeprecationWarning)
+        LOG.warning(self._deprecation_msg)
 
         yield TYPE
         if self._properties is not None:
@@ -352,7 +352,7 @@ class ResourceDefinition(ResourceDefinitionCore, collections.Mapping):
         This is for backwards compatibility with existing code that expects a
         parsed-JSON template snippet.
         """
-        warnings.warn(self._deprecation_msg, DeprecationWarning)
+        LOG.warning(self._deprecation_msg)
 
         if key == TYPE:
             return self.resource_type
@@ -385,7 +385,7 @@ class ResourceDefinition(ResourceDefinitionCore, collections.Mapping):
         This is for backwards compatibility with existing code that expects a
         parsed-JSON template snippet.
         """
-        warnings.warn(self._deprecation_msg, DeprecationWarning)
+        LOG.warning(self._deprecation_msg)
 
         return len(list(iter(self)))
 
