@@ -16,12 +16,14 @@ import types
 
 import eventlet
 from oslo_log import log as logging
+from oslo_utils import encodeutils
 from oslo_utils import excutils
 import six
 from six import reraise as raise_
 
 from heat.common.i18n import _
 from heat.common.i18n import _LI
+from heat.common.i18n import repr_wraper
 from heat.common import timeutils
 
 LOG = logging.getLogger(__name__)
@@ -32,18 +34,18 @@ ENABLE_SLEEP = True
 
 
 def task_description(task):
-    """Return a human-readable string description of a task suitable.
+    """Return a human-readable string description of a task.
 
-    Description is used for logging the status of the task.
+    The description is used to identify the task when logging its status.
     """
     name = task.__name__ if hasattr(task, '__name__') else None
-    if isinstance(task, types.MethodType):
-        if name is not None and hasattr(task, '__self__'):
-            return '%s from %s' % (name, task.__self__)
-    elif isinstance(task, types.FunctionType):
-        if name is not None:
+    if name is not None and isinstance(task, (types.MethodType,
+                                              types.FunctionType)):
+        if getattr(task, '__self__', None) is not None:
+            return '%s from %s' % (six.text_type(name), task.__self__)
+        else:
             return six.text_type(name)
-    return repr(task)
+    return encodeutils.safe_decode(repr(task))
 
 
 class Timeout(BaseException):
@@ -124,7 +126,7 @@ class ExceptionGroup(Exception):
         self.exceptions = list(exceptions)
 
     def __str__(self):
-        return six.text_type([six.text_type(ex) for ex in self.exceptions])
+        return str([str(ex) for ex in self.exceptions])
 
 
 @six.python_2_unicode_compatible
@@ -132,9 +134,9 @@ class TaskRunner(object):
     """Wrapper for a resumable task (co-routine)."""
 
     def __init__(self, task, *args, **kwargs):
-        """Initialise with a task function.
+        """Initialise with a task function and arguments.
 
-        Arguments to be passed to task when it is started.
+        The arguments are passed to task when it is started.
 
         The task function may be a co-routine that yields control flow between
         steps.
@@ -200,7 +202,7 @@ class TaskRunner(object):
     def step(self):
         """Run another step of the task.
 
-        :returns: True if the task is complete; False otherwise.
+        Return True if the task is complete; False otherwise.
         """
         if not self.done():
             assert self._runner is not None, "Task not started"
@@ -282,7 +284,10 @@ def wrappertask(task):
     def wrapper(*args, **kwargs):
         parent = task(*args, **kwargs)
 
-        subtask = next(parent)
+        try:
+            subtask = next(parent)
+        except StopIteration:
+            return
 
         while True:
             try:
@@ -296,9 +301,9 @@ def wrappertask(task):
                     while subtask_running:
                         try:
                             yield step
-                        except GeneratorExit as ex:
+                        except GeneratorExit:
                             subtask.close()
-                            raise ex
+                            raise
                         except:  # noqa
                             try:
                                 step = subtask.throw(*sys.exc_info())
@@ -311,17 +316,24 @@ def wrappertask(task):
                                 subtask_running = False
                 else:
                     yield
-            except GeneratorExit as ex:
+            except GeneratorExit:
                 parent.close()
-                raise ex
+                raise
             except:  # noqa
-                subtask = parent.throw(*sys.exc_info())
+                try:
+                    subtask = parent.throw(*sys.exc_info())
+                except StopIteration:
+                    return
             else:
-                subtask = next(parent)
+                try:
+                    subtask = next(parent)
+                except StopIteration:
+                    return
 
     return wrapper
 
 
+@repr_wraper
 class DependencyTaskGroup(object):
     """Task which manages group of subtasks that have ordering dependencies."""
 
@@ -330,11 +342,10 @@ class DependencyTaskGroup(object):
                  aggregate_exceptions=False):
         """Initialise with the task dependencies.
 
-        Optionally initialise with a task to run on each.
-
-        If no task is supplied, it is assumed that the tasks are stored
-        directly in the dependency tree. If a task is supplied, the object
-        stored in the dependency tree is passed as an argument.
+        A task to run on each dependency may optionally be specified.  If no
+        task is supplied, it is assumed that the tasks are stored directly in
+        the dependency tree. If a task is supplied, the object stored in the
+        dependency tree is passed as an argument.
 
         If an error_wait_time is specified, tasks that are already running at
         the time of an error will continue to run for up to the specified
@@ -361,7 +372,7 @@ class DependencyTaskGroup(object):
     def __repr__(self):
         """Return a string representation of the task."""
         text = '%s(%s)' % (type(self).__name__, self.name)
-        return six.text_type(text)
+        return text
 
     def __call__(self):
         """Return a co-routine which runs the task group."""
@@ -412,8 +423,8 @@ class DependencyTaskGroup(object):
     def _ready(self):
         """Iterate over all subtasks that are ready to start.
 
-        All subtasks' dependencies have been satisfied but they have not yet
-        been started.
+        Ready subtasks are subtasks whose dependencies have all been satisfied,
+        but which have not yet been started.
         """
         for k in self._keys:
             if not self._graph.get(k, True):
@@ -424,7 +435,11 @@ class DependencyTaskGroup(object):
     def _running(self):
         """Iterate over all subtasks that are currently running.
 
-        Subtasks have been started but have not yet completed.
+        Running subtasks are subtasks have been started but have not yet
+        completed.
         """
-        running = lambda k_r: k_r[0] in self._graph and k_r[1].started()
+
+        def running(k_r):
+            return k_r[0] in self._graph and k_r[1].started()
+
         return six.moves.filter(running, six.iteritems(self._runners))
