@@ -11,8 +11,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_db import api as oslo_db_api
-from oslo_db import exception as db_exc
+import uuid
+
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_service import service
@@ -21,11 +21,13 @@ import requests
 import six
 from six.moves.urllib import parse as urlparse
 
+from heat.common import crypt
 from heat.common import exception
 from heat.common.i18n import _
 from heat.common.i18n import _LI
 from heat.db import api as db_api
 from heat.engine import api
+from heat.objects import resource as resource_objects
 from heat.objects import software_config as software_config_object
 from heat.objects import software_deployment as software_deployment_object
 from heat.rpc import api as rpc_api
@@ -78,14 +80,19 @@ class SoftwareConfigService(service.Service):
             raise ValueError(_('server_id must be specified'))
         all_sd = software_deployment_object.SoftwareDeployment.get_all(
             cnxt, server_id)
+
+        # filter out the sds with None config
+        flt_sd = six.moves.filterfalse(lambda sd: sd.config is None,
+                                       all_sd)
         # sort the configs by config name, to give the list of metadata a
         # deterministic and controllable order.
-        all_sd_s = sorted(all_sd, key=lambda sd: sd.config.name)
-        result = [api.format_software_config(sd.config) for sd in all_sd_s]
+        flt_sd_s = sorted(flt_sd, key=lambda sd: sd.config.name)
+        result = [api.format_software_config(sd.config) for sd in flt_sd_s]
         return result
 
-    @oslo_db_api.wrap_db_retry(max_retries=10, retry_on_request=True)
-    def _push_metadata_software_deployments(self, cnxt, server_id, sd):
+    @resource_objects.retry_on_conflict
+    def _push_metadata_software_deployments(
+            self, cnxt, server_id, stack_user_project_id):
         rs = db_api.resource_get_by_physical_resource_id(cnxt, server_id)
         if not rs:
             return
@@ -95,8 +102,8 @@ class SoftwareConfigService(service.Service):
         rows_updated = db_api.resource_update(
             cnxt, rs.id, {'rsrc_metadata': md}, rs.atomic_key)
         if not rows_updated:
-            raise db_exc.RetryRequest(
-                exception.DeploymentConcurrentTransaction(server=server_id))
+            action = _('deployments of server %s') % server_id
+            raise exception.ConcurrentTransaction(action=action)
 
         metadata_put_url = None
         metadata_queue_id = None
@@ -109,8 +116,10 @@ class SoftwareConfigService(service.Service):
             json_md = jsonutils.dumps(md)
             requests.put(metadata_put_url, json_md)
         if metadata_queue_id:
+            project = stack_user_project_id
+            token = self._get_user_token(cnxt, rs, project)
             zaqar_plugin = cnxt.clients.client_plugin('zaqar')
-            zaqar = zaqar_plugin.create_for_tenant(sd.stack_user_project_id)
+            zaqar = zaqar_plugin.create_for_tenant(project, token)
             queue = zaqar.queue(metadata_queue_id)
             queue.post({'body': md, 'ttl': zaqar_plugin.DEFAULT_TTL})
 
@@ -128,7 +137,7 @@ class SoftwareConfigService(service.Service):
                 LOG.info(_LI('Signal object not found: %(c)s %(o)s'), {
                     'c': container, 'o': object_name})
                 return sd
-            raise ex
+            raise
 
         lm = headers.get('last-modified')
 
@@ -151,7 +160,7 @@ class SoftwareConfigService(service.Service):
                     'Signal object not found: %(c)s %(o)s'), {
                         'c': container, 'o': object_name})
                 return sd
-            raise ex
+            raise
         if obj:
             self.signal_software_deployment(
                 cnxt, sd.id, jsonutils.loads(obj),
@@ -160,9 +169,23 @@ class SoftwareConfigService(service.Service):
         return software_deployment_object.SoftwareDeployment.get_by_id(
             cnxt, sd.id)
 
+    def _get_user_token(self, cnxt, rs, project):
+        user = password = None
+        for rd in rs.data:
+            if rd.key == 'password':
+                password = crypt.decrypt(rd.decrypt_method, rd.value)
+            if rd.key == 'user_id':
+                user = rd.value
+        keystone = cnxt.clients.client('keystone')
+        return keystone.stack_domain_user_token(
+            user_id=user, project_id=project, password=password)
+
     def _refresh_zaqar_software_deployment(self, cnxt, sd, deploy_queue_id):
+        rs = db_api.resource_get_by_physical_resource_id(cnxt, sd.server_id)
+        project = sd.stack_user_project_id
+        token = self._get_user_token(cnxt, rs, project)
         zaqar_plugin = cnxt.clients.client_plugin('zaqar')
-        zaqar = zaqar_plugin.create_for_tenant(sd.stack_user_project_id)
+        zaqar = zaqar_plugin.create_for_tenant(project, token)
         queue = zaqar.queue(deploy_queue_id)
 
         messages = list(queue.pop())
@@ -190,9 +213,13 @@ class SoftwareConfigService(service.Service):
 
     def create_software_deployment(self, cnxt, server_id, config_id,
                                    input_values, action, status,
-                                   status_reason, stack_user_project_id):
+                                   status_reason, stack_user_project_id,
+                                   deployment_id=None):
 
+        if deployment_id is None:
+            deployment_id = str(uuid.uuid4())
         sd = software_deployment_object.SoftwareDeployment.create(cnxt, {
+            'id': deployment_id,
             'config_id': config_id,
             'server_id': server_id,
             'input_values': input_values,
@@ -201,7 +228,8 @@ class SoftwareConfigService(service.Service):
             'action': action,
             'status': status,
             'status_reason': status_reason})
-        self._push_metadata_software_deployments(cnxt, server_id, sd)
+        self._push_metadata_software_deployments(
+            cnxt, server_id, stack_user_project_id)
         return api.format_software_deployment(sd)
 
     def signal_software_deployment(self, cnxt, deployment_id, details,
@@ -292,10 +320,15 @@ class SoftwareConfigService(service.Service):
         # only push metadata if this update resulted in the config_id
         # changing, since metadata is just a list of configs
         if config_id:
-            self._push_metadata_software_deployments(cnxt, sd.server_id, sd)
+            self._push_metadata_software_deployments(
+                cnxt, sd.server_id, sd.stack_user_project_id)
 
         return api.format_software_deployment(sd)
 
     def delete_software_deployment(self, cnxt, deployment_id):
+        sd = software_deployment_object.SoftwareDeployment.get_by_id(
+            cnxt, deployment_id)
         software_deployment_object.SoftwareDeployment.delete(
             cnxt, deployment_id)
+        self._push_metadata_software_deployments(
+            cnxt, sd.server_id, sd.stack_user_project_id)
