@@ -69,6 +69,7 @@ class StackLock(object):
             return
 
         stack = stack_object.Stack.get_by_id(self.context, self.stack_id,
+                                             tenant_safe=False,
                                              show_deleted=True)
         if (lock_engine_id == self.engine_id or
                 self.engine_alive(self.context, lock_engine_id)):
@@ -116,22 +117,33 @@ class StackLock(object):
         result = stack_lock_object.StackLock.release(self.stack_id,
                                                      self.engine_id)
         if result is True:
-            LOG.warn(_LW("Lock was already released on stack %s!"),
-                     self.stack_id)
+            LOG.warning(_LW("Lock was already released on stack %s!"),
+                        self.stack_id)
         else:
             LOG.debug("Engine %(engine)s released lock on stack "
                       "%(stack)s" % {'engine': self.engine_id,
                                      'stack': self.stack_id})
 
+    def __enter__(self):
+        self.acquire()
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.release()
+        return False
+
     @contextlib.contextmanager
-    def thread_lock(self):
+    def thread_lock(self, retry=True):
         """Acquire a lock and release it only if there is an exception.
 
         The release method still needs to be scheduled to be run at the
         end of the thread using the Thread.link method.
+
+        :param retry: When True, retry if lock was released while stealing.
+        :type retry: boolean
         """
         try:
-            self.acquire()
+            self.acquire(retry)
             yield
         except exception.ActionInProgress:
             raise
