@@ -13,19 +13,18 @@
 # under the License.
 
 
-"""
-ResourceData object
-"""
+"""ResourceData object."""
 
 from oslo_versionedobjects import base
 from oslo_versionedobjects import fields
 
 from heat.common import exception
 from heat.db import api as db_api
+from heat.objects import base as heat_base
 
 
 class ResourceData(
-    base.VersionedObject,
+    heat_base.HeatObject,
     base.VersionedObjectDictCompat,
     base.ComparableVersionedObject,
 ):
