@@ -12,19 +12,20 @@
 # under the License.
 
 
-"""
-Service object
-"""
+"""Service object."""
 
 from oslo_versionedobjects import base
 from oslo_versionedobjects import fields
 
 from heat.db import api as db_api
+from heat.objects import base as heat_base
 
 
-class Service(base.VersionedObject,
-              base.VersionedObjectDictCompat,
-              base.ComparableVersionedObject):
+class Service(
+        heat_base.HeatObject,
+        base.VersionedObjectDictCompat,
+        base.ComparableVersionedObject,
+):
     fields = {
         'id': fields.StringField(),
         'engine_id': fields.StringField(),
@@ -48,10 +49,8 @@ class Service(base.VersionedObject,
 
     @classmethod
     def _from_db_objects(cls, context, list_obj):
-        return map(lambda obj: cls._from_db_object(context,
-                                                   cls(context),
-                                                   obj),
-                   list_obj)
+        return [cls._from_db_object(context, cls(context), obj)
+                for obj in list_obj]
 
     @classmethod
     def get_by_id(cls, context, service_id):
