@@ -12,20 +12,22 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-"""
-WatchRule object
-"""
+"""WatchRule object."""
 
 from oslo_versionedobjects import base
 from oslo_versionedobjects import fields
 
 from heat.db import api as db_api
+from heat.objects import base as heat_base
 from heat.objects import fields as heat_fields
 from heat.objects import stack
 from heat.objects import watch_data
 
 
-class WatchRule(base.VersionedObject, base.VersionedObjectDictCompat):
+class WatchRule(
+        heat_base.HeatObject,
+        base.VersionedObjectDictCompat,
+):
 
     fields = {
         'id': fields.IntegerField(),
