@@ -13,16 +13,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""
-Client side of the heat engine RPC API.
-"""
+"""Client side of the heat engine RPC API."""
+
+from oslo_utils import reflection
 
 from heat.common import messaging
 from heat.rpc import api as rpc_api
 
 
 class EngineClient(object):
-    '''Client side of the heat engine rpc API.
+    """Client side of the heat engine rpc API.
 
     API version history::
 
@@ -39,7 +39,15 @@ class EngineClient(object):
         1.16 - Adds version, type_name to list_resource_types()
         1.17 - Add files to validate_template
         1.18 - Add show_nested to validate_template
-    '''
+        1.19 - Add show_output and list_outputs for returning stack outputs
+        1.20 - Add resolve_outputs to stack show
+        1.21 - Add deployment_id to create_software_deployment
+        1.22 - Add support for stack export
+        1.23 - Add environment_files to create/update/preview/validate
+        1.24 - Adds ignorable_errors to validate_template
+        1.25 - list_stack_resource filter update
+        1.26 - Add mark_unhealthy
+    """
 
     BASE_RPC_API_VERSION = '1.0'
 
@@ -69,17 +77,15 @@ class EngineClient(object):
         return client.cast(ctxt, method, **kwargs)
 
     def local_error_name(self, error):
-        """
-        Returns the name of the error with any _Remote postfix removed.
+        """Returns the name of the error with any _Remote postfix removed.
 
         :param error: Remote raised error to derive the name from.
         """
-        error_name = error.__class__.__name__
+        error_name = reflection.get_class_name(error, fully_qualified=False)
         return error_name.split('_Remote')[0]
 
     def ignore_error_named(self, error, name):
-        """
-        Raises the error unless its local name matches the supplied name
+        """Raises the error unless its local name matches the supplied name.
 
         :param error: Remote raised error to derive the local name from.
         :param name: Name to compare local name to.
@@ -88,9 +94,7 @@ class EngineClient(object):
             raise error
 
     def identify_stack(self, ctxt, stack_name):
-        """
-        The identify_stack method returns the full stack identifier for a
-        single, live stack given the stack name.
+        """Returns the full stack identifier for a single, live stack.
 
         :param ctxt: RPC context.
         :param stack_name: Name of the stack you want to see,
@@ -104,10 +108,11 @@ class EngineClient(object):
                     show_deleted=False, show_nested=False, show_hidden=False,
                     tags=None, tags_any=None, not_tags=None,
                     not_tags_any=None):
-        """
-        The list_stacks method returns attributes of all stacks.  It supports
-        pagination (``limit`` and ``marker``), sorting (``sort_keys`` and
-        ``sort_dir``) and filtering (``filters``) of the results.
+        """Returns attributes of all stacks.
+
+        It supports pagination (``limit`` and ``marker``), sorting
+        (``sort_keys`` and ``sort_dir``) and filtering (``filters``) of the
+        results.
 
         :param ctxt: RPC context.
         :param limit: the number of stacks to list (integer or string)
@@ -146,8 +151,8 @@ class EngineClient(object):
                      show_deleted=False, show_nested=False, show_hidden=False,
                      tags=None, tags_any=None, not_tags=None,
                      not_tags_any=None):
-        """
-        Return the number of stacks that match the given filters
+        """Returns the number of stacks that match the given filters.
+
         :param ctxt: RPC context.
         :param filters: a dict of ATTR:VALUE to match against stacks
         :param tenant_safe: if true, scope the request by the current tenant
@@ -162,7 +167,7 @@ class EngineClient(object):
             multiple tags using the boolean AND expression
         :param not_tags_any: count stacks not containing these tags, combine
             multiple tags using the boolean OR expression
-        :returns: a integer representing the number of matched stacks
+        :returns: an integer representing the number of matched stacks
         """
         return self.call(ctxt, self.make_msg('count_stacks',
                                              filters=filters,
@@ -176,19 +181,22 @@ class EngineClient(object):
                                              not_tags_any=not_tags_any),
                          version='1.8')
 
-    def show_stack(self, ctxt, stack_identity):
-        """
-        Return detailed information about one or all stacks.
+    def show_stack(self, ctxt, stack_identity, resolve_outputs=True):
+        """Returns detailed information about one or all stacks.
+
         :param ctxt: RPC context.
         :param stack_identity: Name of the stack you want to show, or None to
         show all
+        :param resolve_outputs: If True, stack outputs will be resolved
         """
         return self.call(ctxt, self.make_msg('show_stack',
-                                             stack_identity=stack_identity))
+                                             stack_identity=stack_identity,
+                                             resolve_outputs=resolve_outputs),
+                         version='1.20')
 
-    def preview_stack(self, ctxt, stack_name, template, params, files, args):
-        """
-        Simulates a new stack using the provided template.
+    def preview_stack(self, ctxt, stack_name, template, params, files,
+                      args, environment_files=None):
+        """Simulates a new stack using the provided template.
 
         Note that at this stage the template has already been fetched from the
         heat-api process if using a template-url.
@@ -199,16 +207,22 @@ class EngineClient(object):
         :param params: Stack Input Params/Environment
         :param files: files referenced from the environment.
         :param args: Request parameters/args passed from API
+        :param environment_files: optional ordered list of environment file
+               names included in the files dict
+        :type  environment_files: list or None
         """
         return self.call(ctxt,
                          self.make_msg('preview_stack', stack_name=stack_name,
                                        template=template,
-                                       params=params, files=files, args=args))
+                                       params=params, files=files,
+                                       environment_files=environment_files,
+                                       args=args),
+                         version='1.23')
+
+    def create_stack(self, ctxt, stack_name, template, params, files,
+                     args, environment_files=None):
+        """Creates a new stack using the template provided.
 
-    def create_stack(self, ctxt, stack_name, template, params, files, args):
-        """
-        The create_stack method creates a new stack using the template
-        provided.
         Note that at this stage the template has already been fetched from the
         heat-api process if using a template-url.
 
@@ -218,17 +232,22 @@ class EngineClient(object):
         :param params: Stack Input Params/Environment
         :param files: files referenced from the environment.
         :param args: Request parameters/args passed from API
+        :param environment_files: optional ordered list of environment file
+               names included in the files dict
+        :type  environment_files: list or None
         """
         return self._create_stack(ctxt, stack_name, template, params, files,
-                                  args)
+                                  args, environment_files=environment_files)
 
-    def _create_stack(self, ctxt, stack_name, template, params, files, args,
+    def _create_stack(self, ctxt, stack_name, template, params, files,
+                      args, environment_files=None,
                       owner_id=None, nested_depth=0, user_creds_id=None,
                       stack_user_project_id=None, parent_resource_name=None):
-        """
-        Internal create_stack interface for engine-to-engine communication via
-        RPC.  Allows some additional options which should not be exposed to
-        users via the API:
+        """Internal interface for engine-to-engine communication via RPC.
+
+        Allows some additional options which should not be exposed to users via
+        the API:
+
         :param owner_id: parent stack ID for nested stacks
         :param nested_depth: nested depth for nested stacks
         :param user_creds_id: user_creds record for nested stack
@@ -238,19 +257,19 @@ class EngineClient(object):
         return self.call(
             ctxt, self.make_msg('create_stack', stack_name=stack_name,
                                 template=template,
-                                params=params, files=files, args=args,
-                                owner_id=owner_id,
+                                params=params, files=files,
+                                environment_files=environment_files,
+                                args=args, owner_id=owner_id,
                                 nested_depth=nested_depth,
                                 user_creds_id=user_creds_id,
                                 stack_user_project_id=stack_user_project_id,
                                 parent_resource_name=parent_resource_name),
-            version='1.8')
+            version='1.23')
 
     def update_stack(self, ctxt, stack_identity, template, params,
-                     files, args):
-        """
-        The update_stack method updates an existing stack based on the
-        provided template and parameters.
+                     files, args, environment_files=None):
+        """Updates an existing stack based on the provided template and params.
+
         Note that at this stage the template has already been fetched from the
         heat-api process if using a template-url.
 
@@ -260,20 +279,25 @@ class EngineClient(object):
         :param params: Stack Input Params/Environment
         :param files: files referenced from the environment.
         :param args: Request parameters/args passed from API
+        :param environment_files: optional ordered list of environment file
+               names included in the files dict
+        :type  environment_files: list or None
         """
-        return self.call(ctxt, self.make_msg('update_stack',
-                                             stack_identity=stack_identity,
-                                             template=template,
-                                             params=params,
-                                             files=files,
-                                             args=args))
+        return self.call(ctxt,
+                         self.make_msg('update_stack',
+                                       stack_identity=stack_identity,
+                                       template=template,
+                                       params=params,
+                                       files=files,
+                                       environment_files=environment_files,
+                                       args=args),
+                         version='1.23')
 
     def preview_update_stack(self, ctxt, stack_identity, template, params,
-                             files, args):
-        """
-        The preview_update_stack method returns the resources that would be
-        changed in an update of an existing stack based on the provided
-        template and parameters.
+                             files, args, environment_files=None):
+        """Returns the resources that would be changed in an update.
+
+        Based on the provided template and parameters.
 
         Requires RPC version 1.15 or above.
 
@@ -283,6 +307,9 @@ class EngineClient(object):
         :param params: Stack Input Params/Environment
         :param files: files referenced from the environment.
         :param args: Request parameters/args passed from API
+        :param environment_files: optional ordered list of environment file
+               names included in the files dict
+        :type  environment_files: list or None
         """
         return self.call(ctxt,
                          self.make_msg('preview_update_stack',
@@ -290,31 +317,39 @@ class EngineClient(object):
                                        template=template,
                                        params=params,
                                        files=files,
+                                       environment_files=environment_files,
                                        args=args,
                                        ),
-                         version='1.15')
+                         version='1.23')
 
     def validate_template(self, ctxt, template, params=None, files=None,
-                          show_nested=False):
-        """
-        The validate_template method uses the stack parser to check
-        the validity of a template.
+                          environment_files=None, show_nested=False,
+                          ignorable_errors=None):
+        """Uses the stack parser to check the validity of a template.
 
         :param ctxt: RPC context.
         :param template: Template of stack you want to create.
         :param params: Stack Input Params/Environment
         :param files: files referenced from the environment/template.
+        :param environment_files: ordered list of environment file names
+               included in the files dict
         :param show_nested: if True nested templates will be validated
+        :param ignorable_errors: List of error_code to be ignored as part of
+        validation
         """
-        return self.call(ctxt, self.make_msg('validate_template',
-                                             template=template,
-                                             params=params,
-                                             files=files,
-                                             show_nested=show_nested),
-                         version='1.18')
+        return self.call(ctxt, self.make_msg(
+            'validate_template',
+            template=template,
+            params=params,
+            files=files,
+            show_nested=show_nested,
+            environment_files=environment_files,
+            ignorable_errors=ignorable_errors),
+            version='1.24')
 
     def authenticated_to_backend(self, ctxt):
-        """
+        """Validate the credentials in the RPC context.
+
         Verify that the credentials in the RPC context are valid for the
         current cloud backend.
 
@@ -323,8 +358,7 @@ class EngineClient(object):
         return self.call(ctxt, self.make_msg('authenticated_to_backend'))
 
     def get_template(self, ctxt, stack_identity):
-        """
-        Get the template.
+        """Get the template.
 
         :param ctxt: RPC context.
         :param stack_name: Name of the stack you want to see.
@@ -333,8 +367,7 @@ class EngineClient(object):
                                              stack_identity=stack_identity))
 
     def delete_stack(self, ctxt, stack_identity, cast=True):
-        """
-        The delete_stack method deletes a given stack.
+        """Deletes a given stack.
 
         :param ctxt: RPC context.
         :param stack_identity: Name of the stack you want to delete.
@@ -346,9 +379,7 @@ class EngineClient(object):
                                         stack_identity=stack_identity))
 
     def abandon_stack(self, ctxt, stack_identity):
-        """
-        The abandon_stack method deletes a given stack but
-        resources would not be deleted.
+        """Deletes a given stack but resources would not be deleted.
 
         :param ctxt: RPC context.
         :param stack_identity: Name of the stack you want to abandon.
@@ -362,8 +393,7 @@ class EngineClient(object):
                             support_status=None,
                             type_name=None,
                             heat_version=None):
-        """
-        Get a list of valid resource types.
+        """Get a list of valid resource types.
 
         :param ctxt: RPC context.
         :param support_status: Support status of resource type
@@ -377,8 +407,7 @@ class EngineClient(object):
                          version='1.16')
 
     def list_template_versions(self, ctxt):
-        """
-        Get a list of available template versions
+        """Get a list of available template versions.
 
         :param ctxt: RPC context.
         """
@@ -386,8 +415,7 @@ class EngineClient(object):
                          version='1.11')
 
     def list_template_functions(self, ctxt, template_version):
-        """
-        Get a list of available functions in a given template
+        """Get a list of available functions in a given template.
 
         :param ctxt: RPC context
         :param template_name : name of the template which function list you
@@ -398,8 +426,7 @@ class EngineClient(object):
             version='1.13')
 
     def resource_schema(self, ctxt, type_name):
-        """
-        Get the schema for a resource type.
+        """Get the schema for a resource type.
 
         :param ctxt: RPC context.
         """
@@ -407,8 +434,7 @@ class EngineClient(object):
                                              type_name=type_name))
 
     def generate_template(self, ctxt, type_name, template_type='cfn'):
-        """
-        Generate a template based on the specified type.
+        """Generate a template based on the specified type.
 
         :param ctxt: RPC context.
         :param type_name: The resource type name to generate a template for.
@@ -421,8 +447,8 @@ class EngineClient(object):
 
     def list_events(self, ctxt, stack_identity, filters=None, limit=None,
                     marker=None, sort_keys=None, sort_dir=None,):
-        """
-        The list_events method lists all events associated with a given stack.
+        """Lists all events associated with a given stack.
+
         It supports pagination (``limit`` and ``marker``),
         sorting (``sort_keys`` and ``sort_dir``) and filtering(filters)
         of the results.
@@ -445,8 +471,8 @@ class EngineClient(object):
 
     def describe_stack_resource(self, ctxt, stack_identity, resource_name,
                                 with_attr=False):
-        """
-        Get detailed resource information about a particular resource.
+        """Get detailed resource information about a particular resource.
+
         :param ctxt: RPC context.
         :param stack_identity: Name of the stack.
         :param resource_name: the Resource.
@@ -459,11 +485,10 @@ class EngineClient(object):
                          version='1.2')
 
     def find_physical_resource(self, ctxt, physical_resource_id):
-        """
-        Return an identifier for the resource with the specified physical
-        resource ID.
-        :param ctxt RPC context.
-        :param physcial_resource_id The physical resource ID to look up.
+        """Return an identifier for the resource.
+
+        :param ctxt: RPC context.
+        :param physcial_resource_id: The physical resource ID to look up.
         """
         return self.call(ctxt,
                          self.make_msg(
@@ -471,8 +496,8 @@ class EngineClient(object):
                              physical_resource_id=physical_resource_id))
 
     def describe_stack_resources(self, ctxt, stack_identity, resource_name):
-        """
-        Get detailed resource information about one or more resources.
+        """Get detailed resource information about one or more resources.
+
         :param ctxt: RPC context.
         :param stack_identity: Name of the stack.
         :param resource_name: the Resource.
@@ -482,20 +507,23 @@ class EngineClient(object):
                                              resource_name=resource_name))
 
     def list_stack_resources(self, ctxt, stack_identity,
-                             nested_depth=0, with_detail=False):
-        """
-        List the resources belonging to a stack.
+                             nested_depth=0, with_detail=False,
+                             filters=None):
+        """List the resources belonging to a stack.
+
         :param ctxt: RPC context.
         :param stack_identity: Name of the stack.
         :param nested_depth: Levels of nested stacks of which list resources.
-        :param with_detail: show detail for resoruces in list.
+        :param with_detail: show detail for resources in list.
+        :param filters: a dict with attribute:value to search the resources
         """
         return self.call(ctxt,
                          self.make_msg('list_stack_resources',
                                        stack_identity=stack_identity,
                                        nested_depth=nested_depth,
-                                       with_detail=with_detail),
-                         version='1.12')
+                                       with_detail=with_detail,
+                                       filters=filters),
+                         version='1.25')
 
     def stack_suspend(self, ctxt, stack_identity):
         return self.call(ctxt, self.make_msg('stack_suspend',
@@ -520,8 +548,8 @@ class EngineClient(object):
 
     def resource_signal(self, ctxt, stack_identity, resource_name, details,
                         sync_call=False):
-        """
-        Generate an alarm on the resource.
+        """Generate an alarm on the resource.
+
         :param ctxt: RPC context.
         :param stack_identity: Name of the stack.
         :param resource_name: the Resource.
@@ -535,22 +563,44 @@ class EngineClient(object):
 
                          version='1.3')
 
+    def resource_mark_unhealthy(self, ctxt, stack_identity, resource_name,
+                                mark_unhealthy, resource_status_reason=None):
+        """Mark the resource as unhealthy or healthy.
+
+        :param ctxt: RPC context.
+        :param stack_identity: Name of the stack.
+        :param resource_name: the Resource.
+        :param mark_unhealthy: indicates whether the resource is unhealthy.
+        :param resource_status_reason: reason for health change.
+        """
+        return self.call(
+            ctxt,
+            self.make_msg('resource_mark_unhealthy',
+                          stack_identity=stack_identity,
+                          resource_name=resource_name,
+                          mark_unhealthy=mark_unhealthy,
+                          resource_status_reason=resource_status_reason),
+            version='1.26')
+
     def create_watch_data(self, ctxt, watch_name, stats_data):
-        '''
-        This could be used by CloudWatch and WaitConditions
-        and treat HA service events like any other CloudWatch.
+        """Creates data for CloudWatch and WaitConditions.
+
+        This could be used by CloudWatch and WaitConditions and treat HA
+        service events like any other CloudWatch.
+
         :param ctxt: RPC context.
         :param watch_name: Name of the watch/alarm
         :param stats_data: The data to post.
-        '''
+        """
         return self.call(ctxt, self.make_msg('create_watch_data',
                                              watch_name=watch_name,
                                              stats_data=stats_data))
 
     def show_watch(self, ctxt, watch_name):
-        """
+        """Returns the attributes of one watch/alarm.
+
         The show_watch method returns the attributes of one watch
-        or all watches if no watch_name is passed
+        or all watches if no watch_name is passed.
 
         :param ctxt: RPC context.
         :param watch_name: Name of the watch/alarm you want to see,
@@ -560,9 +610,10 @@ class EngineClient(object):
                                              watch_name=watch_name))
 
     def show_watch_metric(self, ctxt, metric_namespace=None, metric_name=None):
-        """
+        """Returns the datapoints for a metric.
+
         The show_watch_metric method returns the datapoints associated
-        with a specified metric, or all metrics if no metric_name is passed
+        with a specified metric, or all metrics if no metric_name is passed.
 
         :param ctxt: RPC context.
         :param metric_namespace: Name of the namespace you want to see,
@@ -575,12 +626,12 @@ class EngineClient(object):
                                              metric_name=metric_name))
 
     def set_watch_state(self, ctxt, watch_name, state):
-        '''
-        Temporarily set the state of a given watch
+        """Temporarily set the state of a given watch.
+
         :param ctxt: RPC context.
         :param watch_name: Name of the watch
         :param state: State (must be one defined in WatchRule class)
-        '''
+        """
         return self.call(ctxt, self.make_msg('set_watch_state',
                                              watch_name=watch_name,
                                              state=state))
@@ -633,12 +684,14 @@ class EngineClient(object):
     def create_software_deployment(self, cnxt, server_id, config_id=None,
                                    input_values=None, action='INIT',
                                    status='COMPLETE', status_reason='',
-                                   stack_user_project_id=None):
+                                   stack_user_project_id=None,
+                                   deployment_id=None):
         input_values = input_values or {}
         return self.call(cnxt, self.make_msg(
             'create_software_deployment',
             server_id=server_id,
             config_id=config_id,
+            deployment_id=deployment_id,
             input_values=input_values,
             action=action,
             status=status,
@@ -701,3 +754,25 @@ class EngineClient(object):
 
     def list_services(self, cnxt):
         return self.call(cnxt, self.make_msg('list_services'), version='1.4')
+
+    def list_outputs(self, cntx, stack_identity):
+        return self.call(cntx, self.make_msg('list_outputs',
+                                             stack_identity=stack_identity),
+                         version='1.19')
+
+    def show_output(self, cntx, stack_identity, output_key):
+        return self.call(cntx, self.make_msg('show_output',
+                                             stack_identity=stack_identity,
+                                             output_key=output_key),
+                         version='1.19')
+
+    def export_stack(self, ctxt, stack_identity):
+        """Exports the stack data in JSON format.
+
+        :param ctxt: RPC context.
+        :param stack_identity: Name of the stack you want to export.
+        """
+        return self.call(ctxt,
+                         self.make_msg('export_stack',
+                                       stack_identity=stack_identity),
+                         version='1.22')
