@@ -66,3 +66,9 @@ class CooldownMixin(object):
             metadata['cooldown'] = {now: cooldown_reason}
         metadata['scaling_in_progress'] = False
         self.metadata_set(metadata)
+
+    def handle_metadata_reset(self):
+        metadata = self.metadata_get()
+        if 'scaling_in_progress' in metadata:
+            metadata['scaling_in_progress'] = False
+            self.metadata_set(metadata)
