@@ -33,7 +33,8 @@ policy_path = os.path.dirname(os.path.realpath(__file__)) + "/../../policy/"
 
 
 class CfnStackControllerTest(common.HeatTestCase):
-    """
+    """Tests the API class CfnStackController.
+
     Tests the API class which acts as the WSGI controller,
     the endpoint processing API requests after they are routed
     """
@@ -214,7 +215,9 @@ class CfnStackControllerTest(common.HeatTestCase):
 
         self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
         rpc_client.EngineClient.call(
-            dummy_req.context, ('show_stack', {'stack_identity': None})
+            dummy_req.context, ('show_stack', {'stack_identity': None,
+                                               'resolve_outputs': True}),
+            version='1.20'
         ).AndReturn(engine_resp)
 
         self.m.ReplayAll()
@@ -236,7 +239,9 @@ class CfnStackControllerTest(common.HeatTestCase):
 
         self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
         rpc_client.EngineClient.call(
-            dummy_req.context, ('show_stack', {'stack_identity': None})
+            dummy_req.context, ('show_stack', {'stack_identity': None,
+                                               'resolve_outputs': True}),
+            version='1.20'
         ).AndReturn(engine_resp)
 
         self.m.ReplayAll()
@@ -291,7 +296,9 @@ class CfnStackControllerTest(common.HeatTestCase):
         ).AndReturn(identity)
         rpc_client.EngineClient.call(
             dummy_req.context,
-            ('show_stack', {'stack_identity': identity})
+            ('show_stack', {'stack_identity': identity,
+                            'resolve_outputs': True}),
+            version='1.20'
         ).AndReturn(engine_resp)
 
         self.m.ReplayAll()
@@ -306,18 +313,18 @@ class CfnStackControllerTest(common.HeatTestCase):
                         'StackStatusReason': u'Stack successfully created',
                         'Description': u'blah',
                         'Parameters':
-                        [{'ParameterValue': u'admin',
-                          'ParameterKey': u'DBUsername'},
-                         {'ParameterValue': u'F17',
-                          'ParameterKey': u'LinuxDistribution'},
-                         {'ParameterValue': u'm1.large',
-                          'ParameterKey': u'InstanceType'},
+                        [{'ParameterValue': u'wordpress',
+                          'ParameterKey': u'DBName'},
+                         {'ParameterValue': u'admin',
+                          'ParameterKey': u'DBPassword'},
                          {'ParameterValue': u'admin',
                           'ParameterKey': u'DBRootPassword'},
                          {'ParameterValue': u'admin',
-                          'ParameterKey': u'DBPassword'},
-                         {'ParameterValue': u'wordpress',
-                          'ParameterKey': u'DBName'}],
+                          'ParameterKey': u'DBUsername'},
+                         {'ParameterValue': u'm1.large',
+                          'ParameterKey': u'InstanceType'},
+                         {'ParameterValue': u'F17',
+                          'ParameterKey': u'LinuxDistribution'}],
                         'Outputs':
                         [{'OutputKey': u'WebsiteURL',
                           'OutputValue': u'http://10.0.0.8/wordpress',
@@ -330,9 +337,13 @@ class CfnStackControllerTest(common.HeatTestCase):
                         'StackStatus': u'CREATE_COMPLETE',
                         'DisableRollback': 'true',
                         'LastUpdatedTime': u'2012-07-09T09:13:11Z'}]}}}
-
-        self.assertEqual(utils.recursive_sort(expected),
-                         utils.recursive_sort(response))
+        stacks = (response['DescribeStacksResponse']['DescribeStacksResult']
+                  ['Stacks'])
+        stacks[0]['Parameters'] = sorted(
+            stacks[0]['Parameters'], key=lambda k: k['ParameterKey'])
+        response['DescribeStacksResponse']['DescribeStacksResult'] = (
+            {'Stacks': stacks})
+        self.assertEqual(expected, response)
 
     def test_describe_arn(self):
         # Format a dummy GET request to pass into the WSGI handler
@@ -376,7 +387,9 @@ class CfnStackControllerTest(common.HeatTestCase):
         self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
         rpc_client.EngineClient.call(
             dummy_req.context,
-            ('show_stack', {'stack_identity': identity})
+            ('show_stack', {'stack_identity': identity,
+                            'resolve_outputs': True}),
+            version='1.20'
         ).AndReturn(engine_resp)
 
         self.m.ReplayAll()
@@ -391,18 +404,18 @@ class CfnStackControllerTest(common.HeatTestCase):
                         'StackStatusReason': u'Stack successfully created',
                         'Description': u'blah',
                         'Parameters':
-                        [{'ParameterValue': u'admin',
-                          'ParameterKey': u'DBUsername'},
-                         {'ParameterValue': u'F17',
-                          'ParameterKey': u'LinuxDistribution'},
-                         {'ParameterValue': u'm1.large',
-                          'ParameterKey': u'InstanceType'},
+                        [{'ParameterValue': u'wordpress',
+                          'ParameterKey': u'DBName'},
+                         {'ParameterValue': u'admin',
+                          'ParameterKey': u'DBPassword'},
                          {'ParameterValue': u'admin',
                           'ParameterKey': u'DBRootPassword'},
                          {'ParameterValue': u'admin',
-                          'ParameterKey': u'DBPassword'},
-                         {'ParameterValue': u'wordpress',
-                          'ParameterKey': u'DBName'}],
+                          'ParameterKey': u'DBUsername'},
+                         {'ParameterValue': u'm1.large',
+                          'ParameterKey': u'InstanceType'},
+                         {'ParameterValue': u'F17',
+                          'ParameterKey': u'LinuxDistribution'}],
                         'Outputs':
                         [{'OutputKey': u'WebsiteURL',
                           'OutputValue': u'http://10.0.0.8/wordpress',
@@ -415,9 +428,13 @@ class CfnStackControllerTest(common.HeatTestCase):
                         'StackStatus': u'CREATE_COMPLETE',
                         'DisableRollback': 'true',
                         'LastUpdatedTime': u'2012-07-09T09:13:11Z'}]}}}
-
-        self.assertEqual(utils.recursive_sort(expected),
-                         utils.recursive_sort(response))
+        stacks = (response['DescribeStacksResponse']['DescribeStacksResult']
+                  ['Stacks'])
+        stacks[0]['Parameters'] = sorted(
+            stacks[0]['Parameters'], key=lambda k: k['ParameterKey'])
+        response['DescribeStacksResponse']['DescribeStacksResult'] = (
+            {'Stacks': stacks})
+        self.assertEqual(expected, response)
 
     def test_describe_arn_invalidtenant(self):
         # Format a dummy GET request to pass into the WSGI handler
@@ -431,7 +448,9 @@ class CfnStackControllerTest(common.HeatTestCase):
 
         self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
         rpc_client.EngineClient.call(
-            dummy_req.context, ('show_stack', {'stack_identity': identity})
+            dummy_req.context, ('show_stack', {'stack_identity': identity,
+                                               'resolve_outputs': True},),
+            version='1.20'
         ).AndRaise(heat_exception.InvalidTenant(target='test',
                                                 actual='test'))
 
@@ -454,7 +473,9 @@ class CfnStackControllerTest(common.HeatTestCase):
             dummy_req.context, ('identify_stack', {'stack_name': stack_name})
         ).AndReturn(identity)
         rpc_client.EngineClient.call(
-            dummy_req.context, ('show_stack', {'stack_identity': identity})
+            dummy_req.context, ('show_stack', {'stack_identity': identity,
+                                               'resolve_outputs': True}),
+            version='1.20'
         ).AndRaise(AttributeError())
 
         self.m.ReplayAll()
@@ -473,7 +494,7 @@ class CfnStackControllerTest(common.HeatTestCase):
         self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
         rpc_client.EngineClient.call(
             dummy_req.context, ('identify_stack', {'stack_name': stack_name})
-        ).AndRaise(heat_exception.StackNotFound(stack_name='test'))
+        ).AndRaise(heat_exception.EntityNotFound(entity='Stack', name='test'))
 
         self.m.ReplayAll()
 
@@ -508,13 +529,14 @@ class CfnStackControllerTest(common.HeatTestCase):
               'template': self.template,
               'params': engine_parms,
               'files': {},
+              'environment_files': None,
               'args': engine_args,
               'owner_id': None,
               'nested_depth': 0,
               'user_creds_id': None,
               'parent_resource_name': None,
               'stack_user_project_id': None}),
-            version='1.8'
+            version='1.23'
         ).AndRaise(failure)
 
     def _stub_rpc_create_stack_call_success(self, stack_name, engine_parms,
@@ -536,13 +558,14 @@ class CfnStackControllerTest(common.HeatTestCase):
               'template': self.template,
               'params': engine_parms,
               'files': {},
+              'environment_files': None,
               'args': engine_args,
               'owner_id': None,
               'nested_depth': 0,
               'user_creds_id': None,
               'parent_resource_name': None,
               'stack_user_project_id': None}),
-            version='1.8'
+            version='1.23'
         ).AndReturn(engine_resp)
 
         self.m.ReplayAll()
@@ -850,7 +873,9 @@ class CfnStackControllerTest(common.HeatTestCase):
               'template': self.template,
               'params': engine_parms,
               'files': {},
-              'args': engine_args})
+              'environment_files': None,
+              'args': engine_args}),
+            version='1.23'
         ).AndReturn(identity)
 
         self.m.ReplayAll()
@@ -919,7 +944,7 @@ class CfnStackControllerTest(common.HeatTestCase):
         rpc_client.EngineClient.call(
             dummy_req.context,
             ('identify_stack', {'stack_name': stack_name})
-        ).AndRaise(heat_exception.StackNotFound(stack_name='test'))
+        ).AndRaise(heat_exception.EntityNotFound(entity='Stack', name='test'))
 
         self.m.ReplayAll()
 
@@ -996,7 +1021,7 @@ class CfnStackControllerTest(common.HeatTestCase):
         rpc_client.EngineClient.call(
             dummy_req.context,
             ('identify_stack', {'stack_name': stack_name})
-        ).AndRaise(heat_exception.StackNotFound(stack_name='test'))
+        ).AndRaise(heat_exception.EntityNotFound(entity='Stack', name='test'))
 
         self.m.ReplayAll()
 
@@ -1068,8 +1093,10 @@ class CfnStackControllerTest(common.HeatTestCase):
         rpc_client.EngineClient.call(
             dummy_req.context,
             ('validate_template', {'template': json_template, 'params': None,
-                                   'files': None, 'show_nested': False}),
-            version='1.18'
+                                   'files': None, 'environment_files': None,
+                                   'show_nested': False,
+                                   'ignorable_errors': None}),
+            version='1.24'
         ).AndReturn(response)
         self.m.ReplayAll()
 
@@ -1144,7 +1171,7 @@ class CfnStackControllerTest(common.HeatTestCase):
         self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
         rpc_client.EngineClient.call(
             dummy_req.context, ('identify_stack', {'stack_name': stack_name})
-        ).AndRaise(heat_exception.StackNotFound(stack_name='test'))
+        ).AndRaise(heat_exception.EntityNotFound(entity='Stack', name='test'))
 
         self.m.ReplayAll()
 
@@ -1252,7 +1279,7 @@ class CfnStackControllerTest(common.HeatTestCase):
         self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
         rpc_client.EngineClient.call(
             dummy_req.context, ('identify_stack', {'stack_name': stack_name})
-        ).AndRaise(heat_exception.StackNotFound(stack_name='test'))
+        ).AndRaise(heat_exception.EntityNotFound(entity='Stack', name='test'))
 
         self.m.ReplayAll()
 
@@ -1339,7 +1366,7 @@ class CfnStackControllerTest(common.HeatTestCase):
         self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
         rpc_client.EngineClient.call(
             dummy_req.context, ('identify_stack', {'stack_name': stack_name})
-        ).AndRaise(heat_exception.StackNotFound(stack_name='test'))
+        ).AndRaise(heat_exception.EntityNotFound(entity='Stack', name='test'))
 
         self.m.ReplayAll()
 
@@ -1454,7 +1481,7 @@ class CfnStackControllerTest(common.HeatTestCase):
         self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
         rpc_client.EngineClient.call(
             dummy_req.context, ('identify_stack', {'stack_name': stack_name})
-        ).AndRaise(heat_exception.StackNotFound(stack_name='test'))
+        ).AndRaise(heat_exception.EntityNotFound(entity='Stack', name='test'))
 
         self.m.ReplayAll()
 
@@ -1542,8 +1569,7 @@ class CfnStackControllerTest(common.HeatTestCase):
             dummy_req.context,
             ('find_physical_resource',
              {'physical_resource_id': 'aaaaaaaa-9f88-404d-cccc-ffffffffffff'})
-        ).AndRaise(heat_exception.PhysicalResourceNotFound(
-            resource_id='1'))
+        ).AndRaise(heat_exception.EntityNotFound(entity='Resource', name='1'))
 
         self.m.ReplayAll()
 
@@ -1603,8 +1629,9 @@ class CfnStackControllerTest(common.HeatTestCase):
             dummy_req.context,
             ('list_stack_resources', {'stack_identity': identity,
                                       'nested_depth': 0,
-                                      'with_detail': False}),
-            version='1.12'
+                                      'with_detail': False,
+                                      'filters': None}),
+            version='1.25'
         ).AndReturn(engine_resp)
 
         self.m.ReplayAll()
@@ -1635,7 +1662,7 @@ class CfnStackControllerTest(common.HeatTestCase):
         self.m.StubOutWithMock(rpc_client.EngineClient, 'call')
         rpc_client.EngineClient.call(
             dummy_req.context, ('identify_stack', {'stack_name': stack_name})
-        ).AndRaise(heat_exception.StackNotFound(stack_name='test'))
+        ).AndRaise(heat_exception.EntityNotFound(entity='Stack', name='test'))
 
         self.m.ReplayAll()
 
