@@ -14,7 +14,6 @@
 import copy
 import uuid
 
-from glanceclient import exc as glance_exceptions
 import mock
 import mox
 from neutronclient.v2_0 import client as neutronclient
@@ -34,10 +33,9 @@ from heat.engine import scheduler
 from heat.engine import stack as parser
 from heat.engine import template
 from heat.tests import common
-from heat.tests.nova import fakes as fakes_nova
+from heat.tests.openstack.nova import fakes as fakes_nova
 from heat.tests import utils
 
-
 wp_template = '''
 {
   "AWSTemplateFormatVersion" : "2010-09-09",
@@ -97,13 +95,16 @@ class InstancesTest(common.HeatTestCase):
         return (tmpl, stack)
 
     def _mock_get_image_id_success(self, imageId_input, imageId):
-        self.m.StubOutWithMock(glance.GlanceClientPlugin, 'get_image_id')
-        glance.GlanceClientPlugin.get_image_id(
+        self.m.StubOutWithMock(glance.GlanceClientPlugin,
+                               'find_image_by_name_or_id')
+        glance.GlanceClientPlugin.find_image_by_name_or_id(
             imageId_input).MultipleTimes().AndReturn(imageId)
 
     def _mock_get_image_id_fail(self, image_id, exp):
-        self.m.StubOutWithMock(glance.GlanceClientPlugin, 'get_image_id')
-        glance.GlanceClientPlugin.get_image_id(image_id).AndRaise(exp)
+        self.m.StubOutWithMock(glance.GlanceClientPlugin,
+                               'find_image_by_name_or_id')
+        glance.GlanceClientPlugin.find_image_by_name_or_id(
+            image_id).AndRaise(exp)
 
     def _get_test_template(self, stack_name, image_id=None, volumes=False):
         (tmpl, stack) = self._setup_test_stack(stack_name)
@@ -408,8 +409,7 @@ class InstancesTest(common.HeatTestCase):
                                       resource_defns['WebServer'], stack)
 
         self._mock_get_image_id_fail('Slackware',
-                                     exception.EntityNotFound(
-                                         entity='Image', name='Slackware'))
+                                     glance.exceptions.NotFound())
         self.stub_VolumeConstraint_validate()
         self.stub_FlavorConstraint_validate()
         self.stub_KeypairConstraint_validate()
@@ -420,9 +420,8 @@ class InstancesTest(common.HeatTestCase):
         error = self.assertRaises(exception.ResourceFailure, create)
         self.assertEqual(
             "StackValidationFailed: resources.instance_create_image_err: "
-            "Property error: "
-            "WebServer.Properties.ImageId: Error validating value "
-            "'Slackware': The Image (Slackware) could not be found.",
+            "Property error: WebServer.Properties.ImageId: "
+            "Error validating value 'Slackware': Not Found (HTTP 404)",
             six.text_type(error))
 
         self.m.VerifyAll()
@@ -439,8 +438,7 @@ class InstancesTest(common.HeatTestCase):
                                       resource_defns['WebServer'], stack)
 
         self._mock_get_image_id_fail('CentOS 5.2',
-                                     exception.PhysicalResourceNameAmbiguity(
-                                         name='CentOS 5.2'))
+                                     glance.exceptions.NoUniqueMatch())
 
         self.stub_KeypairConstraint_validate()
         self.stub_SnapshotConstraint_validate()
@@ -451,10 +449,9 @@ class InstancesTest(common.HeatTestCase):
         create = scheduler.TaskRunner(instance.create)
         error = self.assertRaises(exception.ResourceFailure, create)
         self.assertEqual(
-            'StackValidationFailed: resources.instance_create_image_err: '
-            'Property error: '
-            'WebServer.Properties.ImageId: Multiple physical '
-            'resources were found with name (CentOS 5.2).',
+            "StackValidationFailed: resources.instance_create_image_err: "
+            "Property error: WebServer.Properties.ImageId: "
+            "Error validating value 'CentOS 5.2': ",
             six.text_type(error))
 
         self.m.VerifyAll()
@@ -469,7 +466,7 @@ class InstancesTest(common.HeatTestCase):
         instance = instances.Instance('instance_create_image_err',
                                       resource_defns['WebServer'], stack)
 
-        self._mock_get_image_id_fail('1', glance_exceptions.NotFound(404))
+        self._mock_get_image_id_fail('1', glance.exceptions.NotFound())
 
         self.stub_VolumeConstraint_validate()
         self.stub_FlavorConstraint_validate()
@@ -480,8 +477,9 @@ class InstancesTest(common.HeatTestCase):
         create = scheduler.TaskRunner(instance.create)
         error = self.assertRaises(exception.ResourceFailure, create)
         self.assertEqual(
-            'StackValidationFailed: resources.instance_create_image_err: '
-            'Property error: WebServer.Properties.ImageId: 404 (HTTP 404)',
+            "StackValidationFailed: resources.instance_create_image_err: "
+            "Property error: WebServer.Properties.ImageId: "
+            "Error validating value '1': Not Found (HTTP 404)",
             six.text_type(error))
 
         self.m.VerifyAll()
@@ -714,7 +712,8 @@ class InstancesTest(common.HeatTestCase):
         self.assertEqual({'test': 123}, instance.metadata_get())
 
     def test_instance_update_instance_type(self):
-        """
+        """Test case for updating InstanceType.
+
         Instance.handle_update supports changing the InstanceType, and makes
         the change making a resize API call against Nova.
         """
@@ -723,10 +722,17 @@ class InstancesTest(common.HeatTestCase):
         instance = self._create_test_instance(return_server,
                                               'ud_type')
 
-        self._stub_glance_for_update()
         update_template = copy.deepcopy(instance.t)
         update_template['Properties']['InstanceType'] = 'm1.small'
 
+        def side_effect(*args):
+            return 2 if args[0] == 'm1.small' else 1
+
+        self.patchobject(nova.NovaClientPlugin, 'find_flavor_by_name_or_id',
+                         side_effect=side_effect)
+        self.patchobject(glance.GlanceClientPlugin, 'find_image_by_name_or_id',
+                         return_value=1)
+
         self.m.StubOutWithMock(self.fc.servers, 'get')
 
         def status_resize(*args):
@@ -761,7 +767,8 @@ class InstancesTest(common.HeatTestCase):
         self.m.VerifyAll()
 
     def test_instance_update_instance_type_failed(self):
-        """
+        """Test case for raising exception due to resize call failed.
+
         If the status after a resize is not VERIFY_RESIZE, it means the resize
         call failed, so we raise an explicit error.
         """
@@ -770,7 +777,14 @@ class InstancesTest(common.HeatTestCase):
         instance = self._create_test_instance(return_server,
                                               'ud_type_f')
 
-        self._stub_glance_for_update()
+        def side_effect(*args):
+            return 2 if args[0] == 'm1.small' else 1
+
+        self.patchobject(nova.NovaClientPlugin, 'find_flavor_by_name_or_id',
+                         side_effect=side_effect)
+        self.patchobject(glance.GlanceClientPlugin, 'find_image_by_name_or_id',
+                         return_value=1)
+
         update_template = copy.deepcopy(instance.t)
         update_template['Properties']['InstanceType'] = 'm1.small'
 
@@ -797,7 +811,7 @@ class InstancesTest(common.HeatTestCase):
         error = self.assertRaises(exception.ResourceFailure, updater)
         self.assertEqual(
             "Error: resources.ud_type_f: "
-            "Resizing to 'm1.small' failed, status 'ERROR'",
+            "Resizing to '2' failed, status 'ERROR'",
             six.text_type(error))
         self.assertEqual((instance.UPDATE, instance.FAILED), instance.state)
         self.m.VerifyAll()
@@ -812,7 +826,8 @@ class InstancesTest(common.HeatTestCase):
         return fake_interface(port, net, ip)
 
     def test_instance_update_network_interfaces(self):
-        """
+        """Test case for updating NetworkInterfaces.
+
         Instance.handle_update supports changing the NetworkInterfaces,
         and makes the change making a resize API call against Nova.
         """
@@ -853,7 +868,8 @@ class InstancesTest(common.HeatTestCase):
         self.m.VerifyAll()
 
     def test_instance_update_network_interfaces_old_include_new(self):
-        """
+        """Test case for updating NetworkInterfaces when old prop includes new.
+
         Instance.handle_update supports changing the NetworkInterfaces,
         and makes the change making a resize API call against Nova.
         """
@@ -888,7 +904,8 @@ class InstancesTest(common.HeatTestCase):
         self.assertEqual((instance.UPDATE, instance.COMPLETE), instance.state)
 
     def test_instance_update_network_interfaces_new_include_old(self):
-        """
+        """Test case for updating NetworkInterfaces when new prop includes old.
+
         Instance.handle_update supports changing the NetworkInterfaces,
         and makes the change making a resize API call against Nova.
         """
@@ -923,7 +940,8 @@ class InstancesTest(common.HeatTestCase):
         self.assertEqual((instance.UPDATE, instance.COMPLETE), instance.state)
 
     def test_instance_update_network_interfaces_new_old_all_different(self):
-        """
+        """Tests updating NetworkInterfaces when new and old are different.
+
         Instance.handle_update supports changing the NetworkInterfaces,
         and makes the change making a resize API call against Nova.
         """
@@ -963,7 +981,8 @@ class InstancesTest(common.HeatTestCase):
         self.assertEqual((instance.UPDATE, instance.COMPLETE), instance.state)
 
     def test_instance_update_network_interfaces_no_old(self):
-        """
+        """Test case for updating NetworkInterfaces when there's no old prop.
+
         Instance.handle_update supports changing the NetworkInterfaces,
         and makes the change making a resize API call against Nova.
         """
@@ -1004,8 +1023,9 @@ class InstancesTest(common.HeatTestCase):
         self.m.VerifyAll()
 
     def test_instance_update_network_interfaces_no_old_empty_new(self):
-        """
-        Instance.handle_update supports changing the NetworkInterfaces
+        """Test case for updating NetworkInterfaces when no old, no new prop.
+
+        Instance.handle_update supports changing the NetworkInterfaces.
         """
         return_server = self.fc.servers.list()[1]
         return_server.id = '1234'
@@ -1083,9 +1103,7 @@ class InstancesTest(common.HeatTestCase):
         self.m.VerifyAll()
 
     def test_instance_update_network_interfaces_empty_new_with_subnet(self):
-        """
-        Test update NetworkInterfaces to empty, and update with subnet.
-        """
+        """Test update NetworkInterfaces to empty, and update with subnet."""
         stack_name = 'ud_network_interfaces_empty_with_subnet'
         self._test_instance_update_with_subnet(
             stack_name, new_interfaces=[])
@@ -1334,9 +1352,10 @@ class InstancesTest(common.HeatTestCase):
             ))
 
     def test_build_nics_with_security_groups(self):
-        """
-        Test the security groups defined in heat template can be associated
-        to a new created port.
+        """Test the security groups can be associated to a new created port.
+
+        Test the security groups defined in heat template can be associated to
+        a new created port.
         """
         return_server = self.fc.servers.list()[1]
         instance = self._create_test_instance(return_server,
@@ -1362,20 +1381,20 @@ class InstancesTest(common.HeatTestCase):
             instance,
             security_groups,
             sg='zero',
-            get_secgroup_raises=exception.PhysicalResourceNotFound)
+            get_secgroup_raises=exception.EntityNotFound)
 
         security_groups = ['wrong_group_name',
                            '0389f747-7785-4757-b7bb-2ab07e4b09c3']
         self._test_security_groups(
             instance,
             security_groups,
-            get_secgroup_raises=exception.PhysicalResourceNotFound)
+            get_secgroup_raises=exception.EntityNotFound)
 
         security_groups = ['wrong_group_name', 'security_group_1']
         self._test_security_groups(
             instance,
             security_groups,
-            get_secgroup_raises=exception.PhysicalResourceNotFound)
+            get_secgroup_raises=exception.EntityNotFound)
 
         security_groups = ['duplicate_group_name', 'security_group_1']
         self._test_security_groups(
