@@ -15,7 +15,7 @@ from heat.tests import common
 from heat.tests import utils
 
 
-class MistralClientPluginTests(common.HeatTestCase):
+class MistralClientPluginTest(common.HeatTestCase):
 
     def test_create(self):
         context = utils.dummy_context()
