@@ -20,12 +20,14 @@ from novaclient import client as nc
 from novaclient import exceptions as nova_exceptions
 from oslo_config import cfg
 from oslo_serialization import jsonutils as json
+from oslo_utils import encodeutils
+import requests
 import six
 
 from heat.common import exception
 from heat.engine.clients.os import nova
 from heat.tests import common
-from heat.tests.nova import fakes as fakes_nova
+from heat.tests.openstack.nova import fakes as fakes_nova
 from heat.tests import utils
 
 
@@ -39,11 +41,8 @@ class NovaClientPluginTestCase(common.HeatTestCase):
         self.nova_plugin._client = self.nova_client
 
 
-class NovaClientPluginTests(NovaClientPluginTestCase):
-    """
-    Basic tests for the helper methods in
-    :module:'heat.engine.clients.os.nova'.
-    """
+class NovaClientPluginTest(NovaClientPluginTestCase):
+    """Basic tests for :module:'heat.engine.clients.os.nova'."""
 
     def test_create(self):
         context = utils.dummy_context()
@@ -75,21 +74,57 @@ class NovaClientPluginTests(NovaClientPluginTestCase):
         observed = self.nova_plugin.get_ip(my_image, 'public', 6)
         self.assertEqual(expected, observed)
 
-    def test_get_flavor_id(self):
+    def test_find_flavor_by_name_or_id(self):
         """Tests the get_flavor_id function."""
         flav_id = str(uuid.uuid4())
         flav_name = 'X-Large'
         my_flavor = mock.MagicMock()
         my_flavor.name = flav_name
         my_flavor.id = flav_id
-        self.nova_client.flavors.list.return_value = [my_flavor]
-        self.assertEqual(flav_id, self.nova_plugin.get_flavor_id(flav_name))
-        self.assertEqual(flav_id, self.nova_plugin.get_flavor_id(flav_id))
-        self.assertRaises(exception.FlavorMissing,
-                          self.nova_plugin.get_flavor_id, 'noflavor')
-        self.assertEqual(3, self.nova_client.flavors.list.call_count)
-        self.assertEqual([(), (), ()],
-                         self.nova_client.flavors.list.call_args_list)
+
+        self.nova_client.flavors.get.side_effect = [
+            my_flavor,
+            nova_exceptions.NotFound(''),
+            nova_exceptions.NotFound(''),
+        ]
+        self.nova_client.flavors.find.side_effect = [
+            my_flavor,
+            nova_exceptions.NotFound(''),
+        ]
+        self.assertEqual(flav_id,
+                         self.nova_plugin.find_flavor_by_name_or_id(flav_id))
+        self.assertEqual(flav_id,
+                         self.nova_plugin.find_flavor_by_name_or_id(flav_name))
+        self.assertRaises(nova_exceptions.ClientException,
+                          self.nova_plugin.find_flavor_by_name_or_id,
+                          'noflavor')
+        self.assertEqual(3, self.nova_client.flavors.get.call_count)
+        self.assertEqual(2, self.nova_client.flavors.find.call_count)
+
+    def test_get_host(self):
+        """Tests the get_host function."""
+        my_host_name = 'myhost'
+        my_host = mock.MagicMock()
+        my_host.host_name = my_host_name
+        my_host.service = 'compute'
+
+        wrong_host = mock.MagicMock()
+        wrong_host.host_name = 'wrong_host'
+        wrong_host.service = 'compute'
+        self.nova_client.hosts.list.side_effect = [
+            [my_host],
+            [wrong_host],
+            exception.EntityNotFound(entity='Host', name='nohost')
+        ]
+        self.assertEqual(my_host, self.nova_plugin.get_host(my_host_name))
+        self.assertRaises(exception.EntityNotFound,
+                          self.nova_plugin.get_host, my_host_name)
+        self.assertRaises(exception.EntityNotFound,
+                          self.nova_plugin.get_host, 'nohost')
+        self.assertEqual(3, self.nova_client.hosts.list.call_count)
+        calls = [mock.call(), mock.call(), mock.call()]
+        self.assertEqual(calls,
+                         self.nova_client.hosts.list.call_args_list)
 
     def test_get_keypair(self):
         """Tests the get_keypair function."""
@@ -101,7 +136,7 @@ class NovaClientPluginTests(NovaClientPluginTestCase):
         self.nova_client.keypairs.get.side_effect = [
             my_key, nova_exceptions.NotFound(404)]
         self.assertEqual(my_key, self.nova_plugin.get_keypair(my_key_name))
-        self.assertRaises(exception.UserKeyPairMissing,
+        self.assertRaises(exception.EntityNotFound,
                           self.nova_plugin.get_keypair, 'notakey')
         calls = [mock.call(my_key_name),
                  mock.call('notakey')]
@@ -130,7 +165,7 @@ class NovaClientPluginTests(NovaClientPluginTestCase):
                          self.nova_plugin.get_net_id_by_label('net_label'))
 
         exc = self.assertRaises(
-            exception.NovaNetworkNotFound,
+            exception.EntityNotFound,
             self.nova_plugin.get_net_id_by_label, 'idontexist')
         expected = 'The Nova network (idontexist) could not be found'
         self.assertIn(expected, six.text_type(exc))
@@ -158,7 +193,7 @@ class NovaClientPluginTests(NovaClientPluginTestCase):
         self.assertEqual(net.id,
                          self.nova_plugin.get_nova_network_id(net.id))
         exc = self.assertRaises(
-            exception.NovaNetworkNotFound,
+            exception.EntityNotFound,
             self.nova_plugin.get_nova_network_id, not_existent_net_id)
         expected = ('The Nova network (%s) could not be found' %
                     not_existent_net_id)
@@ -181,8 +216,46 @@ class NovaClientPluginTests(NovaClientPluginTestCase):
         observed = self.nova_plugin.get_status(server)
         self.assertEqual('ACTIVE', observed)
 
+    def _absolute_limits(self):
+        max_personality = self.m.CreateMockAnything()
+        max_personality.name = 'maxPersonality'
+        max_personality.value = 5
+        max_personality_size = self.m.CreateMockAnything()
+        max_personality_size.name = 'maxPersonalitySize'
+        max_personality_size.value = 10240
+        max_server_meta = self.m.CreateMockAnything()
+        max_server_meta.name = 'maxServerMeta'
+        max_server_meta.value = 3
+        yield max_personality
+        yield max_personality_size
+        yield max_server_meta
 
-class NovaClientPluginRefreshServerTests(NovaClientPluginTestCase):
+    def test_absolute_limits_success(self):
+        limits = mock.Mock()
+        limits.absolute = self._absolute_limits()
+        self.nova_client.limits.get.return_value = limits
+        self.nova_plugin.absolute_limits()
+
+    def test_absolute_limits_retry(self):
+        limits = mock.Mock()
+        limits.absolute = self._absolute_limits()
+        self.nova_client.limits.get.side_effect = [
+            requests.ConnectionError, requests.ConnectionError,
+            limits]
+        self.nova_plugin.absolute_limits()
+        self.assertEqual(3, self.nova_client.limits.get.call_count)
+
+    def test_absolute_limits_failure(self):
+        limits = mock.Mock()
+        limits.absolute = self._absolute_limits()
+        self.nova_client.limits.get.side_effect = [
+            requests.ConnectionError, requests.ConnectionError,
+            requests.ConnectionError]
+        self.assertRaises(requests.ConnectionError,
+                          self.nova_plugin.absolute_limits)
+
+
+class NovaClientPluginRefreshServerTest(NovaClientPluginTestCase):
     msg = ("ClientException: The server has either erred or is "
            "incapable of performing the requested operation.")
 
@@ -215,7 +288,7 @@ class NovaClientPluginRefreshServerTests(NovaClientPluginTestCase):
         server.get.assert_called_once_with()
 
 
-class NovaClientPluginFetchServerTests(NovaClientPluginTestCase):
+class NovaClientPluginFetchServerTest(NovaClientPluginTestCase):
 
     server = mock.Mock()
     # set explicitly as id and name has internal meaning in mock.Mock
@@ -255,7 +328,7 @@ class NovaClientPluginFetchServerTests(NovaClientPluginTestCase):
         self.nova_client.servers.get.assert_called_once_with(self.server.id)
 
 
-class NovaClientPluginCheckActiveTests(NovaClientPluginTestCase):
+class NovaClientPluginCheckActiveTest(NovaClientPluginTestCase):
 
     scenarios = [
         ('active', dict(
@@ -273,7 +346,7 @@ class NovaClientPluginCheckActiveTests(NovaClientPluginTestCase):
     ]
 
     def setUp(self):
-        super(NovaClientPluginCheckActiveTests, self).setUp()
+        super(NovaClientPluginCheckActiveTest, self).setUp()
         self.server = mock.Mock()
         self.server.id = '1234'
         self.server.status = self.status
@@ -314,7 +387,7 @@ class NovaClientPluginCheckActiveTests(NovaClientPluginTestCase):
         self.assertEqual(0, self.r_mock.call_count)
 
 
-class NovaClientPluginUserdataTests(NovaClientPluginTestCase):
+class NovaClientPluginUserdataTest(NovaClientPluginTestCase):
 
     def test_build_userdata(self):
         """Tests the build_userdata function."""
@@ -359,7 +432,7 @@ class NovaClientPluginUserdataTests(NovaClientPluginTestCase):
         self.assertIn('ec2-user', data)
 
 
-class NovaClientPluginMetadataTests(NovaClientPluginTestCase):
+class NovaClientPluginMetadataTest(NovaClientPluginTestCase):
 
     def test_serialize_string(self):
         original = {'test_key': 'simple string value'}
@@ -393,9 +466,7 @@ class NovaClientPluginMetadataTests(NovaClientPluginTestCase):
         self.assertEqual(expected, self.nova_plugin.meta_serialize(original))
 
     def test_serialize_no_value(self):
-        """This test is to prove that the user can only pass in a dict to nova
-        metadata.
-        """
+        """Prove that the user can only pass in a dict to nova metadata."""
         excp = self.assertRaises(exception.StackValidationFailed,
                                  self.nova_plugin.meta_serialize, "foo")
         self.assertIn('metadata needs to be a Map', six.text_type(excp))
@@ -448,17 +519,20 @@ class FlavorConstraintTest(common.HeatTestCase):
         flavor = collections.namedtuple("Flavor", ["id", "name"])
         flavor.id = "1234"
         flavor.name = "foo"
-        client.flavors.list.return_value = [flavor]
 
+        client.flavors.get.side_effect = [flavor,
+                                          nova_exceptions.NotFound(''),
+                                          nova_exceptions.NotFound('')]
+        client.flavors.find.side_effect = [flavor,
+                                           nova_exceptions.NotFound('')]
         constraint = nova.FlavorConstraint()
         ctx = utils.dummy_context()
-        self.assertFalse(constraint.validate("bar", ctx))
-        self.assertTrue(constraint.validate("foo", ctx))
         self.assertTrue(constraint.validate("1234", ctx))
-        nova.NovaClientPlugin._create.assert_called_once_with()
-        self.assertEqual(3, client.flavors.list.call_count)
-        self.assertEqual([(), (), ()],
-                         client.flavors.list.call_args_list)
+        self.assertTrue(constraint.validate("foo", ctx))
+        self.assertFalse(constraint.validate("bar", ctx))
+        self.assertEqual(1, nova.NovaClientPlugin._create.call_count)
+        self.assertEqual(3, client.flavors.get.call_count)
+        self.assertEqual(2, client.flavors.find.call_count)
 
 
 class NetworkConstraintTest(common.HeatTestCase):
@@ -494,6 +568,26 @@ class NetworkConstraintTest(common.HeatTestCase):
         self.assertTrue(constraint.validate(network.id, ctx))
 
 
+class HostConstraintTest(common.HeatTestCase):
+
+    def setUp(self):
+        super(HostConstraintTest, self).setUp()
+        self.ctx = utils.dummy_context()
+        self.mock_get_host = mock.Mock()
+        self.ctx.clients.client_plugin(
+            'nova').get_host = self.mock_get_host
+        self.constraint = nova.HostConstraint()
+
+    def test_validation(self):
+        self.mock_get_host.return_value = mock.MagicMock()
+        self.assertTrue(self.constraint.validate("foo", self.ctx))
+
+    def test_validation_error(self):
+        self.mock_get_host.side_effect = exception.EntityNotFound(
+            entity='Host', name='bar')
+        self.assertFalse(self.constraint.validate("bar", self.ctx))
+
+
 class KeypairConstraintTest(common.HeatTestCase):
 
     def test_validation(self):
@@ -570,7 +664,7 @@ class ConsoleUrlsTest(common.HeatTestCase):
 
         urls = self.nova_plugin.get_console_urls(self.server)
         e = self.assertRaises(exc, urls.__getitem__, self.console_type)
-        self.assertIn('spam', e.message)
+        self.assertIn('spam', encodeutils.exception_to_unicode(e))
         self.console_method.assert_called_once_with(self.console_type)
 
     def test_get_console_urls_reraises_other(self):
@@ -583,29 +677,28 @@ class ConsoleUrlsTest(common.HeatTestCase):
         self.console_method.assert_called_once_with(self.console_type)
 
 
-class NovaClientPluginExtensionsTests(NovaClientPluginTestCase):
+class NovaClientPluginExtensionsTest(NovaClientPluginTestCase):
     """Tests for extensions in novaclient."""
 
-    def test_defines_required_extension_names(self):
-        self.assertEqual(self.nova_plugin.OS_INTERFACE_EXTENSION,
-                         "OSInterface")
-
     def test_has_no_extensions(self):
         self.nova_client.list_extensions.show_all.return_value = []
-        self.assertFalse(self.nova_plugin._has_extension("OSInterface"))
+        self.assertFalse(self.nova_plugin.has_extension(
+            "os-virtual-interfaces"))
 
     def test_has_no_interface_extensions(self):
         mock_extension = mock.Mock()
-        p = mock.PropertyMock(return_value='notOSInterface')
-        type(mock_extension).name = p
+        p = mock.PropertyMock(return_value='os-xxxx')
+        type(mock_extension).alias = p
         self.nova_client.list_extensions.show_all.return_value = [
             mock_extension]
-        self.assertFalse(self.nova_plugin._has_extension("OSInterface"))
+        self.assertFalse(self.nova_plugin.has_extension(
+            "os-virtual-interfaces"))
 
     def test_has_os_interface_extension(self):
         mock_extension = mock.Mock()
-        p = mock.PropertyMock(return_value='OSInterface')
-        type(mock_extension).name = p
+        p = mock.PropertyMock(return_value='os-virtual-interfaces')
+        type(mock_extension).alias = p
         self.nova_client.list_extensions.show_all.return_value = [
             mock_extension]
-        self.assertTrue(self.nova_plugin._has_extension("OSInterface"))
+        self.assertTrue(self.nova_plugin.has_extension(
+            "os-virtual-interfaces"))
