@@ -23,14 +23,11 @@ from heat.tests import common
 from heat.tests import utils
 
 
-class SaharaUtilsTests(common.HeatTestCase):
-    """
-    Basic tests for the helper methods in
-    :module:'heat.engine.resources.clients.os.sahara'.
-    """
+class SaharaUtilsTest(common.HeatTestCase):
+    """Basic tests :module:'heat.engine.resources.clients.os.sahara'."""
 
     def setUp(self):
-        super(SaharaUtilsTests, self).setUp()
+        super(SaharaUtilsTest, self).setUp()
         self.sahara_client = mock.MagicMock()
         con = utils.dummy_context()
         c = con.clients
@@ -128,7 +125,7 @@ class SaharaUtilsTests(common.HeatTestCase):
                                                error_name='NOT_FOUND')
 
         self.sahara_client.plugins.get.side_effect = side_effect
-        self.assertEqual(None, self.sahara_plugin.get_plugin_id(plugin_name))
+        self.assertIsNone(self.sahara_plugin.get_plugin_id(plugin_name))
         self.assertRaises(exception.EntityNotFound,
                           self.sahara_plugin.get_plugin_id, 'noplugin')
 
