@@ -191,6 +191,11 @@ class HeatTestCase(testscenarios.WithScenarios,
                                  generic_rsrc.ResourceTypeUnSupportedLiberty)
         resource._register_class('ResourceTypeSupportedKilo',
                                  generic_rsrc.ResourceTypeSupportedKilo)
+        resource._register_class('ResourceTypeHidden',
+                                 generic_rsrc.ResourceTypeHidden)
+        resource._register_class(
+            'ResourceWithHiddenPropertyAndAttribute',
+            generic_rsrc.ResourceWithHiddenPropertyAndAttribute)
 
     def patchobject(self, obj, attr, **kwargs):
         mockfixture = self.useFixture(mockpatch.PatchObject(obj, attr,
