@@ -101,6 +101,16 @@ class ResourceTypeTest(common.HeatTestCase):
         schema = self.eng.resource_schema(self.ctx, type_name=type_name)
         self.assertEqual(expected, schema)
 
+    def test_resource_schema_for_hidden_type(self):
+        type_name = 'ResourceTypeHidden'
+        self.assertRaises(exception.NotSupported, self.eng.resource_schema,
+                          self.ctx, type_name)
+
+    def test_generate_template_for_hidden_type(self):
+        type_name = 'ResourceTypeHidden'
+        self.assertRaises(exception.NotSupported, self.eng.generate_template,
+                          self.ctx, type_name)
+
     def test_resource_schema_with_attr_type(self):
 
         type_name = 'ResourceWithAttributeType'
@@ -126,6 +136,37 @@ class ResourceTypeTest(common.HeatTestCase):
         schema = self.eng.resource_schema(self.ctx, type_name=type_name)
         self.assertEqual(expected, schema)
 
+    def test_resource_schema_with_hidden(self):
+
+        type_name = 'ResourceWithHiddenPropertyAndAttribute'
+        expected = {
+            'resource_type': type_name,
+            'properties': {
+                'supported': {
+                    'description': "Supported property.",
+                    'type': 'list',
+                    'immutable': False,
+                    'required': False,
+                    'update_allowed': False
+                }
+            },
+            'attributes': {
+                'supported': {'description': 'Supported attribute.',
+                              'type': 'string'},
+                'show': {
+                    'description': 'Detailed information about resource.',
+                    'type': 'map'},
+            },
+            'support_status': {
+                'status': 'SUPPORTED',
+                'version': None,
+                'message': None,
+                'previous_status': None
+            }
+        }
+        schema = self.eng.resource_schema(self.ctx, type_name=type_name)
+        self.assertEqual(expected, schema)
+
     def _no_template_file(self, function):
         env = environment.Environment()
         info = environment.ResourceInfo(env.registry,
