@@ -14,10 +14,12 @@
 import mock
 
 from heat.engine import properties
+from heat.engine import resource
 from heat.engine.resources.openstack.keystone import service
 from heat.engine import stack
 from heat.engine import template
 from heat.tests import common
+from heat.tests import fakes
 from heat.tests import utils
 
 keystone_service_template = {
@@ -45,7 +47,10 @@ class KeystoneServiceTest(common.HeatTestCase):
         self.ctx = utils.dummy_context()
 
         # Mock client
-        self.keystoneclient = mock.MagicMock()
+        self.keystoneclient = mock.Mock()
+        self.patchobject(resource.Resource, 'client',
+                         return_value=fakes.FakeKeystoneClient(
+                             client=self.keystoneclient))
         self.services = self.keystoneclient.services
 
         # Mock client plugin
