@@ -16,8 +16,6 @@ import mock
 from oslo_config import cfg
 import six
 
-from magnumclient.openstack.common.apiclient import exceptions as mc_exc
-
 from heat.common import exception
 from heat.common import template_format
 from heat.engine.clients.os import magnum as mc
@@ -28,6 +26,12 @@ from heat.engine import template
 from heat.tests import common
 from heat.tests import utils
 
+try:
+    from magnumclient.common.apiclient import exceptions as mc_exc
+except ImportError:
+    # NOTE (duvarenkov): For magnumclient versions before 2.0.0.
+    from magnumclient.openstack.common.apiclient import exceptions as mc_exc
+
 
 magnum_template = '''
     heat_template_version: 2015-04-30
