@@ -13,7 +13,6 @@
 
 import mock
 
-from heat.common import exception
 from heat.engine.clients.os import monasca as client_plugin
 from heat.engine import resource
 from heat.engine.resources.openstack.monasca import alarm_definition
@@ -47,10 +46,11 @@ RESOURCE_TYPE = 'OS::Monasca::AlarmDefinition'
 
 
 class MonascaAlarmDefinition(alarm_definition.MonascaAlarmDefinition):
-    '''
+    """This class overrides the is_service_available to return True.
+
     Monasca service is not available by default. So, this class overrides
-    the is_service_available to return True
-    '''
+    the is_service_available to return True.
+    """
     @classmethod
     def is_service_available(cls, context):
         return True
@@ -73,19 +73,20 @@ class MonascaAlarmDefinitionTest(common.HeatTestCase):
 
         self.test_resource = self.stack['test_resource']
 
-        # Mock client plugin
-        self.test_client_plugin = mock.MagicMock()
-        self.test_resource.client_plugin = mock.MagicMock(
-            return_value=self.test_client_plugin)
-        self.test_client_plugin.get_notification.return_value = (
-            'sample_notification'
-        )
-
         # Mock client
         self.test_client = mock.MagicMock()
         self.test_resource.client = mock.MagicMock(
             return_value=self.test_client)
 
+        # Mock client plugin
+        self.test_client_plugin = client_plugin.MonascaClientPlugin(self.ctx)
+        self.test_client_plugin._create = mock.MagicMock(
+            return_value=self.test_client)
+        self.test_resource.client_plugin = mock.MagicMock(
+            return_value=self.test_client_plugin)
+        self.test_client_plugin.get_notification = mock.MagicMock(
+            return_value='sample_notification')
+
     def _get_mock_resource(self):
         value = dict(id='477e8273-60a7-4c41-b683-fdb0bc7cd152')
 
@@ -165,8 +166,6 @@ class MonascaAlarmDefinitionTest(common.HeatTestCase):
                 'name-updated',
             alarm_definition.MonascaAlarmDefinition.DESCRIPTION:
                 'description-updated',
-            alarm_definition.MonascaAlarmDefinition.EXPRESSION:
-                'expression-updated',
             alarm_definition.MonascaAlarmDefinition.ACTIONS_ENABLED:
                 True,
             alarm_definition.MonascaAlarmDefinition.SEVERITY:
@@ -186,7 +185,6 @@ class MonascaAlarmDefinitionTest(common.HeatTestCase):
             alarm_id=self.test_resource.resource_id,
             name='name-updated',
             description='description-updated',
-            expression='expression-updated',
             actions_enabled=True,
             severity='medium',
             ok_actions=['sample_notification'],
@@ -196,36 +194,6 @@ class MonascaAlarmDefinitionTest(common.HeatTestCase):
 
         mock_alarm_patch.assert_called_once_with(**args)
 
-    def test_resource_handle_update_sub_expression(self):
-        '''
-        Monasca does not allow to update the metrics in the expression though
-        it allows to update the metrics measurement condition range. Monasca
-        client raises HTTPUnProcessable in this case
-
-        so UpdateReplace is raised to re-create the alarm-definition with
-        updated new expression.
-        '''
-        # TODO(skraynev): remove it when monasca client will be
-        #                 merged in global requirements
-        class HTTPUnProcessable(Exception):
-            pass
-
-        client_plugin.monasca_exc = mock.Mock()
-        client_plugin.monasca_exc.HTTPUnProcessable = HTTPUnProcessable
-        mock_alarm_patch = self.test_client.alarm_definitions.patch
-        mock_alarm_patch.side_effect = (
-            client_plugin.monasca_exc.HTTPUnProcessable)
-        self.test_resource.resource_id = '477e8273-60a7-4c41-b683-fdb0bc7cd151'
-
-        prop_diff = {alarm_definition.MonascaAlarmDefinition.EXPRESSION:
-                     'expression-updated'}
-
-        self.assertRaises(exception.UpdateReplace,
-                          self.test_resource.handle_update,
-                          json_snippet=None,
-                          tmpl_diff=None,
-                          prop_diff=prop_diff)
-
     def test_resource_handle_delete(self):
         mock_alarm_delete = self.test_client.alarm_definitions.delete
         self.test_resource.resource_id = '477e8273-60a7-4c41-b683-fdb0bc7cd151'
@@ -253,18 +221,6 @@ class MonascaAlarmDefinitionTest(common.HeatTestCase):
         mock_alarm_delete = self.test_client.alarm_definitions.delete
         mock_alarm_delete.side_effect = client_plugin.monasca_exc.NotFound
         self.assertIsNone(self.test_resource.handle_delete())
-        self.assertEqual(1,
-                         self.test_client_plugin.ignore_not_found.call_count)
-        e_type = type(self.test_client_plugin.ignore_not_found.call_args[0][0])
-        self.assertEqual(type(client_plugin.monasca_exc.NotFound()), e_type)
-
-    def test_resource_mapping(self):
-        mapping = alarm_definition.resource_mapping()
-        self.assertEqual(1, len(mapping))
-        self.assertEqual(alarm_definition.MonascaAlarmDefinition,
-                         mapping[RESOURCE_TYPE])
-        self.assertIsInstance(self.test_resource,
-                              alarm_definition.MonascaAlarmDefinition)
 
     def test_resource_show_resource(self):
         mock_notification_get = self.test_client.alarm_definitions.get
