@@ -41,7 +41,7 @@ class FakeClient(fakes.FakeClient, Client):
 
     def __init__(self, *args, **kwargs):
         super(FakeClient, self).__init__('username', 'password', 'project_id',
-                                         'auth_url')
+                                         'auth_url', direct_use=False)
         self.client = FakeHTTPClient(**kwargs)
 
 
