@@ -47,7 +47,9 @@ class TestRequestContext(common.HeatTestCase):
                     'auth_url': 'http://xyz',
                     'aws_creds': 'blah',
                     'region_name': 'RegionOne',
-                    'user_identity': 'fooUser 456tenant'}
+                    'user_identity': 'fooUser 456tenant',
+                    'user_domain_id': None,
+                    'project_domain_id': None}
 
         super(TestRequestContext, self).setUp()
 
@@ -68,7 +70,9 @@ class TestRequestContext(common.HeatTestCase):
             trustor_user_id=self.ctx.get('trustor_user_id'),
             trust_id=self.ctx.get('trust_id'),
             user=self.ctx.get('user'),
-            region_name=self.ctx.get('region_name'))
+            region_name=self.ctx.get('region_name'),
+            user_domain_id=self.ctx.get('user_domain'),
+            project_domain_id=self.ctx.get('project_domain'))
         ctx_dict = ctx.to_dict()
         del(ctx_dict['request_id'])
         self.assertEqual(self.ctx, ctx_dict)
@@ -83,7 +87,9 @@ class TestRequestContext(common.HeatTestCase):
         ctx = context.RequestContext.from_dict(self.ctx)
 
         for k in self.ctx:
-            if k == 'user_identity':
+            if (k == 'user_identity' or
+                    k == 'user_domain_id' or
+                    k == 'project_domain_id'):
                 continue
             self.assertEqual(self.ctx.get(k), ctx.to_dict().get(k))
             override = '%s_override' % k
@@ -115,9 +121,7 @@ class TestRequestContext(common.HeatTestCase):
             self.assertFalse(ctx.is_admin)
 
     def test_keystone_v3_endpoint_in_context(self):
-        """Ensure that the context is the preferred source for the
-        auth_uri.
-        """
+        """Ensure that the context is the preferred source for the auth_uri."""
         cfg.CONF.set_override('auth_uri', 'http://xyz',
                               group='clients_keystone')
         policy_check = 'heat.common.policy.Enforcer.check_is_admin'
@@ -129,7 +133,9 @@ class TestRequestContext(common.HeatTestCase):
                              'http://example.com:5000/v3')
 
     def test_keystone_v3_endpoint_in_clients_keystone_config(self):
-        """Ensure that the [clients_keystone] section of the configuration is
+        """Ensure that the [clients_keystone] section is the preferred source.
+
+        Ensure that the [clients_keystone] section of the configuration is
         the preferred source when the context does not have the auth_uri.
         """
         cfg.CONF.set_override('auth_uri', 'http://xyz',
@@ -150,7 +156,9 @@ class TestRequestContext(common.HeatTestCase):
                 self.assertEqual(ctx.keystone_v3_endpoint, 'http://xyz/v3')
 
     def test_keystone_v3_endpoint_in_keystone_authtoken_config(self):
-        """Ensure that the [keystone_authtoken] section of the configuration
+        """Ensure that the [keystone_authtoken] section is used.
+
+        Ensure that the [keystone_authtoken] section of the configuration
         is used when the auth_uri is not defined in the context or the
         [clients_keystone] section.
         """
@@ -164,7 +172,9 @@ class TestRequestContext(common.HeatTestCase):
             self.assertEqual(ctx.keystone_v3_endpoint, 'http://abc/v3')
 
     def test_keystone_v3_endpoint_not_set_in_config(self):
-        """Ensure an exception is raised when the auth_uri cannot be obtained
+        """Ensure an exception is raised when the auth_uri cannot be obtained.
+
+        Ensure an exception is raised when the auth_uri cannot be obtained
         from any source.
         """
         policy_check = 'heat.common.policy.Enforcer.check_is_admin'
@@ -174,6 +184,28 @@ class TestRequestContext(common.HeatTestCase):
             self.assertRaises(exception.AuthorizationFailure, getattr, ctx,
                               'keystone_v3_endpoint')
 
+    def test_create_trusts_auth_plugin_with_correct_user_domain_id(self):
+        importutils.import_module('keystonemiddleware.auth_token')
+        cfg.CONF.set_override('auth_uri', 'http://abc/v2.0',
+                              group='keystone_authtoken')
+        cfg.CONF.set_override('admin_user', 'heat',
+                              group='keystone_authtoken')
+        cfg.CONF.set_override('admin_password', 'password',
+                              group='keystone_authtoken')
+        policy_check = 'heat.common.policy.Enforcer.check_is_admin'
+        with mock.patch(policy_check) as pc:
+            pc.return_value = False
+            ctx = context.RequestContext(auth_url=None,
+                                         user_domain_id='non-default',
+                                         username='test')
+            with mock.patch('keystoneclient.auth.identity.v3.Password') as ps:
+                ctx.trusts_auth_plugin
+                ps.assert_called_once_with(username='heat',
+                                           password='password',
+                                           user_domain_id='default',
+                                           auth_url='http://abc/v3',
+                                           trust_id=None)
+
 
 class RequestContextMiddlewareTest(common.HeatTestCase):
 
