@@ -44,7 +44,7 @@ class StackConvergenceCreateUpdateDeleteTest(common.HeatTestCase):
         stack.store()
         stack.converge_stack(template=stack.t, action=stack.CREATE)
         self.assertFalse(mock_cr.called)
-        mock_mc.assert_called_once_with(stack.current_traversal)
+        mock_mc.assert_called_once_with()
 
     def test_conv_wordpress_single_instance_stack_create(self, mock_cr):
         stack = tools.get_stack('test_stack', utils.dummy_context(),
@@ -62,7 +62,7 @@ class StackConvergenceCreateUpdateDeleteTest(common.HeatTestCase):
 
         self.assertIsNone(stack_db.prev_raw_template_id)
 
-        self.assertEqual(stack_db.convergence, True)
+        self.assertTrue(stack_db.convergence)
         self.assertEqual({'edges': [[[1, True], None]]}, stack_db.current_deps)
         leaves = stack.convergence_dependencies.leaves()
         expected_calls = []
@@ -92,7 +92,7 @@ class StackConvergenceCreateUpdateDeleteTest(common.HeatTestCase):
         self.assertIsNotNone(stack_db.current_traversal)
         self.assertIsNotNone(stack_db.raw_template_id)
         self.assertIsNone(stack_db.prev_raw_template_id)
-        self.assertEqual(stack_db.convergence, True)
+        self.assertTrue(stack_db.convergence)
         self.assertEqual(sorted([[[3, True], [5, True]],    # C, A
                                  [[3, True], [4, True]],    # C, B
                                  [[1, True], [3, True]],    # E, C
@@ -133,6 +133,7 @@ class StackConvergenceCreateUpdateDeleteTest(common.HeatTestCase):
 
     def _mock_convg_db_update_requires(self, key_id=False):
         """Updates requires column of resources.
+
         Required for testing the generation of convergence dependency graph
         on an update.
         """
@@ -194,7 +195,7 @@ class StackConvergenceCreateUpdateDeleteTest(common.HeatTestCase):
         self.assertIsNotNone(stack_db.raw_template_id)
         self.assertIsNotNone(stack_db.current_traversal)
         self.assertIsNotNone(stack_db.prev_raw_template_id)
-        self.assertEqual(True, stack_db.convergence)
+        self.assertTrue(stack_db.convergence)
         self.assertEqual(sorted([[[7, True], [8, True]],
                                  [[8, True], [5, True]],
                                  [[8, True], [4, True]],
@@ -365,7 +366,7 @@ class StackConvergenceCreateUpdateDeleteTest(common.HeatTestCase):
                                 convergence=True)
         stack.store()
         stack.purge_db = mock.Mock()
-        stack.mark_complete(stack.current_traversal)
+        stack.mark_complete()
         self.assertTrue(stack.purge_db.called)
 
     @mock.patch.object(raw_template_object.RawTemplate, 'delete')
@@ -378,6 +379,37 @@ class StackConvergenceCreateUpdateDeleteTest(common.HeatTestCase):
         stack.purge_db()
         self.assertTrue(mock_tmpl_delete.called)
 
+    @mock.patch.object(parser.Stack, '_delete_credentials')
+    @mock.patch.object(stack_object.Stack, 'delete')
+    def test_purge_db_deletes_creds(self, mock_delete_stack,
+                                    mock_creds_delete, mock_cr):
+        stack = tools.get_stack('test_stack', utils.dummy_context(),
+                                template=tools.string_template_five,
+                                convergence=True)
+        reason = 'stack delete complete'
+        mock_creds_delete.return_value = (stack.COMPLETE, reason)
+        stack.state_set(stack.DELETE, stack.COMPLETE, reason)
+        stack.purge_db()
+        self.assertTrue(mock_creds_delete.called)
+        self.assertTrue(mock_delete_stack.called)
+
+    @mock.patch.object(parser.Stack, '_delete_credentials')
+    @mock.patch.object(stack_object.Stack, 'delete')
+    def test_purge_db_deletes_creds_failed(self, mock_delete_stack,
+                                           mock_creds_delete, mock_cr):
+        stack = tools.get_stack('test_stack', utils.dummy_context(),
+                                template=tools.string_template_five,
+                                convergence=True)
+
+        reason = 'stack delete complete'
+        failed_reason = 'Error deleting trust'
+        mock_creds_delete.return_value = (stack.FAILED, failed_reason)
+        stack.state_set(stack.DELETE, stack.COMPLETE, reason)
+        stack.purge_db()
+        self.assertTrue(mock_creds_delete.called)
+        self.assertFalse(mock_delete_stack.called)
+        self.assertEqual((stack.DELETE, stack.FAILED), stack.state)
+
     @mock.patch.object(raw_template_object.RawTemplate, 'delete')
     def test_purge_db_does_not_delete_previous_template_when_stack_fails(
             self, mock_tmpl_delete, mock_cr):
@@ -415,7 +447,7 @@ class StackConvergenceCreateUpdateDeleteTest(common.HeatTestCase):
                                 convergence=True)
         stack.store()
         stack.prev_raw_template_id = 2
-        stack.t.id = 1
+        stack.t.id = 3
         dummy_res = stack.resources['A']
         a_res_2 = res.Resource('A', dummy_res.t, stack)
         a_res_2.current_template_id = 2
@@ -431,17 +463,131 @@ class StackConvergenceCreateUpdateDeleteTest(common.HeatTestCase):
                         1: a_res_1}
         stack.ext_rsrcs_db = existing_res
         best_res = stack._get_best_existing_rsrc_db('A')
-        # should return resource with template id 1 which is current template
-        self.assertEqual(a_res_1.id, best_res.id)
+        # should return resource with template id 3 which is current template
+        self.assertEqual(a_res_3.id, best_res.id)
 
-        # no resource with current template id as 1
-        existing_res = {2: a_res_2,
-                        3: a_res_3}
+        # no resource with current template id as 3
+        existing_res = {1: a_res_1,
+                        2: a_res_2}
         stack.ext_rsrcs_db = existing_res
         best_res = stack._get_best_existing_rsrc_db('A')
         # should return resource with template id 2 which is prev template
         self.assertEqual(a_res_2.id, best_res.id)
 
+        # no resource with current template id as 3 or 2
+        existing_res = {1: a_res_1}
+        stack.ext_rsrcs_db = existing_res
+        best_res = stack._get_best_existing_rsrc_db('A')
+        # should return resource with template id 1 existing in DB
+        self.assertEqual(a_res_1.id, best_res.id)
+
+    @mock.patch.object(parser.Stack, '_converge_create_or_update')
+    def test_updated_time_stack_create(self, mock_ccu, mock_cr):
+        stack = parser.Stack(utils.dummy_context(), 'convg_updated_time_test',
+                             templatem.Template.create_empty_template(),
+                             convergence=True)
+        stack.converge_stack(template=stack.t, action=stack.CREATE)
+        self.assertIsNone(stack.updated_time)
+        self.assertTrue(mock_ccu.called)
+
+    @mock.patch.object(parser.Stack, '_converge_create_or_update')
+    def test_updated_time_stack_update(self, mock_ccu, mock_cr):
+        tmpl = {'HeatTemplateFormatVersion': '2012-12-12',
+                'Resources': {'R1': {'Type': 'GenericResourceType'}}}
+        stack = parser.Stack(utils.dummy_context(), 'updated_time_test',
+                             templatem.Template(tmpl), convergence=True)
+        stack.converge_stack(template=stack.t, action=stack.UPDATE)
+        self.assertIsNotNone(stack.updated_time)
+        self.assertTrue(mock_ccu.called)
+
+    @mock.patch.object(parser.Stack, '_converge_create_or_update')
+    @mock.patch.object(sync_point_object.SyncPoint,
+                       'delete_all_by_stack_and_traversal')
+    def test_sync_point_delete_stack_create(self, mock_syncpoint_del,
+                                            mock_ccu, mock_cr):
+        stack = parser.Stack(utils.dummy_context(), 'convg_updated_time_test',
+                             templatem.Template.create_empty_template())
+        stack.converge_stack(template=stack.t, action=stack.CREATE)
+        self.assertFalse(mock_syncpoint_del.called)
+        self.assertTrue(mock_ccu.called)
+
+    @mock.patch.object(parser.Stack, '_converge_create_or_update')
+    @mock.patch.object(sync_point_object.SyncPoint,
+                       'delete_all_by_stack_and_traversal')
+    def test_sync_point_delete_stack_update(self, mock_syncpoint_del,
+                                            mock_ccu, mock_cr):
+        tmpl = {'HeatTemplateFormatVersion': '2012-12-12',
+                'Resources': {'R1': {'Type': 'GenericResourceType'}}}
+        stack = parser.Stack(utils.dummy_context(), 'updated_time_test',
+                             templatem.Template(tmpl))
+        stack.current_traversal = 'prev_traversal'
+        stack.converge_stack(template=stack.t, action=stack.UPDATE)
+        self.assertTrue(mock_syncpoint_del.called)
+        self.assertTrue(mock_ccu.called)
+
+
+@mock.patch.object(parser.Stack, '_persist_state')
+class TestConvgStackStateSet(common.HeatTestCase):
+    def setUp(self):
+        super(TestConvgStackStateSet, self).setUp()
+        cfg.CONF.set_override('convergence_engine', True)
+        self.stack = tools.get_stack(
+            'test_stack', utils.dummy_context(),
+            template=tools.wp_template, convergence=True)
+
+    def test_state_set_create_adopt_update_delete_complete(self, mock_ps):
+        self.stack.state_set(self.stack.CREATE, self.stack.COMPLETE,
+                             'Create complete')
+        self.assertTrue(mock_ps.called)
+        mock_ps.reset_mock()
+        self.stack.state_set(self.stack.UPDATE, self.stack.COMPLETE,
+                             'Update complete')
+        self.assertTrue(mock_ps.called)
+        mock_ps.reset_mock()
+        self.stack.state_set(self.stack.DELETE, self.stack.COMPLETE,
+                             'Delete complete')
+        self.assertTrue(mock_ps.called)
+        mock_ps.reset_mock()
+        self.stack.state_set(self.stack.ADOPT, self.stack.COMPLETE,
+                             'Adopt complete')
+        self.assertTrue(mock_ps.called)
+
+    def test_state_set_stack_suspend(self, mock_ps):
+        self.stack.state_set(self.stack.SUSPEND, self.stack.IN_PROGRESS,
+                             'Suspend started')
+        self.assertTrue(mock_ps.called)
+        mock_ps.reset_mock()
+        self.stack.state_set(self.stack.SUSPEND, self.stack.COMPLETE,
+                             'Suspend complete')
+        self.assertFalse(mock_ps.called)
+
+    def test_state_set_stack_resume(self, mock_ps):
+        self.stack.state_set(self.stack.RESUME, self.stack.IN_PROGRESS,
+                             'Resume started')
+        self.assertTrue(mock_ps.called)
+        mock_ps.reset_mock()
+        self.stack.state_set(self.stack.RESUME, self.stack.COMPLETE,
+                             'Resume complete')
+        self.assertFalse(mock_ps.called)
+
+    def test_state_set_stack_snapshot(self, mock_ps):
+        self.stack.state_set(self.stack.SNAPSHOT, self.stack.IN_PROGRESS,
+                             'Snapshot started')
+        self.assertTrue(mock_ps.called)
+        mock_ps.reset_mock()
+        self.stack.state_set(self.stack.SNAPSHOT, self.stack.COMPLETE,
+                             'Snapshot complete')
+        self.assertFalse(mock_ps.called)
+
+    def test_state_set_stack_restore(self, mock_ps):
+        self.stack.state_set(self.stack.RESTORE, self.stack.IN_PROGRESS,
+                             'Restore started')
+        self.assertTrue(mock_ps.called)
+        mock_ps.reset_mock()
+        self.stack.state_set(self.stack.RESTORE, self.stack.COMPLETE,
+                             'Restore complete')
+        self.assertFalse(mock_ps.called)
+
 
 class TestConvgStackRollback(common.HeatTestCase):
 
