@@ -84,7 +84,7 @@ class FormatTest(common.HeatTestCase):
         resource_details_keys = resource_keys.union(set((
             rpc_api.RES_DESCRIPTION,
             rpc_api.RES_METADATA,
-            rpc_api.RES_SCHEMA_ATTRIBUTES,
+            rpc_api.RES_ATTRIBUTES,
         )))
 
         formatted = api.format_stack_resource(res, True)
@@ -101,7 +101,7 @@ class FormatTest(common.HeatTestCase):
     def test_format_stack_resource_no_attrs(self):
         res = self.stack['generic1']
         formatted = api.format_stack_resource(res, True, with_attr=False)
-        self.assertNotIn(rpc_api.RES_SCHEMA_ATTRIBUTES, formatted)
+        self.assertNotIn(rpc_api.RES_ATTRIBUTES, formatted)
         self.assertIn(rpc_api.RES_METADATA, formatted)
 
     def test_format_stack_resource_has_been_deleted(self):
@@ -129,7 +129,7 @@ class FormatTest(common.HeatTestCase):
         res = self.stack['generic1']
 
         formatted = api.format_stack_resource(res, True, with_props=True)
-        formatted_props = formatted[rpc_api.RES_SCHEMA_PROPERTIES]
+        formatted_props = formatted[rpc_api.RES_PROPERTIES]
         self.assertEqual('formatted_res_props', formatted_props)
 
     @mock.patch.object(api, 'format_resource_attributes')
@@ -138,7 +138,7 @@ class FormatTest(common.HeatTestCase):
         res = self.stack['generic1']
 
         formatted = api.format_stack_resource(res, True, with_attr=['a', 'b'])
-        formatted_attrs = formatted[rpc_api.RES_SCHEMA_ATTRIBUTES]
+        formatted_attrs = formatted[rpc_api.RES_ATTRIBUTES]
         self.assertEqual('formatted_resource_attrs', formatted_attrs)
 
     def test_format_resource_attributes(self):
@@ -358,6 +358,7 @@ class FormatTest(common.HeatTestCase):
             'stack_status': 'IN_PROGRESS',
             'stack_status_reason': '',
             'stack_user_project_id': None,
+            'outputs': [],
             'template_description': 'No description',
             'timeout_mins': None,
             'tags': None,
@@ -396,7 +397,51 @@ class FormatTest(common.HeatTestCase):
         info = api.format_stack(self.stack)
         self.assertEqual('foobar', info[rpc_api.STACK_OUTPUTS])
 
+    @mock.patch.object(api, 'format_stack_outputs')
+    def test_format_stack_without_resolving_outputs(self, mock_fmt_outputs):
+        mock_fmt_outputs.return_value = 'foobar'
+        self.stack.action = 'CREATE'
+        self.stack.status = 'COMPLETE'
+        info = api.format_stack(self.stack, resolve_outputs=False)
+        self.assertIsNone(info.get(rpc_api.STACK_OUTPUTS))
+
     def test_format_stack_outputs(self):
+        tmpl = template.Template({
+            'HeatTemplateFormatVersion': '2012-12-12',
+            'Resources': {
+                'generic': {'Type': 'GenericResourceType'}
+            },
+            'Outputs': {
+                'correct_output': {
+                    'Description': 'Good output',
+                    'Value': {'Fn::GetAtt': ['generic', 'Foo']}
+                },
+                'incorrect_output': {
+                    'Value': {'Fn::GetAtt': ['generic', 'Bar']}
+                }
+            }
+        })
+        stack = parser.Stack(utils.dummy_context(), 'test_stack',
+                             tmpl, stack_id=str(uuid.uuid4()))
+        stack.action = 'CREATE'
+        stack.status = 'COMPLETE'
+        stack['generic'].action = 'CREATE'
+        stack['generic'].status = 'COMPLETE'
+        info = api.format_stack_outputs(stack, stack.outputs,
+                                        resolve_value=True)
+        expected = [{'description': 'No description given',
+                     'output_error': 'The Referenced Attribute (generic Bar) '
+                                     'is incorrect.',
+                     'output_key': 'incorrect_output',
+                     'output_value': None},
+                    {'description': 'Good output',
+                     'output_key': 'correct_output',
+                     'output_value': 'generic'}]
+
+        self.assertEqual(expected, sorted(info, key=lambda k: k['output_key'],
+                                          reverse=True))
+
+    def test_format_stack_outputs_unresolved(self):
         tmpl = template.Template({
             'HeatTemplateFormatVersion': '2012-12-12',
             'Resources': {
@@ -420,13 +465,9 @@ class FormatTest(common.HeatTestCase):
         stack['generic'].status = 'COMPLETE'
         info = api.format_stack_outputs(stack, stack.outputs)
         expected = [{'description': 'No description given',
-                     'output_error': 'The Referenced Attribute (generic Bar) '
-                                     'is incorrect.',
-                     'output_key': 'incorrect_output',
-                     'output_value': None},
+                     'output_key': 'incorrect_output'},
                     {'description': 'Good output',
-                     'output_key': 'correct_output',
-                     'output_value': 'generic'}]
+                     'output_key': 'correct_output'}]
 
         self.assertEqual(expected, sorted(info, key=lambda k: k['output_key'],
                                           reverse=True))
@@ -960,9 +1001,7 @@ class FormatValidateParameterTest(common.HeatTestCase):
     ]
 
     def test_format_validate_parameter(self):
-        """
-        Test format of a parameter.
-        """
+        """Test format of a parameter."""
 
         t = template_format.parse(self.template % self.param)
         tmpl = template.Template(t)
