@@ -28,6 +28,7 @@ from heat.engine.resources.openstack.nova import server
 from heat.engine import support
 from heat.tests import common
 from heat.tests import generic_resource
+from heat.tests import utils
 
 
 cfg.CONF.import_opt('environment_dir', 'heat.common.config')
@@ -43,6 +44,7 @@ class EnvironmentTest(common.HeatTestCase):
         expected = {u'parameters': old,
                     u'encrypted_param_names': [],
                     u'parameter_defaults': {},
+                    u'event_sinks': [],
                     u'resource_registry': {u'resources': {}}}
         env = environment.Environment(old)
         self.assertEqual(expected, env.user_env_as_dict())
@@ -51,6 +53,7 @@ class EnvironmentTest(common.HeatTestCase):
         new_env = {u'parameters': {u'a': u'ff', u'b': u'ss'},
                    u'encrypted_param_names': [],
                    u'parameter_defaults': {u'ff': 'new_def'},
+                   u'event_sinks': [],
                    u'resource_registry': {u'OS::Food': u'fruity.yaml',
                                           u'resources': {}}}
         env = environment.Environment(new_env)
@@ -66,6 +69,22 @@ class EnvironmentTest(common.HeatTestCase):
                          env.get_resource_info('OS::Nova::Server',
                                                'my_db_server').name)
 
+    def test_global_registry_many_to_one(self):
+        new_env = {u'parameters': {u'a': u'ff', u'b': u'ss'},
+                   u'resource_registry': {u'OS::Nova::*': 'OS::Heat::None'}}
+        env = environment.Environment(new_env)
+        self.assertEqual('OS::Heat::None',
+                         env.get_resource_info('OS::Nova::Server',
+                                               'my_db_server').name)
+
+    def test_global_registry_many_to_one_no_recurse(self):
+        new_env = {u'parameters': {u'a': u'ff', u'b': u'ss'},
+                   u'resource_registry': {u'OS::*': 'OS::Heat::None'}}
+        env = environment.Environment(new_env)
+        self.assertEqual('OS::Heat::None',
+                         env.get_resource_info('OS::Some::Name',
+                                               'my_db_server').name)
+
     def test_map_one_resource_type(self):
         new_env = {u'parameters': {u'a': u'ff', u'b': u'ss'},
                    u'resource_registry': {u'resources':
@@ -105,7 +124,7 @@ class EnvironmentTest(common.HeatTestCase):
             u'OS::Networking::FloatingIP': 'ip.yaml'}}}}
 
         env = environment.Environment()
-        self.assertRaises(exception.ResourceTypeNotFound,
+        self.assertRaises(exception.EntityNotFound,
                           env.get_resource_info,
                           'OS::Networking::FloatingIP', 'my_fip')
 
@@ -196,6 +215,23 @@ def constraint_mapping():
                          env.get_constraint("nova.flavor").__name__)
         self.assertIs(None, env.get_constraint("no_constraint"))
 
+    def test_event_sinks(self):
+        env = environment.Environment(
+            {"event_sinks": [{"type": "zaqar-queue", "target": "myqueue"}]})
+        self.assertEqual([{"type": "zaqar-queue", "target": "myqueue"}],
+                         env.user_env_as_dict()["event_sinks"])
+        sinks = env.get_event_sinks()
+        self.assertEqual(1, len(sinks))
+        self.assertEqual("myqueue", sinks[0]._target)
+
+    def test_event_sinks_load(self):
+        env = environment.Environment()
+        self.assertEqual([], env.get_event_sinks())
+        env.load(
+            {"event_sinks": [{"type": "zaqar-queue", "target": "myqueue"}]})
+        self.assertEqual([{"type": "zaqar-queue", "target": "myqueue"}],
+                         env.user_env_as_dict()["event_sinks"])
+
 
 class EnvironmentDuplicateTest(common.HeatTestCase):
 
@@ -290,8 +326,12 @@ class GlobalEnvLoadingTest(common.HeatTestCase):
         m_ldir.assert_called_once_with(env_dir + '/*')
 
     def test_continue_on_ioerror(self):
-        """assert we get all files processed even if there are
-        processing exceptions.
+        """Assert we get all files processed.
+
+        Assert we get all files processed even if there are processing
+        exceptions.
+
+        Test uses IOError as side effect of mock open.
         """
         with mock.patch('glob.glob') as m_ldir:
             m_ldir.return_value = ['/etc_etc/heat/environment.d/a.yaml',
@@ -313,8 +353,12 @@ class GlobalEnvLoadingTest(common.HeatTestCase):
         self.assertEqual(expected, m_open.call_args_list)
 
     def test_continue_on_parse_error(self):
-        """assert we get all files processed even if there are
-        processing exceptions.
+        """Assert we get all files processed.
+
+        Assert we get all files processed even if there are processing
+        exceptions.
+
+        Test checks case when env content is incorrect.
         """
         with mock.patch('glob.glob') as m_ldir:
             m_ldir.return_value = ['/etc_etc/heat/environment.d/a.yaml',
@@ -378,7 +422,7 @@ class GlobalEnvLoadingTest(common.HeatTestCase):
         resources._load_global_environment(g_env)
 
         # 3. assert our resource is in now gone.
-        self.assertRaises(exception.ResourceTypeNotFound,
+        self.assertRaises(exception.EntityNotFound,
                           g_env.get_resource_info, 'OS::Nova::Server')
 
         # 4. make sure we haven't removed something we shouldn't have
@@ -404,7 +448,7 @@ class GlobalEnvLoadingTest(common.HeatTestCase):
         resources._load_global_environment(g_env)
 
         # 3. assert our resources are now gone.
-        self.assertRaises(exception.ResourceTypeNotFound,
+        self.assertRaises(exception.EntityNotFound,
                           g_env.get_resource_info, 'AWS::EC2::Instance')
 
         # 4. make sure we haven't removed something we shouldn't have
@@ -467,6 +511,7 @@ class ChildEnvTest(common.HeatTestCase):
         expected = {'parameters': new_params,
                     'encrypted_param_names': [],
                     'parameter_defaults': {},
+                    'event_sinks': [],
                     'resource_registry': {'resources': {}}}
         cenv = environment.get_child_environment(penv, new_params)
         self.assertEqual(expected, cenv.user_env_as_dict())
@@ -476,6 +521,7 @@ class ChildEnvTest(common.HeatTestCase):
         penv = environment.Environment()
         expected = {'parameter_defaults': {},
                     'encrypted_param_names': [],
+                    'event_sinks': [],
                     'resource_registry': {'resources': {}}}
         expected.update(new_params)
         cenv = environment.get_child_environment(penv, new_params)
@@ -487,6 +533,7 @@ class ChildEnvTest(common.HeatTestCase):
         penv = environment.Environment(env=parent_params)
         expected = {'parameter_defaults': {},
                     'encrypted_param_names': [],
+                    'event_sinks': [],
                     'resource_registry': {'resources': {}}}
         expected.update(new_params)
         cenv = environment.get_child_environment(penv, new_params)
@@ -516,7 +563,7 @@ class ChildEnvTest(common.HeatTestCase):
         self.assertIsNotNone(victim)
         cenv = environment.get_child_environment(penv, None,
                                                  item_to_remove=victim)
-        self.assertRaises(exception.ResourceTypeNotFound,
+        self.assertRaises(exception.EntityNotFound,
                           cenv.get_resource_info,
                           'OS::Food', resource_name='abc')
         self.assertNotIn('OS::Food',
@@ -582,7 +629,7 @@ class ChildEnvTest(common.HeatTestCase):
         self.assertIn('hooks', resources['nested_res'])
         self.assertIsNotNone(
             cenv.get_resource_info('OS::Food', resource_name='abc'))
-        self.assertRaises(exception.ResourceTypeNotFound,
+        self.assertRaises(exception.EntityNotFound,
                           cenv.get_resource_info,
                           'OS::Fruit', resource_name='a')
         res = cenv.get_resource_info('OS::Fruit', resource_name='b')
@@ -732,7 +779,9 @@ class ResourceRegistryTest(common.HeatTestCase):
 
         registry = environment.ResourceRegistry(None, {})
         msg = ('Invalid hook type "invalid-type" for resource breakpoint, '
-               'acceptable hook types are: (\'pre-create\', \'pre-update\')')
+               'acceptable hook types are: (\'pre-create\', \'pre-update\', '
+               '\'pre-delete\', \'post-create\', \'post-update\', '
+               '\'post-delete\')')
         ex = self.assertRaises(exception.InvalidBreakPointHook,
                                registry.load, {'resources': resources})
         self.assertEqual(msg, six.text_type(ex))
@@ -779,6 +828,15 @@ class ResourceRegistryTest(common.HeatTestCase):
         self.assertIn('ResourceTypeUnSupportedLiberty', types)
         self.assertIn('GenericResourceType', types)
 
+    def test_list_type_with_is_available_exception(self):
+        registry = resources.global_env().registry
+        self.patchobject(
+            generic_resource.GenericResource,
+            'is_service_available',
+            side_effect=exception.ClientNotAvailable(client_name='generic'))
+        types = registry.get_types(utils.dummy_context())
+        self.assertNotIn('GenericResourceType', types)
+
     def test_list_type_with_invalid_type_name(self):
         registry = resources.global_env().registry
         types = registry.get_types(type_name="r'[^\+]'")
@@ -804,18 +862,23 @@ class ResourceRegistryTest(common.HeatTestCase):
 
 class HookMatchTest(common.HeatTestCase):
 
+    scenarios = [(hook_type, {'hook': hook_type}) for hook_type in
+                 environment.HOOK_TYPES]
+
     def test_plain_matches(self):
+        other_hook = next(hook for hook in environment.HOOK_TYPES
+                          if hook != self.hook)
         resources = {
             u'a': {
                 u'OS::Fruit': u'apples.yaml',
-                u'hooks': [u'pre-create', u'pre-update'],
+                u'hooks': [self.hook, other_hook]
             },
             u'b': {
                 u'OS::Food': u'fruity.yaml',
             },
             u'nested': {
                 u'res': {
-                    u'hooks': 'pre-create',
+                    u'hooks': self.hook,
                 },
             },
         }
@@ -824,77 +887,154 @@ class HookMatchTest(common.HeatTestCase):
             u'OS::Fruit': u'apples.yaml',
             'resources': resources})
 
-        self.assertTrue(registry.matches_hook(
-            'a', environment.HOOK_PRE_CREATE))
-        self.assertFalse(registry.matches_hook(
-            'b', environment.HOOK_PRE_CREATE))
-        self.assertFalse(registry.matches_hook(
-            'OS::Fruit', environment.HOOK_PRE_CREATE))
-        self.assertFalse(registry.matches_hook(
-            'res', environment.HOOK_PRE_CREATE))
-        self.assertFalse(registry.matches_hook(
-            'unknown', environment.HOOK_PRE_CREATE))
+        self.assertTrue(registry.matches_hook('a', self.hook))
+        self.assertFalse(registry.matches_hook('b', self.hook))
+        self.assertFalse(registry.matches_hook('OS::Fruit', self.hook))
+        self.assertFalse(registry.matches_hook('res', self.hook))
+        self.assertFalse(registry.matches_hook('unknown', self.hook))
+
+    def test_wildcard_matches(self):
+        other_hook = next(hook for hook in environment.HOOK_TYPES
+                          if hook != self.hook)
+        resources = {
+            u'prefix_*': {
+                u'hooks': self.hook
+            },
+            u'*_suffix': {
+                u'hooks': self.hook
+            },
+            u'*': {
+                u'hooks': other_hook
+            },
+        }
+        registry = environment.ResourceRegistry(None, {})
+        registry.load({'resources': resources})
+
+        self.assertTrue(registry.matches_hook('prefix_', self.hook))
+        self.assertTrue(registry.matches_hook('prefix_some', self.hook))
+        self.assertFalse(registry.matches_hook('some_prefix', self.hook))
+
+        self.assertTrue(registry.matches_hook('_suffix', self.hook))
+        self.assertTrue(registry.matches_hook('some_suffix', self.hook))
+        self.assertFalse(registry.matches_hook('_suffix_blah', self.hook))
+
+        self.assertTrue(registry.matches_hook('some_prefix', other_hook))
+        self.assertTrue(registry.matches_hook('_suffix_blah', other_hook))
+
+    def test_hook_types(self):
+        resources = {
+            u'hook': {
+                u'hooks': self.hook
+            },
+            u'not-hook': {
+                u'hooks': [hook for hook in environment.HOOK_TYPES if hook !=
+                           self.hook]
+            },
+            u'all': {
+                u'hooks': environment.HOOK_TYPES
+            },
+        }
+        registry = environment.ResourceRegistry(None, {})
+        registry.load({'resources': resources})
+
+        self.assertTrue(registry.matches_hook('hook', self.hook))
+        self.assertFalse(registry.matches_hook('not-hook', self.hook))
+        self.assertTrue(registry.matches_hook('all', self.hook))
+
+
+class ActionRestrictedTest(common.HeatTestCase):
+
+    def test_plain_matches(self):
+        resources = {
+            u'a': {
+                u'OS::Fruit': u'apples.yaml',
+                u'restricted_actions': [u'update', u'replace'],
+            },
+            u'b': {
+                u'OS::Food': u'fruity.yaml',
+            },
+            u'nested': {
+                u'res': {
+                    u'restricted_actions': 'update',
+                },
+            },
+        }
+        registry = environment.ResourceRegistry(None, {})
+        registry.load({
+            u'OS::Fruit': u'apples.yaml',
+            'resources': resources})
+
+        self.assertIn(environment.UPDATE,
+                      registry.get_rsrc_restricted_actions('a'))
+        self.assertNotIn(environment.UPDATE,
+                         registry.get_rsrc_restricted_actions('b'))
+        self.assertNotIn(environment.UPDATE,
+                         registry.get_rsrc_restricted_actions('OS::Fruit'))
+        self.assertNotIn(environment.UPDATE,
+                         registry.get_rsrc_restricted_actions('res'))
+        self.assertNotIn(environment.UPDATE,
+                         registry.get_rsrc_restricted_actions('unknown'))
 
     def test_wildcard_matches(self):
         resources = {
             u'prefix_*': {
-                u'hooks': 'pre-create',
+                u'restricted_actions': 'update',
             },
             u'*_suffix': {
-                u'hooks': 'pre-create',
+                u'restricted_actions': 'update',
             },
             u'*': {
-                u'hooks': 'pre-update',
+                u'restricted_actions': 'replace',
             },
         }
         registry = environment.ResourceRegistry(None, {})
         registry.load({'resources': resources})
 
-        self.assertTrue(registry.matches_hook(
-            'prefix_', environment.HOOK_PRE_CREATE))
-        self.assertTrue(registry.matches_hook(
-            'prefix_some', environment.HOOK_PRE_CREATE))
-        self.assertFalse(registry.matches_hook(
-            'some_prefix', environment.HOOK_PRE_CREATE))
+        self.assertIn(environment.UPDATE,
+                      registry.get_rsrc_restricted_actions('prefix_'))
+        self.assertIn(environment.UPDATE,
+                      registry.get_rsrc_restricted_actions('prefix_some'))
+        self.assertNotIn(environment.UPDATE,
+                         registry.get_rsrc_restricted_actions('some_prefix'))
 
-        self.assertTrue(registry.matches_hook(
-            '_suffix', environment.HOOK_PRE_CREATE))
-        self.assertTrue(registry.matches_hook(
-            'some_suffix', environment.HOOK_PRE_CREATE))
-        self.assertFalse(registry.matches_hook(
-            '_suffix_blah', environment.HOOK_PRE_CREATE))
+        self.assertIn(environment.UPDATE,
+                      registry.get_rsrc_restricted_actions('_suffix'))
+        self.assertIn(environment.UPDATE,
+                      registry.get_rsrc_restricted_actions('some_suffix'))
+        self.assertNotIn(environment.UPDATE,
+                         registry.get_rsrc_restricted_actions('_suffix_blah'))
 
-        self.assertTrue(registry.matches_hook(
-            'some_prefix', environment.HOOK_PRE_UPDATE))
-        self.assertTrue(registry.matches_hook(
-            '_suffix_blah', environment.HOOK_PRE_UPDATE))
+        self.assertIn(environment.REPLACE,
+                      registry.get_rsrc_restricted_actions('some_prefix'))
+        self.assertIn(environment.REPLACE,
+                      registry.get_rsrc_restricted_actions('_suffix_blah'))
 
-    def test_hook_types(self):
+    def test_restricted_action_types(self):
         resources = {
-            u'pre_create': {
-                u'hooks': 'pre-create',
+            u'update': {
+                u'restricted_actions': 'update',
             },
-            u'pre_update': {
-                u'hooks': 'pre-update',
+            u'replace': {
+                u'restricted_actions': 'replace',
             },
-            u'both': {
-                u'hooks': ['pre-create', 'pre-update'],
+            u'all': {
+                u'restricted_actions': ['update', 'replace'],
             },
         }
         registry = environment.ResourceRegistry(None, {})
         registry.load({'resources': resources})
 
-        self.assertTrue(registry.matches_hook(
-            'pre_create', environment.HOOK_PRE_CREATE))
-        self.assertFalse(registry.matches_hook(
-            'pre_create', environment.HOOK_PRE_UPDATE))
+        self.assertIn(environment.UPDATE,
+                      registry.get_rsrc_restricted_actions('update'))
+        self.assertNotIn(environment.UPDATE,
+                         registry.get_rsrc_restricted_actions('replace'))
 
-        self.assertTrue(registry.matches_hook(
-            'pre_update', environment.HOOK_PRE_UPDATE))
-        self.assertFalse(registry.matches_hook(
-            'pre_update', environment.HOOK_PRE_CREATE))
+        self.assertIn(environment.REPLACE,
+                      registry.get_rsrc_restricted_actions('replace'))
+        self.assertNotIn(environment.REPLACE,
+                         registry.get_rsrc_restricted_actions('update'))
 
-        self.assertTrue(registry.matches_hook(
-            'both', environment.HOOK_PRE_CREATE))
-        self.assertTrue(registry.matches_hook(
-            'both', environment.HOOK_PRE_UPDATE))
+        self.assertIn(environment.UPDATE,
+                      registry.get_rsrc_restricted_actions('all'))
+        self.assertIn(environment.REPLACE,
+                      registry.get_rsrc_restricted_actions('all'))
