@@ -562,6 +562,26 @@ class HOTemplateTest(common.HeatTestCase):
 
         self.assertEqual(snippet_resolved, self.resolve(snippet, tmpl))
 
+    def test_str_replace_order(self, tpl=hot_tpl_empty):
+        """Test str_replace function substitution order."""
+
+        snippet = {'str_replace': {'template': '1234567890',
+                                   'params': {'1': 'a',
+                                              '12': 'b',
+                                              '123': 'c',
+                                              '1234': 'd',
+                                              '12345': 'e',
+                                              '123456': 'f',
+                                              '1234567': 'g'}}}
+
+        tmpl = template.Template(tpl)
+
+        self.assertEqual('g890', self.resolve(snippet, tmpl))
+
+    def test_str_replace_liberty_order(self):
+        """Test str_replace function substitution order."""
+        self.test_str_replace_order(hot_liberty_tpl_empty)
+
     def test_str_replace_syntax(self):
         """Test str_replace function syntax.
 
@@ -1021,6 +1041,16 @@ class HOTemplateTest(common.HeatTestCase):
         self.assertIn('Incorrect arguments to \"str_split\"',
                       six.text_type(exc))
 
+    def test_str_split_none_string_to_split(self):
+        tmpl = template.Template(hot_liberty_tpl_empty)
+        snippet = {'str_split': ['.', None]}
+        self.assertIsNone(self.resolve(snippet, tmpl))
+
+    def test_str_split_none_delim(self):
+        tmpl = template.Template(hot_liberty_tpl_empty)
+        snippet = {'str_split': [None, 'check']}
+        self.assertEqual(['check'], self.resolve(snippet, tmpl))
+
     def test_prevent_parameters_access(self):
         """Check parameters section inaccessible using the template as a dict.
 
@@ -1661,6 +1691,10 @@ class StackParametersTest(common.HeatTestCase):
          dict(params={'list': 'foo,bar'},
               snippet={'properties': {'prop1': {'get_param': ['list', 1]}}},
               expected={'properties': {'prop1': 'bar'}})),
+        ('get_list_attr_string_index',
+         dict(params={'list': 'foo,bar'},
+              snippet={'properties': {'prop1': {'get_param': ['list', '1']}}},
+              expected={'properties': {'prop1': 'bar'}})),
         ('get_flat_dict_attr',
          dict(params={'flat_dict':
                       {'key1': 'val1', 'key2': 'val2', 'key3': 'val3'}},
