@@ -11,14 +11,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
 from oslo_serialization import jsonutils
 import six
 
 from heat.common import exception
 from heat.engine.cfn import functions as cfn_funcs
 from heat.engine import constraints
-from heat.engine.hot import functions as hot_funcs
 from heat.engine.hot import parameters as hot_param
 from heat.engine import parameters
 from heat.engine import plugin_manager
@@ -346,7 +344,7 @@ class PropertySchemaTest(common.HeatTestCase):
 
         self.assertEqual(properties.Schema.STRING, schema.type)
         self.assertEqual(description, schema.description)
-        self.assertEqual("m1.large", schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertEqual(1, len(schema.constraints))
 
@@ -355,6 +353,9 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(tuple(allowed_values), allowed_constraint.allowed)
         self.assertEqual(constraint_desc, allowed_constraint.description)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_string_allowed_pattern(self):
         description = "WebServer EC2 instance type"
         allowed_pattern = "[A-Za-z0-9.]*"
@@ -371,7 +372,7 @@ class PropertySchemaTest(common.HeatTestCase):
 
         self.assertEqual(properties.Schema.STRING, schema.type)
         self.assertEqual(description, schema.description)
-        self.assertEqual("m1.large", schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertEqual(1, len(schema.constraints))
 
@@ -380,6 +381,9 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(allowed_pattern, allowed_constraint.pattern)
         self.assertEqual(constraint_desc, allowed_constraint.description)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_string_multi_constraints(self):
         description = "WebServer EC2 instance type"
         allowed_pattern = "[A-Za-z0-9.]*"
@@ -397,7 +401,7 @@ class PropertySchemaTest(common.HeatTestCase):
 
         self.assertEqual(properties.Schema.STRING, schema.type)
         self.assertEqual(description, schema.description)
-        self.assertEqual("m1.large", schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertEqual(2, len(schema.constraints))
 
@@ -409,6 +413,9 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(allowed_pattern, allowed_constraint.pattern)
         self.assertEqual(constraint_desc, allowed_constraint.description)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_param_string_min_len(self):
         param = parameters.Schema.from_dict('name', {
             "Description": "WebServer EC2 instance type",
@@ -419,6 +426,7 @@ class PropertySchemaTest(common.HeatTestCase):
         schema = properties.Schema.from_parameter(param)
 
         self.assertFalse(schema.required)
+        self.assertIsNone(schema.default)
         self.assertEqual(1, len(schema.constraints))
 
         len_constraint = schema.constraints[0]
@@ -426,6 +434,9 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(7, len_constraint.min)
         self.assertIsNone(len_constraint.max)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_param_string_max_len(self):
         param = parameters.Schema.from_dict('name', {
             "Description": "WebServer EC2 instance type",
@@ -436,6 +447,7 @@ class PropertySchemaTest(common.HeatTestCase):
         schema = properties.Schema.from_parameter(param)
 
         self.assertFalse(schema.required)
+        self.assertIsNone(schema.default)
         self.assertEqual(1, len(schema.constraints))
 
         len_constraint = schema.constraints[0]
@@ -443,6 +455,9 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertIsNone(len_constraint.min)
         self.assertEqual(11, len_constraint.max)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_param_string_min_max_len(self):
         param = parameters.Schema.from_dict('name', {
             "Description": "WebServer EC2 instance type",
@@ -454,6 +469,7 @@ class PropertySchemaTest(common.HeatTestCase):
         schema = properties.Schema.from_parameter(param)
 
         self.assertFalse(schema.required)
+        self.assertIsNone(schema.default)
         self.assertEqual(1, len(schema.constraints))
 
         len_constraint = schema.constraints[0]
@@ -461,6 +477,9 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(7, len_constraint.min)
         self.assertEqual(11, len_constraint.max)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_param_no_default(self):
         param = parameters.Schema.from_dict('name', {
             "Description": "WebServer EC2 instance type",
@@ -473,18 +492,20 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(0, len(schema.constraints))
         self.assertFalse(schema.allow_conversion)
 
+        props = properties.Properties({'name': schema}, {'name': 'm1.large'})
+        props.validate()
+
     def test_from_number_param_min(self):
-        default = "42"
         param = parameters.Schema.from_dict('name', {
             "Type": "Number",
-            "Default": default,
+            "Default": "42",
             "MinValue": "10",
         })
 
         schema = properties.Schema.from_parameter(param)
 
         self.assertEqual(properties.Schema.NUMBER, schema.type)
-        self.assertEqual(default, schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertEqual(1, len(schema.constraints))
 
@@ -493,18 +514,20 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(10, value_constraint.min)
         self.assertIsNone(value_constraint.max)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_number_param_max(self):
-        default = "42"
         param = parameters.Schema.from_dict('name', {
             "Type": "Number",
-            "Default": default,
+            "Default": "42",
             "MaxValue": "100",
         })
 
         schema = properties.Schema.from_parameter(param)
 
         self.assertEqual(properties.Schema.NUMBER, schema.type)
-        self.assertEqual(default, schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertEqual(1, len(schema.constraints))
 
@@ -513,11 +536,13 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertIsNone(value_constraint.min)
         self.assertEqual(100, value_constraint.max)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_number_param_min_max(self):
-        default = "42"
         param = parameters.Schema.from_dict('name', {
             "Type": "Number",
-            "Default": default,
+            "Default": "42",
             "MinValue": "10",
             "MaxValue": "100",
         })
@@ -525,7 +550,7 @@ class PropertySchemaTest(common.HeatTestCase):
         schema = properties.Schema.from_parameter(param)
 
         self.assertEqual(properties.Schema.NUMBER, schema.type)
-        self.assertEqual(default, schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertEqual(1, len(schema.constraints))
 
@@ -534,12 +559,14 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(10, value_constraint.min)
         self.assertEqual(100, value_constraint.max)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_number_param_allowed_vals(self):
-        default = "42"
         constraint_desc = "The quick brown fox jumps over the lazy dog."
         param = parameters.Schema.from_dict('name', {
             "Type": "Number",
-            "Default": default,
+            "Default": "42",
             "AllowedValues": ["10", "42", "100"],
             "ConstraintDescription": constraint_desc,
         })
@@ -547,7 +574,7 @@ class PropertySchemaTest(common.HeatTestCase):
         schema = properties.Schema.from_parameter(param)
 
         self.assertEqual(properties.Schema.NUMBER, schema.type)
-        self.assertEqual(default, schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertEqual(1, len(schema.constraints))
         self.assertFalse(schema.allow_conversion)
@@ -557,6 +584,9 @@ class PropertySchemaTest(common.HeatTestCase):
         self.assertEqual(('10', '42', '100'), allowed_constraint.allowed)
         self.assertEqual(constraint_desc, allowed_constraint.description)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_list_param(self):
         param = parameters.Schema.from_dict('name', {
             "Type": "CommaDelimitedList",
@@ -566,10 +596,13 @@ class PropertySchemaTest(common.HeatTestCase):
         schema = properties.Schema.from_parameter(param)
 
         self.assertEqual(properties.Schema.LIST, schema.type)
-        self.assertEqual("foo,bar,baz", schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertFalse(schema.allow_conversion)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_from_json_param(self):
         param = parameters.Schema.from_dict('name', {
             "Type": "Json",
@@ -579,11 +612,13 @@ class PropertySchemaTest(common.HeatTestCase):
         schema = properties.Schema.from_parameter(param)
 
         self.assertEqual(properties.Schema.MAP, schema.type)
-        self.assertEqual({"foo": "bar", "blarg": "wibble"},
-                         schema.default)
+        self.assertIsNone(schema.default)
         self.assertFalse(schema.required)
         self.assertTrue(schema.allow_conversion)
 
+        props = properties.Properties({'test': schema}, {})
+        props.validate()
+
     def test_no_mismatch_in_update_policy(self):
         manager = plugin_manager.PluginManager('heat.engine.resources')
         resource_mapping = plugin_manager.PluginMapping('resource')
@@ -1014,7 +1049,10 @@ class PropertiesTest(common.HeatTestCase):
             'bad_int': 'foo',
             'default_override': 21,
         }
-        double = lambda d: d * 2
+
+        def double(d):
+            return d * 2
+
         self.props = properties.Properties(schema, data, double, 'wibble')
 
     def test_integer_good(self):
@@ -1024,7 +1062,7 @@ class PropertiesTest(common.HeatTestCase):
         self.assertEqual('foofoo', self.props['string'])
 
     def test_bool_not_str(self):
-        self.assertEqual(False, self.props['default_bool'])
+        self.assertFalse(self.props['default_bool'])
 
     def test_missing_required(self):
         self.assertRaises(ValueError, self.props.get, 'required_int')
@@ -1287,7 +1325,6 @@ class PropertiesTest(common.HeatTestCase):
             "DBUsername": {
                 "type": "string",
                 "description": "The WordPress database admin account username",
-                "default": "admin",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1303,7 +1340,6 @@ class PropertiesTest(common.HeatTestCase):
             "LinuxDistribution": {
                 "type": "string",
                 "description": "Distribution of choice",
-                "default": "F17",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1315,7 +1351,6 @@ class PropertiesTest(common.HeatTestCase):
             "InstanceType": {
                 "type": "string",
                 "description": "WebServer EC2 instance type",
-                "default": "m1.large",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1336,7 +1371,6 @@ class PropertiesTest(common.HeatTestCase):
             "DBRootPassword": {
                 "type": "string",
                 "description": "Root password for MySQL",
-                "default": "admin",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1360,7 +1394,6 @@ class PropertiesTest(common.HeatTestCase):
             "DBPassword": {
                 "type": "string",
                 "description": "The WordPress database admin account password",
-                "default": "admin",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1376,7 +1409,6 @@ class PropertiesTest(common.HeatTestCase):
             "DBName": {
                 "type": "string",
                 "description": "The WordPress database name",
-                "default": "wordpress",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1490,7 +1522,6 @@ class PropertiesTest(common.HeatTestCase):
             "InstanceType": {
                 "type": "string",
                 "description": "WebServer EC2 instance type",
-                "default": "m1.large",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1503,7 +1534,6 @@ class PropertiesTest(common.HeatTestCase):
                 ]
             },
             "LinuxDistribution": {
-                "default": "F17",
                 "type": "string",
                 "description": "Distribution of choice",
                 "required": False,
@@ -1518,7 +1548,6 @@ class PropertiesTest(common.HeatTestCase):
             "DBName": {
                 "type": "string",
                 "description": "The WordPress database name",
-                "default": "wordpress",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1533,7 +1562,6 @@ class PropertiesTest(common.HeatTestCase):
             "DBUsername": {
                 "type": "string",
                 "description": "The WordPress database admin account username",
-                "default": "admin",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1548,7 +1576,6 @@ class PropertiesTest(common.HeatTestCase):
             "DBPassword": {
                 "type": "string",
                 "description": "The WordPress database admin account password",
-                "default": "admin",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1563,7 +1590,6 @@ class PropertiesTest(common.HeatTestCase):
             "DBRootPassword": {
                 "type": "string",
                 "description": "Root password for MySQL",
-                "default": "admin",
                 "required": False,
                 'update_allowed': True,
                 'immutable': False,
@@ -1879,579 +1905,3 @@ class PropertiesValidationTest(common.HeatTestCase):
             immutable=True)}
         props = properties.Properties(schema, {})
         self.assertRaises(exception.InvalidSchemaError, props.validate)
-
-
-class TestTranslationRule(common.HeatTestCase):
-
-    def test_translation_rule(self):
-        for r in properties.TranslationRule.RULE_KEYS:
-            props = properties.Properties({}, {})
-            rule = properties.TranslationRule(
-                props,
-                r,
-                ['any'],
-                ['value'] if r == 'Add' else 'value',
-                'value_name' if r == 'Replace' else None)
-            self.assertEqual(rule.properties, props)
-            self.assertEqual(rule.rule, r)
-            if r == 'Add':
-                self.assertEqual(['value'], rule.value)
-            else:
-                self.assertEqual('value', rule.value)
-            if r == 'Replace':
-                self.assertEqual('value_name', rule.value_name)
-            else:
-                self.assertIsNone(rule.value_name)
-
-    def test_invalid_translation_rule(self):
-        props = properties.Properties({}, {})
-        exc = self.assertRaises(ValueError,
-                                properties.TranslationRule, 'proppy', mock.ANY,
-                                mock.ANY)
-        self.assertEqual('Properties must be Properties type. '
-                         'Found %s.' % str, six.text_type(exc))
-
-        exc = self.assertRaises(ValueError,
-                                properties.TranslationRule,
-                                props,
-                                'EatTheCookie',
-                                mock.ANY,
-                                mock.ANY)
-        self.assertEqual('There is no rule EatTheCookie. List of allowed '
-                         'rules is: Add, Replace, Delete.',
-                         six.text_type(exc))
-
-        exc = self.assertRaises(ValueError,
-                                properties.TranslationRule,
-                                props,
-                                properties.TranslationRule.ADD,
-                                'networks.network',
-                                'value')
-        self.assertEqual('source_path should be a list with path instead of '
-                         '%s.' % str, six.text_type(exc))
-
-        exc = self.assertRaises(ValueError,
-                                properties.TranslationRule,
-                                props,
-                                properties.TranslationRule.ADD,
-                                [],
-                                mock.ANY)
-        self.assertEqual('source_path must be non-empty list with path.',
-                         six.text_type(exc))
-
-        exc = self.assertRaises(ValueError,
-                                properties.TranslationRule,
-                                props,
-                                properties.TranslationRule.ADD,
-                                ['any'],
-                                mock.ANY,
-                                'value_name')
-        self.assertEqual('Use value_name only for replacing list elements.',
-                         six.text_type(exc))
-
-        exc = self.assertRaises(ValueError,
-                                properties.TranslationRule,
-                                props,
-                                properties.TranslationRule.ADD,
-                                ['any'],
-                                'value')
-        self.assertEqual('value must be list type when rule is Add.',
-                         six.text_type(exc))
-
-    def test_add_rule_exist(self):
-        schema = {
-            'far': properties.Schema(
-                properties.Schema.LIST,
-                schema=properties.Schema(
-                    properties.Schema.MAP,
-                    schema={
-                        'red': properties.Schema(
-                            properties.Schema.STRING
-                        )
-                    }
-                )
-            ),
-            'bar': properties.Schema(
-                properties.Schema.STRING
-            )}
-
-        data = {
-            'far': [
-                {'red': 'blue'}
-            ],
-            'bar': 'dak'
-        }
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.ADD,
-                                          ['far'],
-                                          [{'red': props.get('bar')}])
-        rule.execute_rule()
-
-        self.assertIn({'red': 'dak'}, props.get('far'))
-
-    def test_add_rule_dont_exist(self):
-        schema = {
-            'far': properties.Schema(
-                properties.Schema.LIST,
-                schema=properties.Schema(
-                    properties.Schema.MAP,
-                    schema={
-                        'red': properties.Schema(
-                            properties.Schema.STRING
-                        )
-                    }
-                )
-            ),
-            'bar': properties.Schema(
-                properties.Schema.STRING
-            )}
-
-        data = {
-            'bar': 'dak'
-        }
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.ADD,
-                                          ['far'],
-                                          [{'red': props.get('bar')}])
-        rule.execute_rule()
-
-        self.assertEqual([{'red': 'dak'}], props.get('far'))
-
-    def test_add_rule_invalid(self):
-        schema = {
-            'far': properties.Schema(
-                properties.Schema.MAP,
-                schema={
-                    'red': properties.Schema(
-                        properties.Schema.STRING
-                    )
-                }
-            ),
-            'bar': properties.Schema(
-                properties.Schema.STRING
-            )}
-
-        data = {
-            'far': 'tran',
-            'bar': 'dak'
-        }
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.ADD,
-                                          ['far'],
-                                          [props.get('bar')])
-        exc = self.assertRaises(ValueError, rule.execute_rule)
-
-        self.assertEqual('Add rule must be used only for lists.',
-                         six.text_type(exc))
-
-    def test_replace_rule_map_exist(self):
-        schema = {
-            'far': properties.Schema(
-                properties.Schema.MAP,
-                schema={
-                    'red': properties.Schema(
-                        properties.Schema.STRING
-                    )
-                }
-            ),
-            'bar': properties.Schema(
-                properties.Schema.STRING
-            )}
-
-        data = {
-            'far': {'red': 'tran'},
-            'bar': 'dak'
-        }
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.REPLACE,
-                                          ['far', 'red'],
-                                          props.get('bar'))
-        rule.execute_rule()
-
-        self.assertEqual({'red': 'dak'}, props.get('far'))
-
-    def test_replace_rule_map_dont_exist(self):
-        schema = {
-            'far': properties.Schema(
-                properties.Schema.MAP,
-                schema={
-                    'red': properties.Schema(
-                        properties.Schema.STRING
-                    )
-                }
-            ),
-            'bar': properties.Schema(
-                properties.Schema.STRING
-            )}
-
-        data = {
-            'bar': 'dak'
-        }
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.REPLACE,
-                                          ['far', 'red'],
-                                          props.get('bar'))
-        rule.execute_rule()
-
-        self.assertEqual({'red': 'dak'}, props.get('far'))
-
-    def test_replace_rule_list_different(self):
-        schema = {
-            'far': properties.Schema(
-                properties.Schema.LIST,
-                schema=properties.Schema(
-                    properties.Schema.MAP,
-                    schema={
-                        'red': properties.Schema(
-                            properties.Schema.STRING
-                        )
-                    }
-                )
-            ),
-            'bar': properties.Schema(
-                properties.Schema.STRING
-            )}
-
-        data = {
-            'far': [{'red': 'blue'},
-                    {'red': 'roses'}],
-            'bar': 'dak'
-        }
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.REPLACE,
-                                          ['far', 'red'],
-                                          props.get('bar'))
-        rule.execute_rule()
-
-        self.assertEqual([{'red': 'dak'}, {'red': 'dak'}], props.get('far'))
-
-    def test_replace_rule_list_same(self):
-        schema = {
-            'far': properties.Schema(
-                properties.Schema.LIST,
-                schema=properties.Schema(
-                    properties.Schema.MAP,
-                    schema={
-                        'red': properties.Schema(
-                            properties.Schema.STRING
-                        ),
-                        'blue': properties.Schema(
-                            properties.Schema.STRING
-                        )
-                    }
-                )
-            )}
-
-        data = {
-            'far': [{'blue': 'white'},
-                    {'red': 'roses'}]
-        }
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.REPLACE,
-                                          ['far', 'red'],
-                                          None,
-                                          'blue')
-        rule.execute_rule()
-
-        self.assertEqual([{'red': 'white', 'blue': None},
-                          {'blue': None, 'red': 'roses'}],
-                         props.get('far'))
-
-    def test_replace_rule_str(self):
-        schema = {
-            'far': properties.Schema(properties.Schema.STRING),
-            'bar': properties.Schema(properties.Schema.STRING)
-        }
-
-        data = {'far': 'one', 'bar': 'two'}
-
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.REPLACE,
-                                          ['bar'],
-                                          props.get('far'))
-        rule.execute_rule()
-
-        self.assertEqual('one', props.get('bar'))
-        self.assertEqual('one', props.get('far'))
-
-    def test_replace_rule_str_value_path_error(self):
-        schema = {
-            'far': properties.Schema(properties.Schema.STRING),
-            'bar': properties.Schema(properties.Schema.STRING)
-        }
-
-        data = {'far': 'one', 'bar': 'two'}
-
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.REPLACE,
-                                          ['bar'],
-                                          value_path=['far'])
-        ex = self.assertRaises(ValueError, rule.execute_rule)
-        self.assertEqual('Cannot use bar and far at the same time.',
-                         six.text_type(ex))
-
-    def test_replace_rule_str_value_path(self):
-        schema = {
-            'far': properties.Schema(properties.Schema.STRING),
-            'bar': properties.Schema(properties.Schema.STRING)
-        }
-
-        data = {'far': 'one'}
-
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.REPLACE,
-                                          ['bar'],
-                                          value_path=['far'])
-        rule.execute_rule()
-
-        self.assertEqual('one', props.get('bar'))
-        self.assertIsNone(props.get('far'))
-
-    def test_replace_rule_str_invalid(self):
-        schema = {
-            'far': properties.Schema(properties.Schema.STRING),
-            'bar': properties.Schema(properties.Schema.INTEGER)
-        }
-
-        data = {'far': 'one', 'bar': 2}
-
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.REPLACE,
-                                          ['bar'],
-                                          props.get('far'))
-        rule.execute_rule()
-
-        exc = self.assertRaises(exception.StackValidationFailed,
-                                props.validate)
-        self.assertEqual("Property error: bar: Value 'one' is not an integer",
-                         six.text_type(exc))
-
-    def test_delete_rule_list(self):
-        schema = {
-            'far': properties.Schema(
-                properties.Schema.LIST,
-                schema=properties.Schema(
-                    properties.Schema.MAP,
-                    schema={
-                        'red': properties.Schema(
-                            properties.Schema.STRING
-                        )
-                    }
-                )
-            )}
-
-        data = {
-            'far': [{'red': 'blue'},
-                    {'red': 'roses'}],
-        }
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.DELETE,
-                                          ['far', 'red'])
-        rule.execute_rule()
-
-        self.assertEqual([{'red': None}, {'red': None}], props.get('far'))
-
-    def test_delete_rule_other(self):
-        schema = {
-            'far': properties.Schema(properties.Schema.STRING)
-        }
-
-        data = {'far': 'one'}
-
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.DELETE,
-                                          ['far'])
-        rule.execute_rule()
-
-        self.assertIsNone(props.get('far'))
-
-    def test_property_json_param_correct_translation(self):
-        """Test case when property with sub-schema takes json param."""
-        schema = {
-            'far': properties.Schema(properties.Schema.MAP,
-                                     schema={
-                                         'bar': properties.Schema(
-                                             properties.Schema.STRING,
-                                         ),
-                                         'dar': properties.Schema(
-                                             properties.Schema.STRING
-                                         )
-                                     })
-        }
-
-        class DummyStack(dict):
-            @property
-            def parameters(self):
-                return mock.Mock()
-
-        param = hot_funcs.GetParam(DummyStack(json_far='json_far'),
-                                   'get_param',
-                                   'json_far')
-        param.parameters = {
-            'json_far': parameters.JsonParam(
-                'json_far',
-                {'Type': 'Json'},
-                '{"dar": "rad"}').value()}
-        data = {'far': param}
-
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.REPLACE,
-                                          ['far', 'bar'],
-                                          value_path=['far', 'dar'])
-
-        rule.execute_rule()
-
-        self.assertEqual('rad', props.get('far').get('bar'))
-
-    def test_property_json_param_to_list_correct_translation(self):
-        """Test case when list property with sub-schema takes json param."""
-        schema = {
-            'far': properties.Schema(properties.Schema.LIST,
-                                     schema=properties.Schema(
-                                         properties.Schema.MAP,
-                                         schema={
-                                             'bar': properties.Schema(
-                                                 properties.Schema.STRING,
-                                             ),
-                                             'dar': properties.Schema(
-                                                 properties.Schema.STRING
-                                             )
-                                         }
-                                     ))
-        }
-
-        class DummyStack(dict):
-            @property
-            def parameters(self):
-                return mock.Mock()
-
-        param = hot_funcs.GetParam(DummyStack(json_far='json_far'),
-                                   'get_param',
-                                   'json_far')
-        param.parameters = {
-            'json_far': parameters.JsonParam(
-                'json_far',
-                {'Type': 'Json'},
-                '{"dar": "rad"}').value()}
-        data = {'far': [param]}
-
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.REPLACE,
-                                          ['far', 'bar'],
-                                          value_name='dar')
-
-        rule.execute_rule()
-
-        self.assertEqual([{'dar': None, 'bar': 'rad'}], props.get('far'))
-
-    def test_property_commadelimitedlist_param_correct_translation(self):
-        """Test when property with sub-schema takes comma_delimited_list."""
-        schema = {
-            'far': properties.Schema(
-                properties.Schema.LIST,
-                schema=properties.Schema(
-                    properties.Schema.STRING,
-                )
-            ),
-            'boo': properties.Schema(
-                properties.Schema.STRING
-            )}
-
-        class DummyStack(dict):
-            @property
-            def parameters(self):
-                return mock.Mock()
-
-        param = hot_funcs.GetParam(DummyStack(list_far='list_far'),
-                                   'get_param',
-                                   'list_far')
-        param.parameters = {
-            'list_far': parameters.CommaDelimitedListParam(
-                'list_far',
-                {'Type': 'CommaDelimitedList'},
-                "white,roses").value()}
-        data = {'far': param, 'boo': 'chrysanthemums'}
-
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.ADD,
-                                          ['far'],
-                                          [props.get('boo')])
-        rule.execute_rule()
-
-        self.assertEqual(['white', 'roses', 'chrysanthemums'],
-                         props.get('far'))
-
-    def test_property_no_translation_removed_function(self):
-        """Test case when list property with sub-schema takes json param."""
-        schema = {
-            'far': properties.Schema(properties.Schema.LIST,
-                                     schema=properties.Schema(
-                                         properties.Schema.MAP,
-                                         schema={
-                                             'bar': properties.Schema(
-                                                 properties.Schema.STRING,
-                                             ),
-                                             'dar': properties.Schema(
-                                                 properties.Schema.STRING
-                                             )
-                                         }
-                                     ))
-        }
-
-        class DummyStack(dict):
-            @property
-            def parameters(self):
-                return mock.Mock()
-
-        param = hot_funcs.Removed(DummyStack(json_far='json_far'),
-                                  'Ref',
-                                  'json_far')
-        param.parameters = {
-            'json_far': parameters.JsonParam(
-                'json_far',
-                {'Type': 'Json'},
-                '{"dar": "rad"}').value()}
-        data = {'far': [param]}
-
-        props = properties.Properties(schema, data)
-
-        rule = properties.TranslationRule(props,
-                                          properties.TranslationRule.REPLACE,
-                                          ['far', 'bar'],
-                                          value_name='dar')
-
-        rule.execute_rule()
-
-        self.assertEqual([param], props.data.get('far'))
