@@ -20,8 +20,9 @@ Unit Tests for heat.rpc.client
 import copy
 
 import mock
+from mox import stubout
 from oslo_messaging._drivers import common as rpc_common
-import stubout
+from oslo_utils import reflection
 
 from heat.common import exception
 from heat.common import identifier
@@ -41,8 +42,7 @@ class EngineRpcAPITestCase(common.HeatTestCase):
         self.rpcapi = rpc_client.EngineClient()
 
     def _to_remote_error(self, error):
-        """Converts the given exception to the one with the _Remote suffix.
-        """
+        """Converts the given exception to the one with the _Remote suffix."""
         exc_info = (type(error), error, None)
         serialized = rpc_common.serialize_remote_exception(exc_info)
         remote_error = rpc_common.deserialize_remote_exception(
@@ -54,7 +54,8 @@ class EngineRpcAPITestCase(common.HeatTestCase):
         self.assertEqual('NotFound', self.rpcapi.local_error_name(ex))
 
         exr = self._to_remote_error(ex)
-        self.assertEqual('NotFound_Remote', exr.__class__.__name__)
+        self.assertEqual('NotFound_Remote',
+                         reflection.get_class_name(exr, fully_qualified=False))
         self.assertEqual('NotFound', self.rpcapi.local_error_name(exr))
 
     def test_ignore_error_named(self):
@@ -79,7 +80,6 @@ class EngineRpcAPITestCase(common.HeatTestCase):
         expected_retval = 'foo' if method == 'call' else None
 
         kwargs.pop('version', None)
-
         if 'expected_message' in kwargs:
             expected_message = kwargs['expected_message']
             del kwargs['expected_message']
@@ -142,13 +142,15 @@ class EngineRpcAPITestCase(common.HeatTestCase):
                               stack_name='wordpress')
 
     def test_show_stack(self):
-        self._test_engine_api('show_stack', 'call', stack_identity='wordpress')
+        self._test_engine_api('show_stack', 'call', stack_identity='wordpress',
+                              resolve_outputs=True)
 
     def test_preview_stack(self):
         self._test_engine_api('preview_stack', 'call', stack_name='wordpress',
                               template={u'Foo': u'bar'},
                               params={u'InstanceType': u'm1.xlarge'},
                               files={u'a_file': u'the contents'},
+                              environment_files=['foo.yaml'],
                               args={'timeout_mins': u'30'})
 
     def test_create_stack(self):
@@ -156,6 +158,7 @@ class EngineRpcAPITestCase(common.HeatTestCase):
                       template={u'Foo': u'bar'},
                       params={u'InstanceType': u'm1.xlarge'},
                       files={u'a_file': u'the contents'},
+                      environment_files=['foo.yaml'],
                       args={'timeout_mins': u'30'})
         call_kwargs = copy.deepcopy(kwargs)
         call_kwargs['owner_id'] = None
@@ -173,6 +176,7 @@ class EngineRpcAPITestCase(common.HeatTestCase):
                               template={u'Foo': u'bar'},
                               params={u'InstanceType': u'm1.xlarge'},
                               files={},
+                              environment_files=['foo.yaml'],
                               args=mock.ANY)
 
     def test_preview_update_stack(self):
@@ -181,6 +185,7 @@ class EngineRpcAPITestCase(common.HeatTestCase):
                               template={u'Foo': u'bar'},
                               params={u'InstanceType': u'm1.xlarge'},
                               files={},
+                              environment_files=['foo.yaml'],
                               args=mock.ANY)
 
     def test_get_template(self):
@@ -200,7 +205,10 @@ class EngineRpcAPITestCase(common.HeatTestCase):
                               template={u'Foo': u'bar'},
                               params={u'Egg': u'spam'},
                               files=None,
-                              show_nested=False)
+                              environment_files=['foo.yaml'],
+                              ignorable_errors=None,
+                              show_nested=False,
+                              version='1.24')
 
     def test_list_resource_types(self):
         self._test_engine_api('list_resource_types',
@@ -245,7 +253,9 @@ class EngineRpcAPITestCase(common.HeatTestCase):
         self._test_engine_api('list_stack_resources', 'call',
                               stack_identity=self.identity,
                               nested_depth=0,
-                              with_detail=False)
+                              with_detail=False,
+                              filters=None,
+                              version=1.25)
 
     def test_stack_suspend(self):
         self._test_engine_api('stack_suspend', 'call',
@@ -323,6 +333,7 @@ class EngineRpcAPITestCase(common.HeatTestCase):
             'create_software_deployment', 'call',
             server_id='9f1f0e00-05d2-4ca5-8602-95021f19c9d0',
             config_id='48e8ade1-9196-42d5-89a2-f709fde42632',
+            deployment_id='86729f02-4648-44d8-af44-d0ec65b6abc9',
             stack_user_project_id='65728b74-cfe7-4f17-9c15-11d4f686e591',
             input_values={},
             action='INIT',
@@ -365,3 +376,28 @@ class EngineRpcAPITestCase(common.HeatTestCase):
 
     def test_list_services(self):
         self._test_engine_api('list_services', 'call', version='1.4')
+
+    def test_stack_list_outputs(self):
+        self._test_engine_api(
+            'list_outputs', 'call', stack_identity=self.identity,
+            version='1.19'
+        )
+
+    def test_stack_show_output(self):
+        self._test_engine_api(
+            'show_output', 'call', stack_identity=self.identity,
+            output_key='test', version='1.19')
+
+    def test_export_stack(self):
+        self._test_engine_api('export_stack',
+                              'call',
+                              stack_identity=self.identity,
+                              version='1.22')
+
+    def test_resource_mark_unhealthy(self):
+        self._test_engine_api('resource_mark_unhealthy', 'call',
+                              stack_identity=self.identity,
+                              resource_name='LogicalResourceId',
+                              mark_unhealthy=True,
+                              resource_status_reason="Any reason",
+                              version='1.26')
