@@ -189,7 +189,7 @@ class StackUpdateTest(common.HeatTestCase):
         self.stack.update(updated_stack)
         self.assertEqual((stack.Stack.UPDATE, stack.Stack.COMPLETE),
                          self.stack.state)
-        self.assertEqual(True, self.stack.disable_rollback)
+        self.assertTrue(self.stack.disable_rollback)
 
     def test_update_tags(self):
         tmpl = {'HeatTemplateFormatVersion': '2012-12-12',
@@ -232,7 +232,7 @@ class StackUpdateTest(common.HeatTestCase):
         self.stack.update(updated_stack)
         self.assertEqual((stack.Stack.UPDATE, stack.Stack.COMPLETE),
                          self.stack.state)
-        self.assertEqual(None, self.stack.tags)
+        self.assertIsNone(self.stack.tags)
 
     def test_update_modify_ok_replace(self):
         tmpl = {'HeatTemplateFormatVersion': '2012-12-12',
@@ -262,6 +262,49 @@ class StackUpdateTest(common.HeatTestCase):
         stored_props = loaded_stack['AResource']._stored_properties_data
         self.assertEqual({'Foo': 'xyz'}, stored_props)
 
+    def test_update_replace_resticted(self):
+        env = environment.Environment()
+        env_snippet = {u'resource_registry': {
+            u'resources': {
+                'AResource': {'restricted_actions': 'update'}
+            }
+        }
+        }
+        env.load(env_snippet)
+        tmpl = {'HeatTemplateFormatVersion': '2012-12-12',
+                'Resources': {'AResource': {'Type': 'ResourceWithPropsType',
+                                            'Properties': {'Foo': 'abc'}}}}
+
+        self.stack = stack.Stack(self.ctx, 'update_test_stack',
+                                 template.Template(tmpl))
+        self.stack.store()
+        self.stack.create()
+        self.assertEqual((stack.Stack.CREATE, stack.Stack.COMPLETE),
+                         self.stack.state)
+
+        tmpl1 = {'HeatTemplateFormatVersion': '2012-12-12',
+                 'Resources': {'AResource': {'Type': 'ResourceWithPropsType',
+                                             'Properties': {'Foo': 'xyz'}}}}
+
+        updated_stack = stack.Stack(self.ctx, 'updated_stack',
+                                    template.Template(tmpl1, env=env))
+
+        self.stack.update(updated_stack)
+        self.assertEqual((stack.Stack.UPDATE, stack.Stack.COMPLETE),
+                         self.stack.state)
+
+        tmpl2 = {'HeatTemplateFormatVersion': '2012-12-12',
+                 'Resources': {'AResource': {'Type': 'GenericResourceType'}}}
+        env_snippet['resource_registry']['resources'][
+            'AResource']['restricted_actions'] = 'replace'
+        env.load(env_snippet)
+        updated_stack = stack.Stack(self.ctx, 'updated_stack',
+                                    template.Template(tmpl2, env=env))
+
+        self.stack.update(updated_stack)
+        self.assertEqual((stack.Stack.UPDATE, stack.Stack.FAILED),
+                         self.stack.state)
+
     def test_update_modify_ok_replace_int(self):
         # create
         # ========
@@ -368,6 +411,113 @@ class StackUpdateTest(common.HeatTestCase):
             {'Type': 'ResourceWithPropsType',
              'Properties': {'Foo': 'abc'}})
 
+    def test_update_replace_create_hook(self):
+        tmpl = {
+            'HeatTemplateFormatVersion': '2012-12-12',
+            'Parameters': {
+                'foo': {'Type': 'String'}
+            },
+            'Resources': {
+                'AResource': {
+                    'Type': 'ResourceWithPropsType',
+                    'Properties': {'Foo': {'Ref': 'foo'}}
+                }
+            }
+        }
+
+        self.stack = stack.Stack(
+            self.ctx, 'update_test_stack',
+            template.Template(
+                tmpl, env=environment.Environment({'foo': 'abc'})))
+        self.stack.store()
+        self.stack.create()
+        self.assertEqual((stack.Stack.CREATE, stack.Stack.COMPLETE),
+                         self.stack.state)
+
+        env2 = environment.Environment({'foo': 'xyz'})
+        # Add a create hook on the resource
+        env2.registry.load(
+            {'resources': {'AResource': {'hooks': 'pre-create'}}})
+        updated_stack = stack.Stack(self.ctx, 'updated_stack',
+                                    template.Template(tmpl, env=env2))
+
+        self.stack.update(updated_stack)
+        # The hook is not called, and update succeeds properly
+        self.assertEqual((stack.Stack.UPDATE, stack.Stack.COMPLETE),
+                         self.stack.state)
+        self.assertEqual('xyz', self.stack['AResource'].properties['Foo'])
+
+    def test_update_replace_delete_hook(self):
+        tmpl = {
+            'HeatTemplateFormatVersion': '2012-12-12',
+            'Parameters': {
+                'foo': {'Type': 'String'}
+            },
+            'Resources': {
+                'AResource': {
+                    'Type': 'ResourceWithPropsType',
+                    'Properties': {'Foo': {'Ref': 'foo'}}
+                }
+            }
+        }
+
+        env = environment.Environment({'foo': 'abc'})
+        env.registry.load(
+            {'resources': {'AResource': {'hooks': 'pre-delete'}}})
+        self.stack = stack.Stack(
+            self.ctx, 'update_test_stack',
+            template.Template(tmpl, env=env))
+        self.stack.store()
+        self.stack.create()
+        self.assertEqual((stack.Stack.CREATE, stack.Stack.COMPLETE),
+                         self.stack.state)
+
+        env2 = environment.Environment({'foo': 'xyz'})
+        env2.registry.load(
+            {'resources': {'AResource': {'hooks': 'pre-delete'}}})
+        updated_stack = stack.Stack(self.ctx, 'updated_stack',
+                                    template.Template(tmpl, env=env2))
+
+        self.stack.update(updated_stack)
+        # The hook is not called, and update succeeds properly
+        self.assertEqual((stack.Stack.UPDATE, stack.Stack.COMPLETE),
+                         self.stack.state)
+        self.assertEqual('xyz', self.stack['AResource'].properties['Foo'])
+
+    def test_update_replace_post_hook(self):
+        tmpl = {
+            'HeatTemplateFormatVersion': '2012-12-12',
+            'Parameters': {
+                'foo': {'Type': 'String'}
+            },
+            'Resources': {
+                'AResource': {
+                    'Type': 'ResWithComplexPropsAndAttrs',
+                    'Properties': {'an_int': {'Ref': 'foo'}}
+                }
+            }
+        }
+        self.stack = stack.Stack(
+            self.ctx, 'update_test_stack',
+            template.Template(tmpl, env=environment.Environment({'foo': 1})))
+        self.stack.store()
+        self.stack.create()
+        self.assertEqual((stack.Stack.CREATE, stack.Stack.COMPLETE),
+                         self.stack.state)
+
+        env2 = environment.Environment({'foo': 2})
+        env2.registry.load(
+            {'resources': {'AResource': {'hooks': 'post-update'}}})
+        updated_stack = stack.Stack(self.ctx, 'updated_stack',
+                                    template.Template(tmpl, env=env2))
+        mock_hook = self.patchobject(self.stack['AResource'], 'trigger_hook')
+
+        self.stack.update(updated_stack)
+        mock_hook.assert_called_once_with('post-update')
+        self.assertEqual((stack.Stack.UPDATE, stack.Stack.COMPLETE),
+                         self.stack.state)
+        self.assertEqual(2, self.stack['AResource'].properties['an_int'])
+
     def test_update_modify_update_failed(self):
         tmpl = {'HeatTemplateFormatVersion': '2012-12-12',
                 'Resources': {'AResource': {'Type': 'ResourceWithPropsType',
@@ -1068,11 +1218,11 @@ class StackUpdateTest(common.HeatTestCase):
         self.m.UnsetStubs()
 
     def test_update_replace_by_reference(self):
-        '''
-        assertion:
-        changes in dynamic attributes, due to other resources been updated
+        """Test case for changes in dynamic attributes.
+
+        Changes in dynamic attributes, due to other resources been updated
         are not ignored and can cause dependent resources to be updated.
-        '''
+        """
         tmpl = {'HeatTemplateFormatVersion': '2012-12-12',
                 'Resources': {
                     'AResource': {'Type': 'ResourceWithPropsType',
@@ -1123,11 +1273,11 @@ class StackUpdateTest(common.HeatTestCase):
         mock_id.assert_called_with()
 
     def test_update_with_new_resources_with_reference(self):
-        '''
-        assertion:
-        check, that during update with new resources which one has
+        """Check correct resolving of references in new resources.
+
+        Check, that during update with new resources which one has
         reference on second, reference will be correct resolved.
-        '''
+        """
         tmpl = {'HeatTemplateFormatVersion': '2012-12-12',
                 'Resources': {
                     'CResource': {'Type': 'ResourceWithPropsType',
@@ -1169,11 +1319,10 @@ class StackUpdateTest(common.HeatTestCase):
         mock_create.assert_called_with()
 
     def test_update_by_reference_and_rollback_1(self):
-        '''
-        assertion:
-        check that rollback still works with dynamic metadata
-        this test fails the first instance
-        '''
+        """Check that rollback still works with dynamic metadata.
+
+        This test fails the first instance.
+        """
         tmpl = {'HeatTemplateFormatVersion': '2012-12-12',
                 'Resources': {
                     'AResource': {'Type': 'ResourceWithPropsType',
@@ -1220,11 +1369,10 @@ class StackUpdateTest(common.HeatTestCase):
         mock_create.assert_called_once_with()
 
     def test_update_by_reference_and_rollback_2(self):
-        '''
-        assertion:
-        check that rollback still works with dynamic metadata
-        this test fails the second instance
-        '''
+        """Check that rollback still works with dynamic metadata.
+
+        This test fails the second instance.
+        """
 
         class ResourceTypeA(generic_rsrc.ResourceWithProps):
             count = 0
@@ -1279,11 +1427,10 @@ class StackUpdateTest(common.HeatTestCase):
         mock_create.assert_called_once_with()
 
     def test_update_failure_recovery(self):
-        '''
-        assertion:
-        check that rollback still works with dynamic metadata
-        this test fails the second instance
-        '''
+        """Check that rollback still works with dynamic metadata.
+
+        This test fails the second instance.
+        """
 
         class ResourceTypeA(generic_rsrc.ResourceWithProps):
             count = 0
@@ -1363,11 +1510,10 @@ class StackUpdateTest(common.HeatTestCase):
         mock_delete_A.assert_called_once_with()
 
     def test_update_failure_recovery_new_param(self):
-        '''
-        assertion:
-        check that rollback still works with dynamic metadata
-        this test fails the second instance
-        '''
+        """Check that rollback still works with dynamic metadata.
+
+        This test fails the second instance.
+        """
 
         class ResourceTypeA(generic_rsrc.ResourceWithProps):
             count = 0
@@ -1458,11 +1604,10 @@ class StackUpdateTest(common.HeatTestCase):
         self.assertEqual(2, mock_create.call_count)
 
     def test_update_failure_recovery_new_param_stack_list(self):
-        '''
-        assertion:
-        check that stack-list is not broken if update fails in between.
-        Also ensure that next update passes
-        '''
+        """Check that stack-list is not broken if update fails in between.
+
+        Also ensure that next update passes.
+        """
 
         class ResourceTypeA(generic_rsrc.ResourceWithProps):
             count = 0
@@ -1559,11 +1704,11 @@ class StackUpdateTest(common.HeatTestCase):
         self.assertEqual(2, mock_create.call_count)
 
     def test_update_replace_parameters(self):
-        '''
-        assertion:
-        changes in static environment parameters
-        are not ignored and can cause dependent resources to be updated.
-        '''
+        """Check that changes in static environment parameters are not ignored.
+
+        Changes in static environment parameters are not ignored and can cause
+        dependent resources to be updated.
+        """
         tmpl = {'HeatTemplateFormatVersion': '2012-12-12',
                 'Parameters': {'AParam': {'Type': 'String'}},
                 'Resources': {
