@@ -12,22 +12,22 @@
 
 import os
 
-import ceilometerclient.client
-import cinderclient.client
-import heatclient.client
+from ceilometerclient import client as ceilometer_client
+from cinderclient import client as cinder_client
+from heat.common.i18n import _
+from heatclient import client as heat_client
 from keystoneclient.auth.identity.generic import password
-import keystoneclient.client
-import keystoneclient.exceptions
+from keystoneclient import exceptions as kc_exceptions
 from keystoneclient import session
-import neutronclient.v2_0.client
-import novaclient.client
-import swiftclient
+from neutronclient.v2_0 import client as neutron_client
+from novaclient import client as nova_client
+from swiftclient import client as swift_client
 
 
 class KeystoneWrapperClient(object):
     """Wrapper object for keystone client
 
-    This Wraps keystone client,so we can encpasulate certain
+    This wraps keystone client, so we can encpasulate certain
     added properties like auth_token, project_id etc.
     """
     def __init__(self, auth_plugin, verify=True):
@@ -59,22 +59,28 @@ class KeystoneWrapperClient(object):
 
 
 class ClientManager(object):
-    """
+    """Provides access to the official python clients for calling various APIs.
+
     Manager that provides access to the official python clients for
     calling various OpenStack APIs.
     """
 
-    CINDERCLIENT_VERSION = '1'
+    CINDERCLIENT_VERSION = '2'
     HEATCLIENT_VERSION = '1'
     NOVACLIENT_VERSION = '2'
     CEILOMETER_VERSION = '2'
 
     def __init__(self, conf):
         self.conf = conf
+        if self.conf.auth_url.find('/v'):
+            self.v2_auth_url = self.conf.auth_url.replace('/v3', '/v2.0')
+            self.auth_version = self.conf.auth_url.split('/v')[1]
+        else:
+            raise ValueError(_('Incorrectly specified auth_url config: no '
+                               'version found.'))
+
         self.insecure = self.conf.disable_ssl_certificate_validation
         self.ca_file = self.conf.ca_file
-        self.v2_auth_url = self.conf.auth_url.replace('/v3', '/v2.0')
-        self.auth_version = self.conf.auth_url.split('/v')[1]
         self.identity_client = self._get_identity_client()
         self.orchestration_client = self._get_orchestration_client()
         self.compute_client = self._get_compute_client()
@@ -93,10 +99,10 @@ class ClientManager(object):
             if endpoint is None:
                 endpoint = self.identity_client.get_endpoint_url(
                     'orchestration', self.conf.region)
-        except keystoneclient.exceptions.EndpointNotFound:
+        except kc_exceptions.EndpointNotFound:
             return None
         else:
-            return heatclient.client.Client(
+            return heat_client.Client(
                 self.HEATCLIENT_VERSION,
                 endpoint,
                 token=token,
@@ -104,8 +110,7 @@ class ClientManager(object):
                 password=self.conf.password)
 
     def _get_identity_client(self):
-        user_domain_name = self.conf.user_domain_name
-        project_domain_name = self.conf.project_domain_name
+        domain = self.conf.domain_name
         kwargs = {
             'username': self.conf.username,
             'password': self.conf.password,
@@ -115,14 +120,15 @@ class ClientManager(object):
         # keystone v2 can't ignore domain details
         if self.auth_version == '3':
             kwargs.update({
-                'user_domain_name': user_domain_name,
-                'project_domain_name': project_domain_name})
+                'project_domain_name': domain,
+                'user_domain_name': domain})
         auth = password.Password(**kwargs)
         if self.insecure:
-            verify = False
+            verify_cert = False
         else:
-            verify = self.ca_file or True
-        return KeystoneWrapperClient(auth, verify=verify)
+            verify_cert = self.ca_file or True
+
+        return KeystoneWrapperClient(auth, verify_cert)
 
     def _get_compute_client(self):
 
@@ -137,7 +143,7 @@ class ClientManager(object):
         )
 
         # Create our default Nova client to use in testing
-        return novaclient.client.Client(
+        return nova_client.Client(
             self.NOVACLIENT_VERSION,
             *client_args,
             service_type='compute',
@@ -150,7 +156,7 @@ class ClientManager(object):
 
     def _get_network_client(self):
 
-        return neutronclient.v2_0.client.Client(
+        return neutron_client.Client(
             username=self.conf.username,
             password=self.conf.password,
             tenant_name=self.conf.tenant_name,
@@ -163,7 +169,7 @@ class ClientManager(object):
     def _get_volume_client(self):
         region = self.conf.region
         endpoint_type = 'publicURL'
-        return cinderclient.client.Client(
+        return cinder_client.Client(
             self.CINDERCLIENT_VERSION,
             self.conf.username,
             self.conf.password,
@@ -187,15 +193,14 @@ class ClientManager(object):
             'insecure': self.insecure,
             'cacert': self.ca_file,
         }
-        return swiftclient.client.Connection(**args)
+        return swift_client.Connection(**args)
 
     def _get_metering_client(self):
-        user_domain_name = self.conf.user_domain_name
-        project_domain_name = self.conf.project_domain_name
+        domain = self.conf.domain_name
         try:
             endpoint = self.identity_client.get_endpoint_url('metering',
                                                              self.conf.region)
-        except keystoneclient.exceptions.EndpointNotFound:
+        except kc_exceptions.EndpointNotFound:
             return None
         else:
             args = {
@@ -213,8 +218,8 @@ class ClientManager(object):
             # v2 auth_url
             if self.auth_version == '3':
                 args.update(
-                    {'user_domain_name': user_domain_name,
-                     'project_domain_name': project_domain_name})
+                    {'user_domain_name': domain,
+                     'project_domain_name': domain})
 
-            return ceilometerclient.client.Client(self.CEILOMETER_VERSION,
-                                                  endpoint, **args)
+            return ceilometer_client.Client(self.CEILOMETER_VERSION,
+                                            endpoint, **args)
