@@ -189,7 +189,9 @@ class AutoscalingGroupBasicTest(AutoscalingGroupTest):
         self.assert_instance_count(stack, 5)
 
     def test_update_group_replace(self):
-        """Make sure that during a group update the non updatable
+        """Test case for ensuring non-updatable props case a replacement.
+
+        Make sure that during a group update the non-updatable
         properties cause a replacement.
         """
         files = {'provider.yaml': self.instance_template}
@@ -219,7 +221,9 @@ class AutoscalingGroupBasicTest(AutoscalingGroupTest):
         self.assertNotEqual(orig_asg_id, rsrc.physical_resource_id)
 
     def test_create_instance_error_causes_group_error(self):
-        """If a resource in an instance group fails to be created, the instance
+        """Test create failing a resource in the instance group.
+
+        If a resource in an instance group fails to be created, the instance
         group itself will fail and the broken inner resource will remain.
         """
         stack_name = self._stack_rand_name()
@@ -252,7 +256,9 @@ class AutoscalingGroupBasicTest(AutoscalingGroupTest):
         self._assert_instance_state(nested_ident, 0, 2)
 
     def test_update_instance_error_causes_group_error(self):
-        """If a resource in an instance group fails to be created during an
+        """Test update failing a resource in the instance group.
+
+        If a resource in an instance group fails to be created during an
         update, the instance group itself will fail and the broken inner
         resource will remain.
         """
@@ -340,8 +346,7 @@ class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
     def update_instance_group(self, updt_template,
                               num_updates_expected_on_updt,
                               num_creates_expected_on_updt,
-                              num_deletes_expected_on_updt,
-                              update_replace):
+                              num_deletes_expected_on_updt):
 
         # setup stack from the initial template
         files = {'provider.yaml': self.instance_template}
@@ -408,9 +413,10 @@ class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
             self.assertNotIn(deletes_expected, updt_names)
 
     def test_instance_group_update_replace(self):
-        """
-        Test simple update replace with no conflict in batch size and
-        minimum instances in service.
+        """Test simple update replace.
+
+        Test update replace with no conflict in batch size and minimum
+        instances in service.
         """
         updt_template = self.ig_tmpl_with_updt_policy()
         grp = updt_template['Resources']['JobServerGroup']
@@ -423,13 +429,13 @@ class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
         self.update_instance_group(updt_template,
                                    num_updates_expected_on_updt=10,
                                    num_creates_expected_on_updt=0,
-                                   num_deletes_expected_on_updt=0,
-                                   update_replace=True)
+                                   num_deletes_expected_on_updt=0)
 
     def test_instance_group_update_replace_with_adjusted_capacity(self):
-        """
-        Test update replace with capacity adjustment due to conflict in
-        batch size and minimum instances in service.
+        """Test update replace with capacity adjustment.
+
+        Test update replace with capacity adjustment due to conflict in batch
+        size and minimum instances in service.
         """
         updt_template = self.ig_tmpl_with_updt_policy()
         grp = updt_template['Resources']['JobServerGroup']
@@ -442,13 +448,10 @@ class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
         self.update_instance_group(updt_template,
                                    num_updates_expected_on_updt=8,
                                    num_creates_expected_on_updt=2,
-                                   num_deletes_expected_on_updt=2,
-                                   update_replace=True)
+                                   num_deletes_expected_on_updt=2)
 
     def test_instance_group_update_replace_huge_batch_size(self):
-        """
-        Test update replace with a huge batch size.
-        """
+        """Test update replace with a huge batch size."""
         updt_template = self.ig_tmpl_with_updt_policy()
         group = updt_template['Resources']['JobServerGroup']
         policy = group['UpdatePolicy']['AutoScalingRollingUpdate']
@@ -460,13 +463,10 @@ class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
         self.update_instance_group(updt_template,
                                    num_updates_expected_on_updt=10,
                                    num_creates_expected_on_updt=0,
-                                   num_deletes_expected_on_updt=0,
-                                   update_replace=True)
+                                   num_deletes_expected_on_updt=0)
 
     def test_instance_group_update_replace_huge_min_in_service(self):
-        """
-        Test update replace with a huge number of minimum instances in service.
-        """
+        """Update replace with huge number of minimum instances in service."""
         updt_template = self.ig_tmpl_with_updt_policy()
         group = updt_template['Resources']['JobServerGroup']
         policy = group['UpdatePolicy']['AutoScalingRollingUpdate']
@@ -479,11 +479,11 @@ class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
         self.update_instance_group(updt_template,
                                    num_updates_expected_on_updt=9,
                                    num_creates_expected_on_updt=1,
-                                   num_deletes_expected_on_updt=1,
-                                   update_replace=True)
+                                   num_deletes_expected_on_updt=1)
 
     def test_instance_group_update_no_replace(self):
-        """
+        """Test simple update only and no replace.
+
         Test simple update only and no replace (i.e. updated instance flavor
         in Launch Configuration) with no conflict in batch size and
         minimum instances in service.
@@ -500,11 +500,11 @@ class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
         self.update_instance_group(updt_template,
                                    num_updates_expected_on_updt=10,
                                    num_creates_expected_on_updt=0,
-                                   num_deletes_expected_on_updt=0,
-                                   update_replace=False)
+                                   num_deletes_expected_on_updt=0)
 
     def test_instance_group_update_no_replace_with_adjusted_capacity(self):
-        """
+        """Test update only and no replace with capacity adjustment.
+
         Test update only and no replace (i.e. updated instance flavor in
         Launch Configuration) with capacity adjustment due to conflict in
         batch size and minimum instances in service.
@@ -521,8 +521,7 @@ class AutoscalingGroupUpdatePolicyTest(AutoscalingGroupTest):
         self.update_instance_group(updt_template,
                                    num_updates_expected_on_updt=8,
                                    num_creates_expected_on_updt=2,
-                                   num_deletes_expected_on_updt=2,
-                                   update_replace=False)
+                                   num_deletes_expected_on_updt=2)
 
 
 class AutoScalingSignalTest(AutoscalingGroupTest):
@@ -632,16 +631,16 @@ outputs:
         md = self.client.resources.metadata(stack_identifier, 'custom_lb')
         actual_md = len(md['IPs'].split(','))
         if actual_md != expected:
-            LOG.warn('check_instance_count exp:%d, meta:%s' % (expected,
-                                                               md['IPs']))
+            LOG.warning('check_instance_count exp:%d, meta:%s' % (expected,
+                                                                  md['IPs']))
             return False
 
         stack = self.client.stacks.get(stack_identifier)
         inst_list = self._stack_output(stack, 'InstanceList')
         actual = len(inst_list.split(','))
         if actual != expected:
-            LOG.warn('check_instance_count exp:%d, act:%s' % (expected,
-                                                              inst_list))
+            LOG.warning('check_instance_count exp:%d, act:%s' % (expected,
+                                                                 inst_list))
         return actual == expected
 
     def test_scaling_meta_update(self):
@@ -730,7 +729,7 @@ outputs:
         self._wait_for_resource_status(
             stack_identifier, 'JobServerGroup', 'SUSPEND_COMPLETE')
 
-        # Send a signal and a exception will raise
+        # Send a signal and an exception will raise
         ex = self.assertRaises(exc.BadRequest,
                                self.client.resources.signal,
                                stack_identifier, 'ScaleUpPolicy')
