@@ -131,6 +131,7 @@ class InstanceGroupBasicTest(InstanceGroupTest):
 
     def test_basic_create_works(self):
         """Make sure the working case is good.
+
         Note this combines test_override_aws_ec2_instance into this test as
         well, which is:
         If AWS::EC2::Instance is overridden, InstanceGroup will automatically
@@ -177,8 +178,10 @@ class InstanceGroupBasicTest(InstanceGroupTest):
         self.assert_instance_count(stack, 5)
 
     def test_update_group_replace(self):
-        """Make sure that during a group update the non updatable
-        properties cause a replacement.
+        """Test case for ensuring non-updatable props case a replacement.
+
+        Make sure that during a group update the non-updatable properties cause
+        a replacement.
         """
         files = {'provider.yaml': self.instance_template}
         env = {'resource_registry':
@@ -207,7 +210,9 @@ class InstanceGroupBasicTest(InstanceGroupTest):
         self.assertNotEqual(orig_asg_id, rsrc.physical_resource_id)
 
     def test_create_instance_error_causes_group_error(self):
-        """If a resource in an instance group fails to be created, the instance
+        """Test create failing a resource in the instance group.
+
+        If a resource in an instance group fails to be created, the instance
         group itself will fail and the broken inner resource will remain.
         """
         stack_name = self._stack_rand_name()
@@ -240,7 +245,9 @@ class InstanceGroupBasicTest(InstanceGroupTest):
         self._assert_instance_state(nested_ident, 0, 2)
 
     def test_update_instance_error_causes_group_error(self):
-        """If a resource in an instance group fails to be created during an
+        """Test update failing a resource in the instance group.
+
+        If a resource in an instance group fails to be created during an
         update, the instance group itself will fail and the broken inner
         resource will remain.
         """
@@ -376,7 +383,8 @@ class InstanceGroupUpdatePolicyTest(InstanceGroupTest):
             self.assertNotIn(deletes_expected, updt_names)
 
     def test_instance_group_update_replace(self):
-        """
+        """Test simple update replace with no conflict.
+
         Test simple update replace with no conflict in batch size and
         minimum instances in service.
         """
@@ -395,7 +403,8 @@ class InstanceGroupUpdatePolicyTest(InstanceGroupTest):
                                    update_replace=True)
 
     def test_instance_group_update_replace_with_adjusted_capacity(self):
-        """
+        """Test update replace with capacity adjustment.
+
         Test update replace with capacity adjustment due to conflict in
         batch size and minimum instances in service.
         """
@@ -414,9 +423,7 @@ class InstanceGroupUpdatePolicyTest(InstanceGroupTest):
                                    update_replace=True)
 
     def test_instance_group_update_replace_huge_batch_size(self):
-        """
-        Test update replace with a huge batch size.
-        """
+        """Test update replace with a huge batch size."""
         updt_template = self.ig_tmpl_with_updt_policy()
         group = updt_template['Resources']['JobServerGroup']
         policy = group['UpdatePolicy']['RollingUpdate']
@@ -432,9 +439,7 @@ class InstanceGroupUpdatePolicyTest(InstanceGroupTest):
                                    update_replace=True)
 
     def test_instance_group_update_replace_huge_min_in_service(self):
-        """
-        Test update replace with a huge number of minimum instances in service.
-        """
+        """Update replace with huge number of minimum instances in service."""
         updt_template = self.ig_tmpl_with_updt_policy()
         group = updt_template['Resources']['JobServerGroup']
         policy = group['UpdatePolicy']['RollingUpdate']
@@ -451,7 +456,8 @@ class InstanceGroupUpdatePolicyTest(InstanceGroupTest):
                                    update_replace=True)
 
     def test_instance_group_update_no_replace(self):
-        """
+        """Test simple update only and no replace with no conflict.
+
         Test simple update only and no replace (i.e. updated instance flavor
         in Launch Configuration) with no conflict in batch size and
         minimum instances in service.
@@ -472,7 +478,8 @@ class InstanceGroupUpdatePolicyTest(InstanceGroupTest):
                                    update_replace=False)
 
     def test_instance_group_update_no_replace_with_adjusted_capacity(self):
-        """
+        """Test update only and no replace with capacity adjustment.
+
         Test update only and no replace (i.e. updated instance flavor in
         Launch Configuration) with capacity adjustment due to conflict in
         batch size and minimum instances in service.
