@@ -10,15 +10,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_utils import reflection
+
 from heat_integrationtests.common import test
 
 
 class ScenarioTestsBase(test.HeatIntegrationTest):
-    "This class define common parameters for scenario tests"
+    """This class defines common parameters for scenario tests."""
 
     def setUp(self):
         super(ScenarioTestsBase, self).setUp()
-        self.check_skip_test()
+        self.check_skip()
 
         self.client = self.orchestration_client
         self.sub_dir = 'templates'
@@ -58,8 +60,8 @@ class ScenarioTestsBase(test.HeatIntegrationTest):
 
         return stack_id
 
-    def check_skip_test(self):
-        test_cls_name = self.__class__.__name__
+    def check_skip(self):
+        test_cls_name = reflection.get_class_name(self, fully_qualified=False)
         test_method_name = '.'.join([test_cls_name, self._testMethodName])
         test_skipped = (self.conf.skip_scenario_test_list and (
             test_cls_name in self.conf.skip_scenario_test_list or
